# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmCategoriesController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false


  protect_from_forgery with: :exception
  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_roles
  before_action :get_vm_version  
  before_action :clear_import_session, only: [ :create, :modify, :delete, :export, :read_import_file ]
  before_action :clear_export_session, only: [ :index, :create, :modify, :delete ]
  before_action -> { get_summary_table_lines('categories') }

  # 権限チェック
  def check_roles
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    if [ 'create', 'modify', 'delete' ].include?(action_name) and !@sidfm_roles.include?(:admin) then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
  end

  def is_file_output_executable?
    @sidfm_roles.include?(:admin)
  end 

  def get_sort_value(action_idx)
    [ 'new', 'modify', 'delete' ].index(action_idx)
  end

  def form_params
  end

  def get_openid_connect_category_id
    category_id = -1
    #sidfm_categoriesは今のところ1つのみ設定可能になっている
    category_ids = Sidfm.openid_connect_settings_user_category_ids
    category_id = category_ids.split(',').first.to_i if category_ids.length > 0
    category_id
  end
 
  def openid_connect_enable?
    settings = (Setting.send('plugin_redmine_openid_connect')).symbolize_keys
    ret = (settings[:enabled] ? true : false) #enabledがある時は、文字列のfalseになっている
  end

  def process_related_rules(**opts)
    r = true
    param = {
      :role_admin => @sidfm_roles.include?(:admin),
      :user_category_ids => (@sidfm_roles.include?(:admin) ? nil : User.current.categories.pluck(:id)),
      :action => opts[:action],
      :messages_necessary => false
    }
    case opts[:action]
    when 'delete' then
      param[:delete_category_ids] = opts[:delete_category_ids]
    when 'import' then
      param[:delete_category_ids] = opts[:delete_category_ids]
    else
      #invalid
    end
    ret = SidfmdRule.categories_related_rule_info(param)
    SidfmdRuleCmCategorySub.where(:id => ret[:delete_sub_category_cond_ids]).destroy_all if ret[:delete_sub_category_cond_ids].any?
    SidfmdRuleCmHostSub.where(:id => ret[:delete_sub_host_cond_ids]).destroy_all if ret[:delete_sub_host_cond_ids].any?
    SidfmdRuleChpUserSub.where(:id => ret[:delete_chp_user_sub_ids]).destroy_all if ret[:delete_chp_user_sub_ids].any?
    SidfmdRuleCmCtag.where(:id => ret[:delete_cm_ctag_ids]).destroy_all if ret[:delete_cm_ctag_ids].any?
    SidfmdRuleAmCtag.where(:id => ret[:delete_am_ctag_ids]).destroy_all if ret[:delete_am_ctag_ids].any?
    SidfmdRuleCond.where(:id => ret[:delete_cond_ids]).destroy_all if ret[:delete_cond_ids].any?
    SidfmdRuleAction.where(:id => ret[:delete_action_ids]).destroy_all if ret[:delete_action_ids].any?
    SidfmdRule.where(:id => ret[:delete_rule_ids]).destroy_all if ret[:delete_rule_ids].any?
    r
  end

  def get_max_hostlim
    hosts = SidfmdHost.get_hosts_by_lm(:'sort[hostname]' => :asc)
    num = hosts ? hosts[:license_number] : nil
  end

  def hostlim_over?(hostlim, max_host_num)
    ret = false
    if hostlim && max_host_num then
      if max_host_num then
        ret = hostlim.to_i > max_host_num.to_i
      end
    end
    ret
  end
  
  def create_summary_category_html(category, **opts)
    data = {
      :chkbox => view_context.create_summary_category_chkbox(category.id, opts[:checked], category.is_system),
      :name => view_context.create_summary_category_name(category.name),
      :description => view_context.create_summary_category_description(category.desc),
      :hostlim => view_context.create_summary_category_hostlim(category.is_system, category.hostlim),
      :class => view_context.create_summary_category_class(category.is_system),
      :operation => view_context.create_summary_category_operation(
        category,
        opts[:is_openid_connect_enable],
        opts[:openid_connect_category_id],
        opts[:openid_connect_auth_name],
        @sidfm_roles
      )
    }
  end

  def parse_params
    filtered_params = params.fetch(:pages, {}).permit(:offset, :per_page, :checked, :all_checked)
    @categories = {}
    @categories[:per_page] = (@summary_table_lines || filtered_params[:per_page] || 10).to_i
    @categories[:offset] = (filtered_params[:offset] || 0).to_i
    @categories[:checked] = JSON.parse(filtered_params[:checked] || '[]')
    @categories[:all_checked] = filtered_params[:all_checked] || ''
    @categories[:order] = 'name_asc' #現在は不使用 jsのpages_formのために設定
    @categories[:filters] = {} #現在は不使用 jsのpages_formのために設定
  end

  public

  def index
    filtered_params = params.fetch(:pages, {}).permit(:clear_session)
    if filtered_params[:clear_session] then
      clear_import_session
      redirect_to :action => :index
    end
    parse_params
    @category_names = SidfmdCategory.all.select(:name).distinct(:name).collect {|x| x.name }
=begin
    @categories[:content] = SidfmdCategory.order(
      "CASE WHEN is_system IS TRUE THEN id ELSE -1 END"
    ).order('name COLLATE "C"')
    if !@sidfm_roles.include?(:admin) then
      @categories[:content] = @categories[:content].where(:id => SidfmdCategoriesUser.where(:user_id => User.current.id).select(:sidfmd_category_id).distinct)
    end
    @all_c_cat_ids = @categories[:content].where(:is_system => false).pluck(:id)
    @categories[:content] = @categories[:content].offset(@categories[:offset]).limit(@categories[:per_page])
    @categories[:total_size] = @categories[:content].except(:order, :offset, :limit).count
    @users = User.where(:admin => false).where.not(:type => 'AnonymousUser').order(:lastname, :firstname)
  
    @import_notices = session[:import_notices]
    clear_import_session
    @is_openid_connect_enable = openid_connect_enable?
    @openid_connect_category_id = get_openid_connect_category_id
    @openid_connect_auth_name = OPENID_CONNECT_AUTH_NAME
=end
    @import_notices = session[:import_notices]
    clear_import_session
    @max_hostlim = get_max_hostlim
  end

  def create
    filtered_params = params.permit(:func, :name, :desc, :hostlim)
    p_name = filtered_params['name']
    if p_name.nil? or p_name.size == 0 then
      render :layout => false, :json => { :err_message => 'カテゴリ名が入力されていません。' }
      return
    end
    p_name = p_name.strip
    if SidfmdCategory.where(:name => p_name).count > 0 then
      render :layout => false, :json => { :err_message => '同名のカテゴリが存在します。' }
      return
    end
    p_desc = filtered_params['desc']
    if p_desc.nil? or p_desc.size == 0 then
      render :layout => false, :json => { :err_message => '概要が入力されていません。' } and return
    end
    if p_name.size > 250 then
      render :layout => false, :json => { :err_message => 'カテゴリ名が長すぎます。（最大250文字）' } and return
    end
    if p_desc.size > 1000 then
      render :layout => false, :json => { :err_message => '概要が長すぎます。（最大1000文字）' } and return
    end
    hostlim = filtered_params[:hostlim]
    if hostlim and hostlim.size > 0 then
      hostlim = hostlim.to_i 
      if hostlim < 0 then
        render :layout => false, :json => { :err_message => "#{SidfmdCategory::HOSTLIM_LABEL}に、負の値は設定できません。" } and return
      end
    end
    max_host_num = get_max_hostlim
    if hostlim_over?(hostlim, max_host_num) then
      render :layout => false, :json => { :err_message => "#{SidfmdCategory::HOSTLIM_LABEL}に設定可能な値は、最大#{max_host_num}(ライセンスのホスト数上限値)です。" } and return
    end
    begin
      SidfmdCategory.create(:name => p_name,
                            :desc => p_desc,
                            :hostlim => hostlim,
                            :is_system => false)
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'カテゴリの作成に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def modify
    filtered_params = params.fetch(:form, {}).permit(:id, :name, :desc, :hostlim)
    category = SidfmdCategory.find_by_id(filtered_params['id'].to_i)
    err_messages = []
    if category then
      opts = {}
      if category.is_system then
        render :layout => false, :json => { :err_message => "不正な操作です。" } and return
      end
      p_name = filtered_params['name']
      if p_name.nil? or p_name.size == 0 then
        err_messages << 'カテゴリ名が入力されていません。'
        render :layout => false, :json => { :err_message => err_messages.join("\n") } and return
      end
      p_name = p_name.strip
      if (SidfmdCategory.where(:name => p_name) - [category]).size > 0 then
        err_messages << '変更予定のカテゴリ名について、同名のカテゴリが存在します。'
      end
      if category.name != p_name then
        opts[:name] = p_name
      end
      p_desc = filtered_params['desc']
      if p_desc.nil? or p_desc.size == 0 then
        err_messages << '概要が入力されていません。'
        render :layout => false, :json => { :err_message => err_messages.join("\n") } and return
      end
      if category.desc != p_desc then
        opts[:desc] = p_desc
      end
      if p_name.size > 250 then
        err_messages << 'カテゴリ名が長すぎます。（最大250文字）'
      end
      if p_desc.size > 1000 then
        err_messages << '概要が長すぎます。（最大1000文字）'
      end
      opts[:hostlim] = filtered_params[:hostlim]
      if opts[:hostlim] and opts[:hostlim].size > 0 then
        opts[:hostlim] = opts[:hostlim].to_i
        if opts[:hostlim] < 0 then
          err_messages << "#{SidfmdCategory::HOSTLIM_LABEL}に負の値は設定できません。"
        end
      end
      max_host_num = get_max_hostlim
      if hostlim_over?(opts[:hostlim], max_host_num) then
        err_messages << "#{SidfmdCategory::HOSTLIM_LABEL}に設定可能な値は、最大#{max_host_num}(ライセンスのホスト数上限値)です。"
      end

      host_ids = []
      html_ids = []
      if err_messages.empty? then
        begin
          SidfmdCategory.transaction do
            if category.update_attributes(opts) then
              #View更新のためのid取得
              host_ids = SidfmdHostView.where(
                :id => SidfmdHvCategory.where(:sidfmd_category_id => category.id).select(:sidfmd_host_view_id)
              ).pluck(:sidfmd_host_id)
              html_ids = SidfmdHostView.relate_htmlids(host_ids).uniq
            end
          end
          Rails.cache.clear
        rescue Exception => e
          log_exception(e)
          err_messages << 'カテゴリの更新に失敗しました。'
        end
      end
    else
      err_messages << '更新対象のカテゴリが存在しません。'
    end
    render :layout => false, 
           :json => { 
             :err_message => err_messages.join("\n"),
             :host_ids => host_ids,
             :html_ids => html_ids                 
           }
  end

  def delete
    #filtered_params = params.fetch(:formdel, {}).permit(:id, :rule_info)
    filtered_params = params.permit(:id)
    category_id = filtered_params['id'].to_i
    err_message = ''
    unless category_id then
      err_message = '不正な入力です。'
      render :layout => false, :json => {:err_message => err_message} and return  
    end
    unless category_id > 0 then
      err_message = '不正な入力です。'
      render :layout => false, :json => {:err_message => err_message} and return
    end
    
    if openid_connect_enable? and category_id == get_openid_connect_category_id then
      err_message = "#{OPENID_CONNECT_AUTH_NAME}のカスタムカテゴリに指定されているため、削除できません。"
      render :layout => false, :json => {:err_message => err_message} and return
    end

    ctag_related_issue_ids = []
    begin
      SidfmdCategory.transaction do
        category = SidfmdCategory.lock.find_by(:id => category_id)
        if category and !category.is_system then
          #関連するルール・ルールの条件・操作の削除
          process_rules = process_related_rules(
            :delete_category_ids => [ category_id ],
            :action => 'delete'
          )
          if !process_rules then
            render :layout => false, :json => { :err_message => 'カテゴリの削除に失敗しました。' } and return
          end
          SidfmdCategoriesHost.where(:sidfmd_category_id => category_id).delete_all
          SidfmdCategoriesUser.where(:sidfmd_category_id => category_id).delete_all
          cat_ctags_ids = SidfmdCtag.where(:sidfmd_category_id => category_id).pluck(:id)
          ctag_related_issue_ids = SidfmdCtag.delete_ctags(cat_ctags_ids)
          cat_groups = SidfmdCatGroup.only_categories([category_id])
          if cat_groups.size > 0 then
            SidfmdCatGroup.where(:id => cat_groups.map{|grp| grp[:id]}).destroy_all
          end
          category.destroy
          if get_openid_connect_category_id == category_id then
            Sidfm.openid_connect_settings_user_category_ids = ''
          end
        else
          err_message = '不正な入力です。'
          render :layout => false, :json => {:err_message => err_message} and return
        end
      end

      #View更新のためのid取得
      host_ids = SidfmdHostView.where(
                   :id => SidfmdHvCategory.where(:sidfmd_category_id => category_id).select(:sidfmd_host_view_id)
                 ).pluck(:sidfmd_host_id)
      html_ids = SidfmdHostView.relate_htmlids(host_ids).uniq
      html_ids |= SidfmdIssue.where(:id => ctag_related_issue_ids).distinct.pluck(:html_id) if ctag_related_issue_ids.any?
    rescue Exception => e
      log_exception(e)
      err_message = 'カテゴリの削除に失敗しました。'
    end
    Rails.cache.clear
    render :layout => false, :json => { :host_ids => host_ids, :html_ids => html_ids, :err_message => err_message }
  end

  def export
    GC.start
    if !is_file_output_executable? then
      render :layout => false, :json => { :err_message => '許可されていない操作です。' } and return
    end

    f_params = params.permit(:target, :ids, :executed_size, :process)
    if !f_params[:ids] or !f_params[:target] or !f_params[:executed_size] or !f_params[:process] then
      logger.info('params error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    executed_size = f_params[:executed_size].to_i

    begin
      c_cat_ids = JSON.parse(f_params[:ids])
    rescue JSON::ParserError => e
      logger.info('json parser error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    if  ![ 'tsv' ].include?(f_params[:target]) then
      logger.info('file target error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    if f_params[:process] == 'init' then
      header = [ 'カテゴリID', 'カテゴリ名', '概要', SidfmdCategory::HOSTLIM_LABEL, '削除' ]
      c_cat_ids = if c_cat_ids.any? then
                    SidfmdCategory.where(:is_system => false, :id => c_cat_ids).order('name COLLATE "C"').pluck(:id)
                  else
                    SidfmdCategory.where(:is_system => false).order('name COLLATE "C"').pluck(:id)
                  end
      init_tsv_summary_table_progress_download(c_cat_ids, header, c_cat_ids.size)
    else
      create_tsv_data = Proc.new{|id|
        cat = SidfmdCategory.find_by(:id => id)
        item = []
        if cat then
          item << cat.id.to_s
          item << cat.name
          item << cat.desc
          item << (cat.hostlim ? cat.hostlim.to_s : '')
          item << ''
          item
        else
          item = nil
        end
        item
      }
      json_tsv_summary_table_progress_download(c_cat_ids, create_tsv_data, executed_size, 100)
    end
  end

  def download
    send_summary_table_tsv_file("vmlist-categories-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
  end

  def read_import_file
    if !is_file_output_executable? then
      render :layout => false, :json => { :err_message => '許可されていない操作です。' } and return
    end

    f_params = params.permit(:categories_file)
    cats_file = f_params[:categories_file]
    is_err = false
    r = {}
    begin 
      session[:import_notices] = []
      session[:import_actions] = []
      session[:import_check_data] = {
        :ids => {},
        :names => {},
        :dup_names => []
      }
      cats_file = cats_file.read
      if cats_file.getbyte(0) == 255 and cats_file.getbyte(1) == 254 then
        cats_file = cats_file[2, cats_file.length - 2]
      end
      cats_file = cats_file.force_encoding('UTF-16LE').encode('UTF-8')
      csv = CSV.parse(cats_file, :col_sep => "\t", :headers => :first_row) 
      cnt = 1
      import_categories = []
      csv.each do |line|
        cnt += 1
        item = {}
        item[:category_id] = line['カテゴリID'].to_s.strip
        item[:name] = line['カテゴリ名'].to_s.strip
        item[:desc] = line['概要'].to_s
        item[:hostlim] = line[SidfmdCategory::HOSTLIM_LABEL].to_s.strip
        item[:is_delete] = line['削除'].to_s.strip.downcase
        item[:line_num] = cnt
        import_categories << item
      end
      session[:import_categories] = import_categories
      r[:total_size] = import_categories.size
    rescue Exception => e
      log_exception(e)
      is_err = true
    end
    r[:err] = is_err
    render :layout => false, :json => r
  end

  def progress_import
    r = {}
    executed_size = 0
    is_err = false
    is_exception = false
    max_host_num = get_max_hostlim
    is_openid_connect_enable = openid_connect_enable?
    begin
      actions = session[:import_actions]
      check_data = session[:import_check_data].symbolize_keys
      session[:import_categories].shift(100).each do |cat|
        executed_size += 1
        cat = cat.symbolize_keys
        action = nil
        is_minor_error = false
        #入力チェック
        if cat[:name].empty? then
          session[:import_notices] << "ファイルの#{cat[:line_num]}行目: カテゴリ名を入力して下さい。"
          is_minor_error = true
        else
          if cat[:name].size > SidfmdCategory::MAX_NAME_LENGTH then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: カテゴリ名は#{SidfmdCategory::MAX_NAME_LENGTH}字以内で入力して下さい。"
            is_minor_error = true
          elsif cat[:name].empty? then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: カテゴリ名を空白文字以外で入力して下さい。"
            is_minor_error = true
          end
        end

        if cat[:category_id].present? then
          cat[:category_id] = cat[:category_id].tr('０-９','0-9').strip
          if (cat[:category_id] =~ /^[0-9]+$/) == nil then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: カテゴリIDは数値のみで入力して下さい。"
            is_minor_error = true
          end
        end
        
        if cat[:desc].empty? then
          session[:import_notices] << "ファイルの#{cat[:line_num]}行目: 概要を入力して下さい。"
          is_minor_error = true
        else
          if cat[:desc].size > SidfmdCategory::MAX_DESC_LENGTH then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: 概要は#{SidfmdCategory::MAX_DESC_LENGTH}字以内で入力して下さい。"
            is_minor_error = true
          end
        end

        if cat[:hostlim].size > 0 then
          if cat[:hostlim].match(/[^0-9]/) then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: #{SidfmdCategory::HOSTLIM_LABEL}には、0以上の数値(整数)を入力して下さい。※上限を設定しない場合は空白にして下さい。"
            is_minor_error = true
          elsif hostlim_over?(cat[:hostlim], max_host_num) then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: #{SidfmdCategory::HOSTLIM_LABEL}に設定可能な値は、最大#{max_host_num}(ライセンスのホスト数上限値)です。"
            is_minor_error = true
          end
        end

        if cat[:is_delete].present? then
          if cat[:is_delete] != 'true' then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: 削除列には「TRUE」もしくは「true」を入力して下さい。"
            is_minor_error = true
          else
            if is_openid_connect_enable and cat[:category_id] == get_openid_connect_category_id.to_s then
              session[:import_notices] << "ファイルの#{cat[:line_num]}行目: #{OPENID_CONNECT_AUTH_NAME}のカスタムカテゴリに指定されているため、削除できません。"
              is_minor_error = true
            end
          end
          if cat[:category_id].empty? then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: カテゴリIDを入力して下さい。"
            is_minor_error = true 
          end
        end

        if cat[:category_id].present? and check_data[:ids][cat[:category_id]] then
          dup_line = check_data[:ids][cat[:category_id]]
          session[:import_notices] << "ファイルの#{dup_line}行目と#{cat[:line_num]}行目: カテゴリIDが重複しています。"
          is_err = true
        end
        if cat[:is_delete] != 'true' and check_data[:names][cat[:name]] then
          dup_line = check_data[:names][cat[:name]]
          session[:import_notices] << "ファイルの#{dup_line}行目と#{cat[:line_num]}行目: カテゴリ名が重複しています。"
          check_data[:dup_names] << cat[:name]
          is_err = true
        end 

        check_data[:ids][cat[:category_id]] = cat[:line_num]
        check_data[:names][cat[:name]] = cat[:line_num] if cat[:is_delete] != 'true'
        
        if is_minor_error or is_err then
          next
        end
     
        if cat[:category_id].empty? then
          #新規登録
          action = {
            :action => 'new',
            :cat_name => cat[:name],
            :data => cat,
            :action_name => '新規登録'
          }
        elsif cat[:is_delete] == 'true' then
          #削除
          t_cat = SidfmdCategory.find_by(:id => cat[:category_id])
          if !t_cat then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: カテゴリが存在しません。"
            next
          end
          action = {
            :action => 'delete',
            :cat_id => cat[:category_id],
            :data => cat,
            :cat_name => t_cat.name,
            :action_name => '削除'
          }
        else
          #変更(チェック)
          t_cat = SidfmdCategory.find_by(:id => cat[:category_id])
          if !t_cat then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: カテゴリが存在しません。"
            next
          elsif t_cat.is_system then
            session[:import_notices] << "ファイルの#{cat[:line_num]}行目: システムカテゴリは変更できません。"
            next
          end
          options = []
          if t_cat.name != cat[:name] then
            options << {
              :action => 'name_change',
              :new_data => cat[:name],
              :old_data => t_cat.name,
              :action_name => 'カテゴリ名変更'
            }
          end

          if t_cat.desc != cat[:desc] then
            options << {
              :action => 'desc_change',
              :new_data => cat[:desc],
              :old_data => t_cat.desc,
              :action_name => '概要変更'
            }
          end

          if (!t_cat.hostlim and cat[:hostlim].size > 0) or
             (t_cat.hostlim and cat[:hostlim].size == 0) or
             (t_cat.hostlim and cat[:hostlim].size > 0 and t_cat.hostlim != cat[:hostlim].to_i)
          then
            options << {
              :action => 'hostlim_change',
              :new_data => cat[:hostlim],
              :old_data => t_cat.hostlim,
              :action_name => "#{SidfmdCategory::HOSTLIM_LABEL}変更"
            }
          end
          if options.any? then
            action = {
              :action => 'modify',
              :cat_id => cat[:category_id],
              :cat_name => t_cat.name,
              :options => options,
              :action_name => '変更'
            }
          end
        end
        
        actions << action if action
      end

      if executed_size == 0 then
        #カテゴリ名の変更/新規登録で、カテゴリ名に重複がないかチェック
        name_mod_ids = []
        names = []
        actions.each do |ac|
          ac = ac.deep_symbolize_keys
          if ac[:action] == 'modify' then
            cat_name_mod_op = ac[:options].find{|op| op[:action_name] == 'カテゴリ名変更'}
            if cat_name_mod_op then
              name_mod_ids << ac[:cat_id]
              names << cat_name_mod_op[:new_data]
            end
          elsif ac[:action] == 'new' then
            names << ac[:cat_name]
          elsif ac[:action] == 'delete' then
            name_mod_ids << ac[:cat_id]
          end
        end
        err_name_lines = []
        SidfmdCategory.where.not(:id => name_mod_ids, :name => check_data[:dup_names])
                      .where(:name => names)
                      .each do |existing_cat|
          err_name_lines << check_data[:names][existing_cat.name].to_i
        end
        if err_name_lines.any? then
          is_err = true
          err_name_lines.sort.each do |err_line|
            session[:import_notices] << "ファイルの#{err_line}行目: 既存のカテゴリと重複するカテゴリ名を登録することはできません。"
          end
        end
      end
      session[:import_actions] = actions
      session[:import_check_data] = check_data
    rescue Exception => e
      log_exception(e)
      is_exception = true
    end
    r[:err] = is_err
    r[:is_exception] = is_exception
    r[:executed_size] = executed_size
    render :layout => false, :json => r
  end

  def import
    if !is_file_output_executable? then
      redirect_to :action => :index, :notice =>  '許可されていない操作です。' and return
    end
    @notices = session[:import_notices]
    @actions = session[:import_actions]
    if !@actions then
      redirect_to :action => :index, :notice => 'インポートに失敗しました。' and return
    elsif @actions.empty? then
      if @notices.any? then
        redirect_to :action => :index
      else
        redirect_to :action => :index, :notice => '変更がありません。' and return
      end
    else
      @actions = @actions.sort{|a, b|
        r = get_sort_value(a['action']) <=> get_sort_value(b['action'])
      }
    end
  end

  def import_apply
    f_params = params.permit(:action_data, :delete_category_ids, :process_rules)
    err_message = ''
    begin
      action = JSON.parse(f_params[:action_data], :symbolize_names => true)
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = 'パラメータが不正です。'
    end

    if f_params[:process_rules] then
      begin
        delete_category_ids = JSON.parse(f_params[:delete_category_ids])
      rescue Exception => e
        log_exception(e)
        render :layout => false, :json => { :rules_error => true } and return 
      end
      if err_message.size == 0 then
        begin 
          SidfmdRule.transaction do
            process_rules = process_related_rules(
              :delete_category_ids => delete_category_ids,
              :action => 'import'
            )
            if !process_rules then
              render :layout => false, :json => { :rules_error => true } and return
            end
          end
        rescue Exception => e
          render :layout => false, :json => { :rules_error => true } and return
        end
      end
    end

    if err_message.size > 0 then
      render :layout => false, :json => { :err_message => err_message } and return
    end

    case action[:action]
    when 'new' then
      #新規登録
      SidfmdCategory.create(
        name: action[:data][:name],
        desc: action[:data][:desc],
        hostlim: action[:data][:hostlim],
        is_system: false
      )
    when 'modify' then
      #変更
      t_cat = SidfmdCategory.find_by(:id => action[:cat_id])
      success = true
      SidfmdCategory.transaction do
        if t_cat and !t_cat.is_system then
          action[:options].each do |op|
            case op[:action]
            when 'name_change' then
              success &= t_cat.update(:name => op[:new_data])
            when 'desc_change' then
              success &= t_cat.update(:desc => op[:new_data])
            when 'hostlim_change' then
              success &= t_cat.update(:hostlim => op[:new_data])
            end
          end
        end
        unless success then
          err_message = "エラーが発生したため、「#{data[:cat_name]}」の変更は行われませんでした。"
          raise ActiveRecord::Rollback
        else
          if !(action[:options].size == 1 and action[:options].first[:action] == 'hostlim_change') then
            #View更新
            host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => action[:cat_id]).distinct.pluck(:sidfmd_host_id)
            html_ids = SidfmdHostView.relate_htmlids(host_ids).uniq
            SidfmdVulView.sync_data(html_ids)
            SidfmdHostView.sync_data(host_ids)
          end
        end
      end
    when 'delete' then
      t_cat = SidfmdCategory.find_by(:id => action[:cat_id])
      if t_cat then
        t_cat_id = t_cat.id
        if openid_connect_enable? and t_cat_id == get_openid_connect_category_id then
          err_message = "#{t_cat.name}は、#{OPENID_CONNECT_AUTH_NAME}のカスタムカテゴリに指定されているため、削除できませんでした。"
        else
          is_delete_err = false
          host_ids = []
          ctag_related_issue_ids = []
          begin
            SidfmdCategory.transaction do
              host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => action[:cat_id]).distinct.pluck(:sidfmd_host_id)
              cat_ctags_ids = SidfmdCtag.where(:sidfmd_category_id => t_cat_id).pluck(:id)
              ctag_related_issue_ids = SidfmdCtag.delete_ctags(cat_ctags_ids)
              cat_groups = SidfmdCatGroup.only_categories([t_cat_id])
              if cat_groups.size > 0 then
                SidfmdCatGroup.where(:id => cat_groups.map{|grp| grp[:id]}).destroy_all
              end
              if t_cat.destroy then
                if get_openid_connect_category_id == t_cat_id then
                  Sidfm.openid_connect_settings_user_category_ids = ''
                end
              end
            end
          rescue Exception => e
            is_delete_err = true
            err_message = "#{t_cat.name}の削除に失敗しました。"
          end
          if !is_delete_err then
            #View更新
            html_ids = SidfmdHostView.relate_htmlids(host_ids).uniq
            html_ids |= SidfmdIssue.where(:id => ctag_related_issue_ids).distinct.pluck(:html_id) if ctag_related_issue_ids.any?
            SidfmdVulView.sync_data(html_ids)
            SidfmdHostView.sync_data(host_ids)
          end
        end
      end
    end
    Rails.cache.clear
    render :layout => false, :json => { :err_message => err_message }
  end

  def update_view
    f_params = params.permit(:host_ids)
    host_ids = JSON.parse(f_params[:host_ids])
    target_host_ids = host_ids.shift(100)
    executed_hosts = target_host_ids.size
    target_html_ids = SidfmdHostView.relate_htmlids(target_host_ids)
    SidfmdVulView.sync_data(target_html_ids.uniq)
    SidfmdHostView.sync_data(target_host_ids)
    render :layout => false, :json => { :executed_hosts => executed_hosts, :unexecuted_host_ids => host_ids }
  end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    parse_params
    @categories[:per_page] = f_params[:per_page].to_i
    @categories[:content] = SidfmdCategory.order(
      "CASE WHEN is_system IS TRUE THEN id ELSE -1 END"
    ).order('name COLLATE "C"')
    if !@sidfm_roles.include?(:admin) then
      @categories[:content] = @categories[:content].where(:id => User.current.categories.select(:id))
    end
    @categories[:total_size] = @categories[:content].count
    @all_c_cat_ids = @categories[:content].where(:is_system => false).pluck(:id)
    @categories[:content] = @categories[:content].offset(@categories[:offset]).limit(@categories[:per_page])
  
    @is_openid_connect_enable = openid_connect_enable?
    @openid_connect_category_id = get_openid_connect_category_id
    @openid_connect_auth_name = OPENID_CONNECT_AUTH_NAME
    ret = {
      :rows => @categories[:content].map{|category|
        create_summary_category_html(
          category,
          {
            :checked =>  @categories[:checked],
            :is_openid_connect_enable => @is_openid_connect_enable,
            :openid_connect_category_id => @openid_connect_category_id,
            :openid_connect_auth_name => @openid_connect_auth_name
          }
        )
      },
      :all_ids => @all_c_cat_ids,
      :total => @categories[:total_size],
      :sidfm_roles => @sidfm_roles,
      :user_cats => (@sidfm_roles.include?(:admin) ? nil : User.current.categories.pluck(:id))
    }
    render :layout => false, :json => ret
  end

  #カテゴリに紐づいているカスタムタグ/カスタムタグの脆弱性がないかチェック
  def api_category_related_ctags
    f_params = params.permit(:ids)
    cat_ids = JSON.parse(f_params[:ids])
    cat_related_ctag_ids = SidfmdCtag.where(:sidfmd_category_id => cat_ids).pluck(:id)
    is_ctag_related_issue = SidfmdCtag.issue_related_ctags(cat_related_ctag_ids).exists?
    render :layout => false, :json => { :is_related_ctag => cat_related_ctag_ids.any?, :is_ctag_related_issue => is_ctag_related_issue }
  end

  #指定したカテゴリに関連するカテゴリグループのチェック
  #今の所カテゴリ削除時のみチェック
  def api_check_related_cat_groups
    f_params = params.permit(:ids, :action_name)
    ret = {}
    ids = JSON.parse(f_params[:ids])
    case f_params[:action_name]
    when 'delete' then
      ret[:delete_cat_groups] = SidfmdCatGroup.only_categories(ids)
    end
    render :layout => false, :json => ret
  end
end
