#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmCategoriesHelper

  # サマリー表のチェックボックス
  def create_summary_category_chkbox(category_id, checked, is_disabled)
    content = check_box_tag(
      'table[selected]',
      category_id,
      checked.include?(category_id),
      :class => (is_disabled ? '' : 'categorySelect'),
      :disabled => is_disabled
    )
  end

  # サマリー表のカテゴリ名
  def create_summary_category_name(category_name)
    content = content_tag(
      :div,
      "#{category_name}",
      :class => "word-break-all",
      :title => "#{category_name}",
    )
  end

  # サマリー表の概要
  def create_summary_category_description(description)
    content = content_tag(
      :div,
      "#{description}"
    )
  end

  # サマリー表のホスト登録上限数
  def create_summary_category_hostlim(is_system, hostlim)
    content = 
      if is_system then
        content_tag(
          :span,
          "制限なし"
        )
        .concat(
          content_tag(
            :a,
            :data => {
              :toggle => "tooltip",
              :placement => "right",
              :html => "true",
              :title => "システムカテゴリに制限はありません。"
            }
          ) do
            concat( content_tag(:i, nil, :class => "fas fa-question-circle c_gray_70") )
          end
        )
      else  
        hostlim ? hostlim : "制限なし"
      end 
  end

  # サマリー表の区分
  def create_summary_category_class(is_system)
    is_system ? "システム" : "カスタム"
  end

  # サマリー表の操作
  def create_summary_category_operation(category, is_openid_connect_enable, openid_connect_category_id, openid_connect_auth_name, sidfm_roles)
    content =
      if category.is_system then
        content_tag(:span, "システムカテゴリは変更できません。")
      else
        content_tag(
          :div,
          :class => "d-flex justify-content-center"
        ) do
          concat(
            button_tag(
              "編集",
              :type => 'button',
              :class => 'btn btn-sm btn-edit mr-1 text-nowrap',
              :data => {
                :toggle => 'modal',
                :target => '#modal-form',
                :backdrop => 'static',
                :keyboard => 'true',
                :category => category.id.to_s,
                :title => "カテゴリの編集",
                :userid => category.user_id ? category.user_id.to_i.to_s : '0',
                :name => category.name,
                :desc => category.desc,
                :users => category.users.select(:id).collect {|x| x.id }.to_json,
                :func => 'mod',
                :hostlim => category.hostlim,
              },
              :disabled => !sidfm_roles.include?(:admin),
              :title => sidfm_roles.include?(:admin) ? '' : 'カテゴリ情報の操作にはSIDfmVMの管理権限が必要です',
              :autocomplete => 'off'
            )
          )
          .concat(
            form_tag(
              url_for(:controller => :sidfm_categories, :action => :delete),
              :method => :post,
              :data => { :formdel => "category_#{category.id}" }
            ) do
              concat(
                hidden_field_tag('formdel[id]', category.id, :id => "formdel_id_#{category.id}")
              )
              .concat(
                if sidfm_roles.include?(:admin) then
                  if is_openid_connect_enable and category.id == openid_connect_category_id then
                    button_tag(
                      "削除",
                      :type => 'button',
                      :class => 'btn btn-sm btn-delete text-nowrap disabled',
                      :data => {
                        :title => "#{openid_connect_auth_name}のカスタムカテゴリに指定されています",
                        :toggle => 'tooltip'
                      }
                    )
                  else
                    button_tag(
                      "削除",
                      :type => 'button',
                      :class => 'btn btn-sm btn-delete text-nowrap',
                      :data => {
                        :conftext => "カテゴリ「#{category.name}」を削除してもよろしいですか？",
                        :delcategory => category.id,
                        :categoryname => category.name
                      }
                    )
                  end
                else
                  submit_tag(
                    "削除",
                    :class => 'btn btn-sm btn-delete text-nowrap',
                    :disabled => 'disabled',
                    :title => 'カテゴリ情報の操作にはSIDfmVMの管理権限が必要です'
                  )
                end
              )
            end
          )
        end
      end
  end

end
