#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdIssue < ActiveRecord::Base
  belongs_to :issue
  belongs_to :sidfmd_host

  has_many :sidfmd_issue_cves, :dependent => :destroy
  has_many :sidfmd_issue_sris, :dependent => :destroy
  has_many :sidfmd_issue_affprods, :dependent => :destroy
  has_many :sidfmd_issue_affvers, :dependent => :destroy
  has_many :sidfmd_issue_affpkgs, :dependent => :destroy
  has_many :sidfmd_issue_ctags, :dependent => :destroy

  scope :enable_hosts, -> { where(:sidfmd_host_id =>  SidfmdHost.enable_hosts.select(:id)) }
  scope :categorized, -> { where(:sidfmd_host_id => SidfmdCategoriesHost.select(:sidfmd_host_id).distinct(:sidfmd_host_id)) }

  scope :full_content, -> { except(:offset, :limit) }
  scope :total_size, -> { except(:order, :offset, :limit).length }

  scope :order_param, ->(p) {
    case p
    #when 'id_asc' then order(:id => :asc)
    #when 'id_desc' then order(:id => :desc)
    when 'title_asc' then order(:title => :asc)
    when 'title_desc' then order(:title => :desc)
    #when 'created_on_asc' then order('issues.created_on ASC').order(:id => :asc)
    #when 'created_on_desc' then order('issues.created_on DESC').order(:id => :asc)
    else order(:title => :asc)
    end
  }

  # 管理ユーザ
  ADMIN_USER_ID = begin
                 ret = User.find_by_login('sidfmadm')
                 ret = User.find_by_login('admin') unless ret
                 ret ? ret[:id] : nil
               end

  # 未確認
  STATUS_NOTCONFIRMED = 1
  # 対応中
  STATUS_ONGOING = 2
  # 対策済
  STATUS_FIXED = 3
  # 影響無
  STATUS_NOTAFFECT = 4

  STATUS_ALL = [ STATUS_NOTCONFIRMED,
                 STATUS_ONGOING,
                 STATUS_FIXED,
                 STATUS_NOTAFFECT ].freeze
  STATUS_UNFINISHED = [ STATUS_NOTCONFIRMED, STATUS_ONGOING ].freeze
  STATUS_FINISHED = [ STATUS_FIXED, STATUS_NOTAFFECT ].freeze
  SORT_PRIORITIES = {
    SidfmdIssue::STATUS_NOTCONFIRMED => 200,
    SidfmdIssue::STATUS_ONGOING => 100,
    SidfmdIssue::STATUS_FIXED => 300,
    SidfmdIssue::STATUS_NOTAFFECT => 400,
  }.freeze

  # Status Name Table
  STATUS_NAMES =
    [ '(不正)',
      IssueStatus.find(1).name.to_s,
      IssueStatus.find(2).name.to_s,
      IssueStatus.find(3).name.to_s,
      IssueStatus.find(4).name.to_s ].map(&:freeze).freeze
  STATUS_LABELS =
    [ nil, :notconfirmed, :ongoing, :fixed, :notaffect ].freeze

  def self.label2value(label)
    idx = STATUS_LABELS.index(label)
    (idx and idx > 0) ? idx : 1
  end

  # status_idの配列から、{ <status_id> => { :id => <status_id>, :name => <name>, :count => <count> }, ... }
  # のようにstatus_id毎に集計する。
  def self.count_status_ids(status_ids)
    ret = {}
    status_ids.sort.each do |status_id|
      if ret.has_key?(status_id) then
        ret[status_id][:value] += 1
      else
        ret[status_id] = {
          :id => status_id,
          :name => STATUS_NAMES[status_id],
          :value => 1,
          :color => ('status_' + STATUS_LABELS[status_id].to_s).to_sym,
        }
      end
    end
    ret.values
  end

  # status_idから対応状況名称に変換
  def self.status_id_to_s(i_status_id)
    ret = '(不正)'
    if STATUS_ALL.include?(i_status_id) then
      ret = IssueStatus.find(i_status_id).name
    end
    ret
  end

  # due_dateから対応日付に変換
  def self.due_date_to_s(i_due_date)
    if i_due_date.respond_to?(:strftime) then
      i_due_date = i_due_date.strftime('%Y/%m/%d')
    end
    i_due_date ? i_due_date : '未設定'
  end

  def get_sri
    sc = sidfmd_host.system_category
    sidfmd_issue_sris.where(:category_id => sc.id).first
  end

  def self.risk_host_count_sri
    ### FIXME ###
  end

  def self.risk_host_count_cvss
    ### FIXME ###
  end

  def self.risk_host_count_softek
    cnt = enable_hosts.group(:sidfmd_host_id).maximum(:risk_id)
    ret = {}
    cnt.values.each do |k|
      if ret.has_key?(k) then
        ret[k] += 1
      else
        ret[k] = 1
      end
    end
    ret
  end

  # issueの状態をカウントして返す。
  # { status_id => issueの件数, ... }の形となる。
  def self.status_count_htmlid(htmlid)
    status2color = [ '', :danger, :warning, :info, :success ]

    cnt = SidfmdIssue.enable_hosts.joins(:issue).where(:html_id => htmlid).group(:status_id).order('issues.status_id').count
    ret = []
    cnt.each_pair do |k, v|
      ret << {
        :id => htmlid,
        :value => v,
        :name => STATUS_NAMES[k],
        :color => ('status_' + STATUS_LABELS[k].to_s).to_sym,
        :placement => 'top' }
    end
    ret
  end

  def self.status_count
    hs_count = enable_hosts.joins(:issue).group(:sidfmd_host_id, :status_id).count
    hs = {}
    hs_count.keys.each do |k|
      hostid = k[0]
      statusid = k[1]
      if hs.has_key?(k[0]) then
        hs[k[0]] = k[1] if hs[k[0]] > k[1]
      else
        hs[k[0]] = k[1]
      end
    end
    sh_cnt = {}
    hs.each do |k, v|
      if sh_cnt.has_key?(v) then
        sh_cnt[v] += 1
      else
        sh_cnt[v] = 1
      end
    end
    sh_cnt
  end

  # 登録済で、かつ、未確認・確認中の対策予定日を返す。
  # 対策予定日=nilは未設定。
  def self.get_due_dates
    self.enable_hosts.joins(:issue).where(:issues => { :status_id => STATUS_UNFINISHED }).select('issues.due_date').distinct('issues.due_date').order('due_date DESC')
  end

  def self.get_max_sri(htmlid)
    ### FIXME ###
    1
  end

  def self.get_max_cvss(htmlid)
    ### FIXME ###
    10.0
  end

  # 対応が必要な脆弱性かどうかを返す
  # sidfmd_issue自体をチェックするのではなく、htmlidに対して検証する
  # ため、クラスメソッドになっていることに注意。
  def self.corresp_required_htmlid?(htmlid, index)
    case index
    when :sri then
      corresp_required_htmlid_sri?(htmlid)
    when :cvss then
      corresp_required_htmlid_cvss?(htmlid)
    when :softek then
      corresp_required_htmlid_softek?(htmlid)
    else
      false
    end
  end


  # 対応が必要な脆弱性かどうかを返す(基準:SRI)。
  def self.corresp_required_htmlid_sri?(htmlid)
    ret = false

    cr_sri = Sidfm.corresp_required_sri

    issues = SidfmdIssue.enable_hosts.joins(:sidfmd_issue_sris).where(:html_id => htmlid).where('sidfmd_issue_sris.risk_id <= ?', cr_sri)
    if issues.size > 0 then
      issues.each do |issue|
        if issue.get_sri.risk_id <= cr_sri then
          ret = true
          break
        end
      end
    end

    ret
  end

  # 対応が必要な脆弱性かどうかを返す(基準:CVSS)。
  def self.corresp_required_htmlid_cvss?(htmlid)
    cr_cvss = Sidfm.corresp_required_cvss

    issues = SidfmdIssue.enable_hosts.joins(:sidfmd_issue_cves).where(:html_id => htmlid).where('sidfmd_issue_cves.c3_base_score > ? OR sidfmd_issue_cves.c2_base_score > ?', cr_cvss, cr_cvss)
    issues ? issues.count > 0 : false
  end

  # 対応が必要な脆弱性かどうかを返す(基準:Softek)。
  def self.corresp_required_htmlid_softek?(htmlid)
    cr_risk = Sidfm.corresp_required_risk

    issues = SidfmdIssue.enable_hosts.where('risk_id >= ?', cr_risk)
    issues ? issues.count > 0 : false
  end

  def has_exploitable?
    SidfmdIssueCve.joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => sidfmd_host_id }).where('c2_exploitable IS TRUE OR c3_exploitable IS TRUE').count > 0
  end

  # SidfmdHost#due_date_scheduled_percentのhtmlid版
  # 対策予定日(due_date)設定済の脆弱性の件数の割合
  # STATUS_FINISHEDはカウントから除外。
  # STATUS_UNFINISHEDかつ issues.due_dateがnil以外を設定済として扱う。
  def self.due_date_scheduled_percent_htmlid(htmlid)
    s = SidfmdIssue.joins(:issue).where(:html_id => htmlid).where(:issues => { :status_id => STATUS_UNFINISHED })
    cnt_all = s.count
    cnt_due_date = s.where.not(:issues => { :due_date => nil }).count
    cnt_all > 0 ? cnt_due_date.to_f / cnt_all.to_f : nil
  end

  # SidfmdHost#finished_percentのhtmlid版
  # 対応状況(%)を返す。
  # STATUS_FINISHEDを対応済としてカウント。
  def self.finished_percent_htmlid(htmlid)
    s = SidfmdIssue.joins(:issue).where(:html_id => htmlid)
    cnt_all = s.count
    cnt_finished = s.where(:issues => { :status_id => STATUS_FINISHED }).count
    cnt_all > 0 ? cnt_finished.to_f / cnt_all.to_f : nil
  end

  # 対応状況のデータを返す。
  # 基本的にはIssueStatusの情報にlabelというフィールドを追加したものになる。
  def self.statuses
    ret = JSON.parse(IssueStatus.all.order(:id).to_json)
    ret.each do |i|
      case i['id']
      when STATUS_NOTCONFIRMED then i['label'] = :notconfirmed
      when STATUS_ONGOING then i['label'] = :ongoing
      when STATUS_FIXED then i['label'] = :fixed
      when STATUS_NOTAFFECT then i['label'] = :notaffect
      end
    end
    ret
  end

  # 対応状況毎の脆弱性数分布
  def self.distribution_status(category_id = 0)
    dists = SidfmdIssue.enable_hosts
    if category_id > 0 then
      dists = dists
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id).distinct)
    else
      dists = dists.categorized
    end
    dists = dists.joins(:issue).group(:html_id, :status_id).count
    dists_htmlid = {}
    dists.each_pair do |dist, v|
      if dists_htmlid.has_key?(dist[0]) then
        if dists_htmlid[dist[0]] > dist[1] then
          dists_htmlid[dist[0]] = dist[1]
        end
      else
        dists_htmlid[dist[0]] = dist[1]
      end
    end
    ret = {
      :notconfirmed => 0,
      :ongoing => 0,
      :fixed => 0,
      :notaffect => 0,
    }
    dists_htmlid.each_pair do |html_id, status_id|
      case status_id
      when SidfmdIssue::STATUS_NOTCONFIRMED then
        ret[:notconfirmed] += 1
      when SidfmdIssue::STATUS_ONGOING then
        ret[:ongoing] += 1
      when SidfmdIssue::STATUS_FIXED then
        ret[:fixed] += 1
      when SidfmdIssue::STATUS_NOTAFFECT then
        ret[:notaffect] += 1
      end
    end
    ret
  end

  # 危険度毎の脆弱性数分布
  def self.distribution_risk(idx = :sri, category_id = 0)
    case idx
    when :sri then self.distribution_risk_sri(category_id)
    when :cvss then self.distribution_risk_cvss(category_id)
    when :softek then self.distribution_risk_softek(category_id)
    end
  end

  # 危険度毎の脆弱性数分布(SRI)
  def self.distribution_risk_sri(category_id = 0)
    hosts = SidfmdHost.enable_hosts
    if category_id > 0 then
      hosts = hosts.where(:id =>
                          SidfmdCategoriesHost
                            .where(:sidfmd_category_id => category_id)
                            .select(:sidfmd_host_id).distinct)
    else
      hosts = hosts.categorized
    end
    dists = {}
    hosts.each do |host|
      scat = host.system_category.id
      issues = host.sidfmd_issues
      issues.each do |issue|
        sri = issue.sidfmd_issue_sris.where(:category_id => scat).first
        unless dists.has_key?(issue.html_id) then
          dists[issue.html_id] = 0
        end
        if dists[issue.html_id] < sri.risk_id then
          dists[issue.html_id] = sri.risk_id
        end
      end
    end
    ret = {
      :critical => 0,
      :high => 0,
      :medium => 0,
      :low => 0,
      :none => 0,
    }
    risks = [ :none, :low, :medium, :high, :critical ]
    dists.each_pair do |html_id, risk_id|
      ret[risks[risk_id]] += 1
    end
    ret
  end

  # 危険度毎の脆弱性数分布(CVSS)
  def self.distribution_risk_cvss(category_id = 0)
    htmlids = SidfmdIssue.enable_hosts
    if category_id > 0 then
      htmlids = htmlids
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id).distinct)
    else
      htmlids = htmlids.categorized
    end
    htmlids = htmlids.select(:html_id).distinct
    ret = {
      :d4 => 0,
      :d3 => 0,
      :d2 => 0,
      :d1 => 0,
      :notreg => 0,
    }
    htmlids.each do |htmlid|
      issue = SidfmdIssue
        .where(:html_id => htmlid.html_id)
        .order(:id => :desc)
        .limit(1).first
      max_score = (issue.sidfmd_issue_cves.select('MAX(c2_base_score) AS c2_max_score').select('MAX(c3_base_score) AS c3_max_score'))[0]
      if max_score then
        c2_max_score = max_score.c2_max_score
        c3_max_score = max_score.c3_max_score
        if c3_max_score.nil? and c2_max_score.nil? then
          ret[:notreg] += 1
        elsif c3_max_score and c2_max_score.nil? then
          # CVSS 3
          if c3_max_score >= 9.0 then
            ret[:d4] += 1
          elsif c3_max_score < 9.0 and c3_max_score >= 7.0 then
            ret[:d3] += 1
          elsif c3_max_score < 7.0 and c3_max_score >= 4.0 then
            ret[:d2] += 1
          else
            ret[:d1] += 1
          end
        else
          # CVSS 2
          if c2_max_score >= 9.0 then
            ret[:d4] += 1
          elsif c2_max_score < 9.0 and c2_max_score >= 7.0 then
            ret[:d3] += 1
          elsif c2_max_score < 7.0 and c2_max_score >= 4.0 then
            ret[:d2] += 1
          else
            ret[:d1] += 1
          end
        end
      else
        ret[:notreg] += 1
      end
    end
    ret
  end
    
  # 危険度毎の脆弱性数分布(SofTek)
  def self.distribution_risk_softek(category_id = 0)
    dists =
      SidfmdIssue.enable_hosts
    if category_id > 0 then
      dists = dists
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id)
                 .distinct)
    else
      dists = dists.categorized
    end
    dists = dists
      .group(:html_id, :risk_id).count
    ret = {
      :critical => 0,
      :high => 0,
      :medium => 0,
      :low => 0,
      :illegal => 0,
    }
    risks = [ :illegal, :low, :medium, :high, :critical ]
    dists.each_pair do |k, v|
      ret[risks[k[1]]] += 1
    end
    ret
  end

  # ヒートマップ用のデータを返す
  # :html_id, :title, :risk_id :risk_name
  def self.enable_all_vuls(idx = :sri, category_id = 0)
    case idx
    when :sri then self.enable_all_vuls_sri(category_id)
    when :cvss then self.enable_all_vuls_cvss(category_id)
    when :softek then self.enable_all_vuls_softek(category_id)
    end
  end

  # ヒートマップ用のデータを返す(SRI)
  def self.enable_all_vuls_sri(category_id = 0)
    hosts = SidfmdHost.enable_hosts
    if category_id > 0 then
      hosts = hosts.where(:id =>
                          SidfmdCategoriesHost
                            .where(:sidfmd_category_id => category_id)
                            .select(:sidfmd_host_id).distinct)
    else
      hosts = hosts.categorized
    end
    dists = {}
    hosts.each do |host|
      scat = host.system_category.id
      issues = host.sidfmd_issues
      issues.each do |issue|
        sri = issue.sidfmd_issue_sris.where(:category_id => scat).first
        unless dists.has_key?(issue.html_id) then
          dists[issue.html_id] = { :title => issue.title, :risk_id => 0 }
        end
        if dists[issue.html_id][:risk_id] < sri.risk_id then
          dists[issue.html_id][:risk_id] = sri.risk_id
        end
      end
    end
    risks = [ '無', '低', '中', '高', '重大' ]
    ret = []
    dists.each_pair do |html_id, v|
      ret << { :html_id => html_id,
        :title => v[:title],
        :risk_id => v[:risk_id],
        :risk_name => risks[v[:risk_id]],
      }
    end
    ret.sort! {|x, y| x[:html_id] <=> y[:html_id] }
  end

  # ヒートマップ用のデータを返す(CVSS)
  def self.enable_all_vuls_cvss(category_id = 0)
    htmlids = SidfmdIssue.enable_hosts
    if category_id > 0 then
      htmlids = htmlids
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id).distinct)
    else
      htmlids = htmlids.categorized
    end
    htmlids = htmlids.select(:html_id).distinct
    ret = []
    htmlids.each do |htmlid|
      issue = SidfmdIssue
        .where(:html_id => htmlid.html_id)
        .order(:id => :desc)
        .limit(1).first
      max_score = (issue.sidfmd_issue_cves.select('MAX(c2_base_score) AS c2_max_score').select('MAX(c3_base_score) AS c3_max_score'))[0]
      if max_score then
        c2_max_score = max_score.c2_max_score
        c3_max_score = max_score.c3_max_score
        if c3_max_score.nil? and c2_max_score.nil? then
          ret << {
            :html_id => htmlid.html_id,
            :title => issue.title,
            :risk_id => 'notreg',
            :risk_name => '未登録',
          }
        elsif c3_max_score and c2_max_score.nil? then
          # CVSS 3
          ret << {
            :html_id => htmlid.html_id,
            :title => issue.title,
            :risk_name => sprintf('%2.1f', c3_max_score),
            :risk_id => if c3_max_score >= 9.0 then
                          'd4'
                        elsif c3_max_score < 9.0 and c3_max_score >= 7.0 then
                          'd3'
                        elsif c3_max_score < 7.0 and c3_max_score >= 4.0 then
                          'd2'
                        else
                          'd1'
                        end,
          }
        else
          # CVSS 2
          ret << {
            :html_id => htmlid.html_id,
            :title => issue.title,
            :risk_name => sprintf('%2.1f', c2_max_score),
            :risk_id => if c2_max_score >= 9.0 then
                          'd4'
                        elsif c2_max_score < 9.0 and c2_max_score >= 7.0 then
                          'd3'
                        elsif c2_max_score < 7.0 and c2_max_score >= 4.0 then
                          'd2'
                        else
                          'd1'
                        end,
          }
        end
      else
        ret << {
          :html_id => htmlid.html_id,
          :title => issue.title,
          :risk_id => 'notreg',
          :risk_name => '未登録',
        }
      end
    end
    ret.sort! {|x, y| x[:html_id] <=> y[:html_id] }
  end

  # ヒートマップ用のデータを返す(SofTek)
  def self.enable_all_vuls_softek(category_id = 0)
    dists =
      SidfmdIssue.enable_hosts
    if category_id > 0 then
      dists = dists
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id)
                 .distinct)
    else
      dists = dists.categorized
    end
    dists = dists
      .group(:html_id, :title, :risk_id).count
    risks = [ 'Illeg', 'Low', 'Medium', 'High', 'Critical' ]
    ret = []
    dists.each_pair do |k, v|
      ret << {
        :html_id => k[0],
        :title => k[1],
        :risk_id => k[2],
        :risk_name => risks[k[2]],
      }
    end
    ret.sort!{|x, y| x[:html_id] <=> y[:html_id] }
  end

  # status_idのが配列からstatus_bar用のデータを作成して返す。
  def self.statusids2count(statusids)
    status2color = {
      STATUS_NOTAFFECT => :status_notaffect,
      STATUS_FIXED => :status_fixed,
      STATUS_ONGOING => :status_ongoing,
      STATUS_NOTCONFIRMED => :status_notconfirmed
    }
    ret = []
    STATUS_ALL.each do |status_id|
      ret << {
        :id => status_id,
        :value => statusids.count(status_id),
        :name => IssueStatus.find(status_id).name,
        :color => status2color[status_id],
        :placement => 'top',
      }
    end
    ret
  end

  # journal付で対応状況の変更を行う
  # 必ずこれで変更すること。
  # transaction, lockは呼び元で考慮する。 
  def change_status_with_journal(user_id, new_status_id, notes)
    journal_hash = {
      :journalized_id => self.issue.id,
      :journalized_type => 'Issue',
      :user_id => user_id,
      :notes => notes,
      :private_notes => false,
    }
    journal = Journal.create(journal_hash)
    old_value = self.issue.status_id
    detail_hash = {
      :journal_id => journal.id,
      :property => 'attr',
      :prop_key => 'status_id',
      :old_value => old_value ? "#{old_value}" : nil,
      :value => "#{new_status_id}",
    }
    detail = JournalDetail.create(detail_hash)
    self.issue.update_attribute :status_id, new_status_id
  end

  # journal付で対応状況の変更を行う
  # 必ずこれで変更すること。
  # transaction, lockは呼び元で考慮する。 
  # change_status_with_journalの複数版だが、機能を向上してこちらに移行する予定。
  # old_status_idsがnilなら強制的に更新
  def self.change_statuses_with_journal(issue_ids, old_status_ids, new_status_id, notes, user_id = ADMIN_USER_ID)
=begin    if old_status_ids then
      issues = Issue.where(:id => issue_ids).where(:status_id => old_status_ids)
    else
      issues = Issue.where(:id => issue_ids)
    end
    issues.each do |issue|
      journal_hash = {
        :journalized_id => issue.id,
        :journalized_type => 'Issue',
        :user_id => user_id,
        :notes => notes,
        :private_notes => false,
      }
      journal = Journal.create(journal_hash)
      old_value = issue.status_id
      detail_hash = {
        :journal_id => journal.id,
        :property => 'attr',
        :prop_key => 'status_id',
        :old_value => old_value ? "#{old_value}" : nil,
        :value => "#{new_status_id}",
      }
      JournalDetail.create(detail_hash)
    end
    issues.update_all(:status_id => new_status_id)
=end
    if old_status_ids then
      issues = Issue.where(:id => issue_ids).where(:status_id => old_status_ids)
    else
      issues = Issue.where(:id => issue_ids)
    end
    journals = []
    issues.each do |issue|
      old_value = issue.status_id
      next if old_value == new_status_id
      journal_hash = {
        :journalized_id => issue.id,
        :journalized_type => 'Issue',
        :user_id => user_id,
        :notes => notes,
        :private_notes => false,
      }
      journal = Journal.new(journal_hash)
      detail_hash = {
        :property => 'attr',
        :prop_key => 'status_id',
        :old_value => old_value ? "#{old_value}" : nil,
        :value => "#{new_status_id}",
      }
      journal.details.build(detail_hash)
      journals << journal
    end
    Journal.import journals, recursive: true
    issues.update_all(:status_id => new_status_id)
  end

  # journal付で対応状況の変更を行う
  # 必ずこれで変更すること。
  # transaction, lockは呼び元で考慮する。 
  # change_due_date_with_journalの複数版だが、機能を向上してこちらに移行する予定。
  # old_due_dateがnilなら強制的に更新
  def self.change_due_dates_with_journal(issue_ids, force_flg, new_due_date, notes, user_id = ADMIN_USER_ID)
=begin    if !force_flg then
      issues = Issue.where(:id => issue_ids).where(:due_date => nil)
    else 
      issues = Issue.where(:id => issue_ids)
    end
    issues.each do |issue|
      journal_hash = {
        :journalized_id => issue.id,
        :journalized_type => 'Issue',
        :user_id => user_id,
        :notes => notes,
        :private_notes => false,
      }
      journal = Journal.create(journal_hash)
      old_value = issue.due_date
      detail_hash = {
        :journal_id => journal.id,
        :property => 'attr',
        :prop_key => 'due_date',
        :old_value => old_value ? old_value.strftime('%Y/%m/%d') : nil,
        :value => new_due_date ? new_due_date.strftime('%Y/%m/%d') : nil,
      }
      JournalDetail.create(detail_hash)
    end
    issues.update_all(:due_date => new_due_date)
=end
    if !force_flg then
      issues = Issue.where(:id => issue_ids).where(:due_date => nil)
    else
      issues = Issue.where(:id => issue_ids)
    end
    journals = []
    issues.each do |issue|
      old_value = issue.due_date
      next if old_value == new_due_date
      journal_hash = {
        :journalized_id => issue.id,
        :journalized_type => 'Issue',
        :user_id => user_id,
        :notes => notes,
        :private_notes => false,
      }
      journal = Journal.new(journal_hash)
      detail_hash = {
        :property => 'attr',
        :prop_key => 'due_date',
        :old_value => old_value ? old_value.strftime('%Y/%m/%d') : nil,
        :value => new_due_date ? new_due_date.strftime('%Y/%m/%d') : nil,
      }
      journal.details.build(detail_hash)
      journals << journal
    end
    Journal.import journals, recursive: true
    issues.update_all(:due_date => new_due_date)
  end

  # journal付で対応日付の変更を行う 
  # 必ずこれで変更すること。
  # transaction, lockは呼び元で考慮する。 
  def change_due_date_with_journal(user_id, new_due_date, notes)
    journal_hash = {
      :journalized_id => self.issue.id,
      :journalized_type => 'Issue',
      :user_id => user_id,
      :notes => notes,
      :private_notes => false,
    }
    journal = Journal.create(journal_hash)
    old_value = self.issue.due_date
    detail_hash = {
      :journal_id => journal.id,
      :property => 'attr',
      :prop_key => 'due_date',
      :old_value => old_value ? old_value.strftime('%Y/%m/%d') : nil,
      :value => new_due_date ? new_due_date.strftime('%Y/%m/%d') : nil,
    }
    detail = JournalDetail.create(detail_hash)
    self.issue.update_attribute :due_date, new_due_date
  end

  # 影響しているパッケージ一覧
  def affect_packages(suffix = '.rpm')
    ret = []
    deli = '-'
    dist = self.sidfmd_host.get_dist_data
    if dist and 'Ubuntu' == dist.name then
      deli = '_'
    end
    pkgs = sidfmd_issue_affpkgs
    pkgs.each do |pkg|
      temp = pkg.attributes
      temp.delete('id')
      temp.delete('sidfmd_issue_id')
      temp[:fullname] = "#{pkg['name']}#{deli}#{pkg['version']}"
      temp[:fullname] << "-#{pkg['release']}" if pkg['release']
      temp[:fullname] << "#{pkg['arch']}" if pkg['arch']
      temp[:fullname] << "#{suffix}" if (suffix and suffix.size > 0)
      ret << temp
    end
    ret.sort! {|x, y| x['name'] <=> y['name'] }
    ret.uniq!
    ret
  end

  # 影響しているプロダクト一覧
  def affect_products
    ret = []
    prds = sidfmd_issue_affprods
    prds.each do |prd|
      temp = prd.attributes
      temp.delete('id')
      temp.delete('sidfmd_issue_id')
      temp.delete('create_datetime')
      temp.delete('update_datetime')
      temp[:display_name] = temp['name']
      ret << temp
    end
    vers = sidfmd_issue_affvers
    vers.each do |ver|
      temp = ver.attributes
      temp.delete('id')
      temp.delete('sidfmd_issue_id')
      temp.delete('create_datetime')
      temp.delete('update_datetime')
      temp[:display_name] = temp['fullname']
      ret << temp
    end
    ret.sort! {|x, y| x[:display_name] <=> y[:display_name] }
    ret.uniq!
    ret
  end

  def affect_lproducts
    ret = []
    host = sidfmd_host
    local_vul = SidfmdLocalVul.find_by(:html_id => self.html_id)
    if host then
      host_lapps = host.sidfmd_host_lapps
      local_vul_lapps = SidfmdLocalVulLapp.joins(:sidfmd_local_vul).where(:sidfmd_local_vuls => { :html_id => self.html_id })
      
      ret += SidfmdLproduct.where(
        :id => local_vul_lapps.select(:sidfmd_lproduct_id)
      )
      .where(
        :id => host_lapps.where("sidfmd_lversion_id IS NULL OR sidfmd_lversion_id = 0").select(:sidfmd_lproduct_id)
      )
      .select(:id, :name)
      .map{|lprod|
        {
          :lproduct_id => lprod.id,
          :name => lprod.name,
          :display_name => lprod.name
        }
      }

      ret += SidfmdLversion.where(
        :id => local_vul_lapps.select(:sidfmd_lversion_id)
      )
      .where(
        :id => host_lapps.where("sidfmd_lversion_id IS NOT NULL AND sidfmd_lversion_id != 0").select(:sidfmd_lversion_id)
      )
      .select(:id, :sidfmd_lproduct_id, :fullname, :version)
      .map{|lver|
        {
          :lproduct_version_id => lver.id,
          :sidfmd_lproduct_id => lver.sidfmd_lproduct_id,
          :fullname => lver.fullname,
          :version => lver.version,
          :display_name => lver.fullname
        }
      }
    end
    ret.sort_by! {|x| x[:display_name] }
    ret.uniq!
    ret
  end

  # 脆弱性一覧のデータを生成する。
  def self.create_all_content(**opts)
    [ :category_id, :role_admin, :corresp_required ].each do |k|
      raise ArgumentError, "must be #{k.to_s}" unless opts.has_key?(k)
    end
    unless opts[:role_admin] then
      [ :user_host_ids, :user_category_ids ].each do |k|
        raise ArgumentError, "must be #{k.to_s}" unless opts.has_key?(k)
      end
    end
    cache_key = '//vuls'
    if opts[:role_admin] then
      cache_key += '/admin/#{opts[:category_id]}'
    else
      cache_key += "/user/#{opts[:user_category_ids].sort.join(',')}/#{opts[:category_id]}"
    end
    opts[:no_cache] ? create_all_content_base(opts) : Rails.cache.fetch(cache_key, { :expires_in => 1.hour, :race_condition_ttl => 1.minute }) { create_all_content_base(opts) }
  end

  # sub method for create_all_content
  def self.create_all_content_base(opts)
    i_htmlids =
      SidfmdIssue.where(
        :sidfmd_host_id => SidfmdHost.where(:id => SidfmdCategoriesHost.distinct.select(:sidfmd_host_id))
                                     .where(:flg_delete => false)
                                     .select(:id)
      )
    if opts[:html_ids] then
      i_htmlids = i_htmlids
        .where(:html_id => opts[:html_ids])
    end
    if opts[:role_admin] then
      if opts[:category_id] > 0 then
        i_htmlids = i_htmlids
          .where(:sidfmd_host_id =>
                 SidfmdCategoriesHost
                   .joins(:sidfmd_host)
                   .where(:sidfmd_hosts => { :flg_delete => false })
                   .where(:sidfmd_category_id => opts[:category_id])
                   .select(:sidfmd_host_id)
                   .distinct)
      end
    else
      i_htmlids = i_htmlids
        .where(:sidfmd_host_id => opts[:user_host_ids])
      if opts[:category_id] > 0 then
        cat = SidfmdCategory.find(opts[:category_id])
        if cat then
          i_htmlids = i_htmlids
            .where(:sidfmd_host_id =>
                   SidfmdCategoriesHost
                     .joins(:sidfmd_host)
                     .where(:sidfmd_hosts => { :flg_delete => false })
                     .where(:sidfmd_category_id => opts[:category_id])
                     .where(:sidfmd_host_id => opts[:user_host_ids])
                     .select(:sidfmd_host_id)
                     .distinct)
        end
      end
    end
    i_htmlids =
      i_htmlids.select(:html_id).group(:html_id)
      .select("(ARRAY_TO_STRING((SELECT ARRAY_AGG(DISTINCT(title)) FROM sidfmd_issues AS tt_sidfmd_issues WHERE sidfmd_issues.html_id=tt_sidfmd_issues.html_id),' / ')) AS title")
      .select("EXISTS(SELECT risk_id FROM sidfmd_issues AS ri_sidfmd_issues WHERE sidfmd_issues.html_id=ri_sidfmd_issues.html_id AND ri_sidfmd_issues.risk_id=4) AS risk_id4")
      .select("EXISTS(SELECT risk_id FROM sidfmd_issues AS ri_sidfmd_issues WHERE sidfmd_issues.html_id=ri_sidfmd_issues.html_id AND ri_sidfmd_issues.risk_id=3) AS risk_id3")
      .select("EXISTS(SELECT risk_id FROM sidfmd_issues AS ri_sidfmd_issues WHERE sidfmd_issues.html_id=ri_sidfmd_issues.html_id AND ri_sidfmd_issues.risk_id=2) AS risk_id2")
      .select("EXISTS(SELECT risk_id FROM sidfmd_issues AS ri_sidfmd_issues WHERE sidfmd_issues.html_id=ri_sidfmd_issues.html_id AND ri_sidfmd_issues.risk_id=1) AS risk_id1")
      .select("EXISTS(SELECT area_id FROM sidfmd_issues AS ai_sidfmd_issues WHERE sidfmd_issues.html_id=ai_sidfmd_issues.html_id AND ai_sidfmd_issues.area_id=3) AS area_id3")
      .select("EXISTS(SELECT area_id FROM sidfmd_issues AS ai_sidfmd_issues WHERE sidfmd_issues.html_id=ai_sidfmd_issues.html_id AND ai_sidfmd_issues.area_id=2) AS area_id2")
      .select("EXISTS(SELECT area_id FROM sidfmd_issues AS ai_sidfmd_issues WHERE sidfmd_issues.html_id=ai_sidfmd_issues.html_id AND ai_sidfmd_issues.area_id=1) AS area_id1")
    flg_short_description = SidfmdQueueVul.column_names.include?('short_description')
    if flg_short_description then
      i_htmlids = i_htmlids.
        select("(
SELECT ds_sidfmd_issues.short_description
FROM sidfmd_issues AS ds_sidfmd_issues
WHERE ds_sidfmd_issues.html_id=sidfmd_issues.html_id
ORDER BY ds_sidfmd_issues.short_description DESC
LIMIT 1
) AS l_short_description")
    end
    filter_hostids = proc {|prefix: nil, p_where: false, p_and: false, s_and: false, id: false|
      ret = ''
      unless opts[:role_admin] then
        if p_where then
          ret << "WHERE "
        elsif p_and then
          ret << "AND "
        end
        if opts[:user_host_ids].size > 0 then
          ret << "#{prefix}." if prefix
          ret << "#{id ? 'id' : 'sidfmd_host_id'} IN (#{opts[:user_host_ids].join(',')})"
        else
          ret << "false"
        end
        ret << " AND " if s_and
      end
      ret
    }
    i_htmlids = i_htmlids
      .select("
(
 SELECT DISTINCT(BOOL_OR(COALESCE(c3_exploitable,c2_exploitable,FALSE)))
 FROM sidfmd_issues AS e_sidfmd_issues
 INNER JOIN issues
  ON e_sidfmd_issues.issue_id=issues.id
 INNER JOIN sidfmd_hosts AS e_sidfmd_hosts
  ON e_sidfmd_hosts.id=e_sidfmd_issues.sidfmd_host_id
 INNER JOIN sidfmd_issue_cves
  ON e_sidfmd_issues.id=sidfmd_issue_cves.sidfmd_issue_id
 WHERE e_sidfmd_hosts.flg_delete IS FALSE AND
       e_sidfmd_issues.html_id=sidfmd_issues.html_id AND
       issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})
       #{filter_hostids.call(prefix: 'e_sidfmd_issues', p_and: true)}
 LIMIT 1
) AS exploitable")
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT DISTINCT(
         CAST(sidfmd_hosts.id AS text) || ':&&&&&:' ||
         ARRAY_TO_STRING(ARRAY(
          SELECT DISTINCT(CAST(sidfmd_categories_hosts.sidfmd_category_id AS text))
          FROM sidfmd_categories_hosts
          WHERE sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id
         ),'!') || ':&&&&&:' ||
         sidfmd_hosts.hostname || ':&&&&&:' ||
         COALESCE(sidfmd_hosts.ext_name,'') || ':&&&&&:' ||
         COALESCE(sidfmd_hosts.ext_name,sidfmd_hosts.hostname,'')
        )
 FROM sidfmd_issues AS h_sidfmd_issues
 INNER JOIN sidfmd_hosts
  ON sidfmd_hosts.id=h_sidfmd_issues.sidfmd_host_id
 WHERE sidfmd_hosts.flg_delete IS FALSE AND
       sidfmd_hosts.id IN (
         SELECT DISTINCT(sidfmd_host_id)
         FROM sidfmd_categories_hosts
         INNER JOIN sidfmd_categories
          ON sidfmd_categories.id=sidfmd_categories_hosts.sidfmd_category_id
         WHERE sidfmd_categories.is_system IS TRUE
       ) AND
       h_sidfmd_issues.html_id=sidfmd_issues.html_id
       #{filter_hostids.call(prefix: 'h_sidfmd_issues', p_and: true)}
),',&&&&&,') AS hosts")
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT 
  CAST(sidfmd_categories_hosts.sidfmd_category_id AS text) || ':&&&&&:' ||
  CAST(sidfmd_categories.is_system AS text) || ':&&&&&:' ||
  sidfmd_categories.name
 FROM sidfmd_issues AS h_sidfmd_issues
 INNER JOIN sidfmd_hosts
  ON sidfmd_hosts.id=h_sidfmd_issues.sidfmd_host_id
 INNER JOIN sidfmd_categories_hosts
  ON sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id
 INNER JOIN sidfmd_categories
  ON sidfmd_categories.id=sidfmd_categories_hosts.sidfmd_category_id
 WHERE sidfmd_hosts.flg_delete IS FALSE AND
       h_sidfmd_issues.html_id=sidfmd_issues.html_id
       #{filter_hostids.call(prefix: 'h_sidfmd_issues', p_and: true)}
), ',&&&&&,') AS categories")
      .select("ARRAY_TO_STRING(ARRAY(

SELECT
 DISTINCT(
  sidfmd_issue_cves.cve_id || '!' ||
  CAST(COALESCE(sidfmd_issue_cves.c2_base_score,-1.0) AS text) || '!' ||
  CAST(COALESCE(sidfmd_issue_cves.c3_base_score,-1.0) AS text) || '!' ||
  COALESCE(sidfmd_issue_cves.c2_vector,'') || '!' ||
  COALESCE(sidfmd_issue_cves.c3_vector,'') || '!' ||
  COALESCE(TO_CHAR(sidfmd_issue_cves.c2_create_datetime,'YYYY/MM/DD HH24:MI:SS'),'') || '!' ||
  COALESCE(TO_CHAR(sidfmd_issue_cves.c2_update_datetime,'YYYY/MM/DD HH24:MI:SS'),'') || '!' ||
  COALESCE(TO_CHAR(sidfmd_issue_cves.c3_create_datetime,'YYYY/MM/DD HH24:MI:SS'),'') || '!' ||
  COALESCE(TO_CHAR(sidfmd_issue_cves.c3_update_datetime,'YYYY/MM/DD HH24:MI:SS'),'')
 )
FROM sidfmd_issue_cves
INNER JOIN sidfmd_issues AS cve_sidfmd_issues
 ON cve_sidfmd_issues.id=sidfmd_issue_cves.sidfmd_issue_id
WHERE cve_sidfmd_issues.html_id=sidfmd_issues.html_id

),',') AS cves")
    .select("
ARRAY_TO_STRING(ARRAY(
 SELECT
  CAST(base_sidfmd_hosts.id AS text)
  || '|' ||
  CAST(issues.status_id AS text)
  || '|' ||
  (CASE WHEN issues.due_date IS NULL THEN '' ELSE TO_CHAR(issues.due_date,'YYYY/MM/DD') END)
  || '|' ||
  TO_CHAR(issues.created_on,'YYYY/mm/dd HH24:MI:SS +09')
  || '|' ||
  CAST((
    SELECT zzz_sidfmd_issues.risk_id
    WHERE zzz_sidfmd_issues.html_id < 0
    UNION ALL
    SELECT sidfmd_issue_sris.risk_id
    FROM sidfmd_issue_sris
    INNER JOIN sidfmd_issues AS zsri_sidfmd_issues
     ON sidfmd_issue_sris.sidfmd_issue_id=zsri_sidfmd_issues.id
    WHERE zsri_sidfmd_issues.sidfmd_host_id=base_sidfmd_hosts.id AND
          zsri_sidfmd_issues.id=zzz_sidfmd_issues.id AND
          sidfmd_issue_sris.category_id=(
           SELECT sidfmd_categories_hosts.sidfmd_category_id
           FROM sidfmd_categories_hosts
           INNER JOIN sidfmd_categories
            ON sidfmd_categories.id=sidfmd_categories_hosts.sidfmd_category_id
           WHERE sidfmd_categories.is_system IS TRUE AND
            sidfmd_categories_hosts.sidfmd_host_id=base_sidfmd_hosts.id
           ORDER BY sidfmd_categories_hosts.sidfmd_category_id
           LIMIT 1
          )
    UNION ALL
    SELECT zzz_sidfmd_issues.risk_id
    LIMIT 1
  ) AS text)
  || '|' ||
  CAST((
   SELECT 0.0
   WHERE zzz_sidfmd_issues.html_id < 0
   UNION ALL
   SELECT COALESCE(MAX(COALESCE(sidfmd_issue_cves.c3_base_score,sidfmd_issue_cves.c2_base_score,0.0)),0.0)
   FROM sidfmd_issue_cves
   INNER JOIN sidfmd_issues AS zcvss_sidfmd_issues
    ON sidfmd_issue_cves.sidfmd_issue_id=zcvss_sidfmd_issues.id
   WHERE zcvss_sidfmd_issues.sidfmd_host_id=base_sidfmd_hosts.id AND
         zcvss_sidfmd_issues.id=zzz_sidfmd_issues.id
   UNION ALL
   SELECT 0.0
   LIMIT 1
  ) AS text)
 FROM sidfmd_hosts AS base_sidfmd_hosts
 INNER JOIN sidfmd_issues AS zzz_sidfmd_issues
  ON zzz_sidfmd_issues.sidfmd_host_id=base_sidfmd_hosts.id
 INNER JOIN issues
  ON zzz_sidfmd_issues.issue_id=issues.id
 WHERE base_sidfmd_hosts.flg_delete IS FALSE AND
       zzz_sidfmd_issues.html_id=sidfmd_issues.html_id AND
       base_sidfmd_hosts.id IN (SELECT sidfmd_host_id FROM sidfmd_categories_hosts)
       #{filter_hostids.call(prefix: 'zzz_sidfmd_issues', p_and: true) }
),',') AS sri_cvss_status_ids_orig")
    .select("
      (
        ARRAY(
          SELECT DISTINCT(
            ARRAY_TO_STRING(
              ARRAY[sidfmd_ctag_id, sidfmd_issues_for_ctags.sidfmd_host_id]
            ,'!')
          )
          FROM sidfmd_issue_ctags
          INNER JOIN sidfmd_issues AS sidfmd_issues_for_ctags ON sidfmd_issues_for_ctags.id = sidfmd_issue_ctags.sidfmd_issue_id
          INNER JOIN sidfmd_ctags ON sidfmd_ctags.id = sidfmd_issue_ctags.sidfmd_ctag_id
          WHERE sidfmd_issues_for_ctags.html_id = sidfmd_issues.html_id#{
            opts[:role_admin] ?
            "" :
            " AND (sidfmd_ctags.sidfmd_category_id IS NULL OR sidfmd_ctags.sidfmd_category_id IN (SELECT unnest(ARRAY[#{(opts[:user_category_ids].any? ? opts[:user_category_ids] : [-1]).join(', ')}])))"
          }
        )
      ) AS ctags /*管理者以外は、カスタムタグのカテゴリがユーザのカテゴリに属するカスタムタグのみ取得する*/
    ")
    ret = []
    # 依存関係に注意
    i_htmlids.each do |i|
      sri_cvss_status_ids =
        i[:sri_cvss_status_ids_orig].split(/,/, -1).collect {|x|
        y = x.split(/\|/, -1)
        {
          :sidfmd_host_id => y[0] ? y[0].to_i : 0,
          :status_id => y[1].to_i,
          :due_date => (y[2] and y[2].size > 0) ? y[2] : nil,
          :created_on => (y[3] and y[3].size > 0) ? DateTime.parse(y[3]) : DateTime.current,
          :sri => y[4] ? y[4].to_i : 0,
          :cvss => (y[5] && y[5].to_f > 0.0) ? y[5].to_f : nil,
        }
      }
 
      item = {
        :sri_cvss_status_ids => sri_cvss_status_ids,
        :html_id => i[:html_id],
        :title => i[:title],
        :softek => {
          :risk_id => if i[:risk_id4]  then
                        4
                      elsif i[:risk_id3] then
                        3
                      elsif i[:risk_id2] then
                        2
                      elsif i[:risk_id1] then
                        1
                      else
                        1
                      end,
          :area_id => if i[:area_id3] then
                        3
                      elsif i[:area_id2] then
                        2
                      elsif i[:area_id1] then
                        1
                      else
                        0
                      end,
        },
        :short_description => flg_short_description ? (i[:l_short_description] || '') : '',
        :exploitable => i[:exploitable],
        :hosts => i[:hosts].split(/,&&&&&,/).collect {|x|
          if /\A(\d+):&&&&&:([0-9!]*):&&&&&:(.*):&&&&&:(.*):&&&&&:(.*)\z/ =~ x then
            id = Regexp.last_match(1).to_i
            hostname = Regexp.last_match(3)
            ext_name = Regexp.last_match(4)
            display_name = Regexp.last_match(5)
            categories = Regexp.last_match(2).split(/!/).collect {|y| y.to_i }
            if ext_name and ext_name.size == 0 then
              ext_name = nil
            end
            if opts[:category_id] <= 0 then
              if opts[:role_admin] then
                { :id => id, :hostname => hostname, :ext_name => ext_name, :display_name => display_name }
              elsif opts[:user_host_ids].include?(id) then
                { :id => id, :hostname => hostname, :ext_name => ext_name, :display_name => display_name }
              else
                nil
              end
            elsif opts[:category_id] > 0 and categories.include?(opts[:category_id]) then
              { :id => id, :hostname => hostname, :ext_name => ext_name, :display_name => display_name }
            else
              nil
            end
          else
            # invalid data
            nil
          end
        }.uniq.compact.sort {|x, y| x[:hostname] <=> y[:hostname] },
        :cves => i[:cves] ? i[:cves].split(/,/, -1).collect {|x|
          if /\A([^!]+)!([^!]+)!([^!]+)!([^!]*)!([^!]*)!([^!]*)!([^!]*)!([^!]*)!([^!]*)\z/ =~ x then
            cve_id = Regexp.last_match(1)
            c2_base_score = Regexp.last_match(2).to_f
            c3_base_score = Regexp.last_match(3).to_f
            c2_vector = Regexp.last_match(4)
            c3_vector = Regexp.last_match(5)
            c2_create_datetime = Regexp.last_match(6)
            c2_update_datetime = Regexp.last_match(7)
            c3_create_datetime = Regexp.last_match(8)
            c3_update_datetime = Regexp.last_match(9)
            { :cve_id => cve_id,
              :c2_base_score => c2_base_score,
              :c3_base_score => c3_base_score,
              :c2_vector => c2_vector,
              :c3_vector => c3_vector,
              :c2_create_datetime => c2_create_datetime,
              :c2_update_datetime => c2_update_datetime,
              :c3_create_datetime => c3_create_datetime,
              :c3_update_datetime => c3_update_datetime, }
          else
            []
          end
        }.compact.uniq : [],
        :ctags => i.ctags.map{|ctag| #SidfmdVulViewは、カスタムタグのカテゴリとホストのカテゴリの関係性は考慮しない
          ctag_info = ctag.split(/!/)
          ctag_id = ctag_info[0].to_i
          issue_host_id = ctag_info[1].to_i
          { :ctag_id => ctag_id, :sidfmd_host_id => issue_host_id }
        }.uniq,
      }
      temp_categories = i[:categories].split(/,&&&&&,/).collect {|x|
        if /\A(\d+):&&&&&:(true|false):&&&&&:(.*)\z/ =~ x then
          id = Regexp.last_match(1).to_i
          is_system = ('true' == Regexp.last_match(2))
          cat_name = Regexp.last_match(3)
          { :id => id, :is_system => is_system, :name => cat_name }
        else
          # invalid data
          nil
        end
      }.uniq.compact.sort {|x, y| x[:id] <=> y[:id] }
      item[:categories] = if opts[:role_admin] then
                            temp_categories
                          else
                            temp_categories.collect {|x|
                              if x[:is_system] then
                                x
                              elsif opts[:user_category_ids].include?(x[:id]) then
                                x
                              else
                                nil
                              end
                            }.compact
                          end
      if item[:categories] then
        item[:categories].sort! {|x, y|
          r = (x[:is_system] ? 100 : 500) <=> (y[:is_system] ? 100 : 500)
          if r == 0 then
            r = x[:name] <=> y[:name]
          end
          r
        }
      end
      update_value_by_scsids(item, opts[:corresp_required])
      ret << item
    end
    ret
  end

  # sri_cvss_status_idsを元に算出した項目の更新を行う。
  # item: SidfmdIssue.create_all_contentで取得したデータの1列
  def self.update_value_by_scsids(item, corresp_required)
    item[:status_ids] = item[:sri_cvss_status_ids].collect {|x| x[:status_id] }
    item[:status_ids] = [] unless item[:status_ids]
    item[:max_sri] = item[:sri_cvss_status_ids].find_all {|x| SidfmdIssue::STATUS_UNFINISHED.include?(x[:status_id]) }.collect {|x| x[:sri] }.compact.max
    item[:max_sri] = 0 unless item[:max_sri]
    item[:due_dates] = item[:sri_cvss_status_ids].collect {|x| x[:due_date] }
    item[:due_dates_unfinished] = item[:sri_cvss_status_ids].find_all {|x| SidfmdIssue::STATUS_UNFINISHED.include?(x[:status_id]) }.collect {|x| x[:due_date] }
    item[:min_status_id] = item[:status_ids].min
    item[:cvss] = {
      :score => item[:sri_cvss_status_ids].collect {|x| x[:cvss] }.compact.max
    }
    item[:cvss][:label] = SidfmdIssueCve.labelling(item[:cvss][:score])
    item[:warning_orig] = {
      :sri => item[:max_sri] ? (item[:max_sri] >= corresp_required[:sri]) : false,
      :cvss => item[:cvss][:score] ? (item[:cvss][:score] >= corresp_required[:cvss]) : true,
    }
    item[:warning] = {
      :sri => SidfmdIssue::STATUS_UNFINISHED.include?(item[:min_status_id]) ? item[:warning_orig][:sri] : false,
      :cvss => SidfmdIssue::STATUS_UNFINISHED.include?(item[:min_status_id]) ? item[:warning_orig][:cvss] : false,
    }
    item[:fixed_warning] = {
      :sri => SidfmdIssue::STATUS_FINISHED.include?(item[:min_status_id]) ? item[:warning_orig][:sri] : false,
      :cvss => SidfmdIssue::STATUS_FINISHED.include?(item[:min_status_id]) ? item[:warning_orig][:cvss] : false,
    }
    item[:not_fixed_warning] = {
      :sri => item[:warning][:sri],
      :cvss => item[:warning][:cvss],
    }
    item[:not_fixed_hosts] = item[:sri_cvss_status_ids].count {|x| SidfmdIssue::STATUS_UNFINISHED.include?(x[:status_id]) }
    item[:created_ons] = item[:sri_cvss_status_ids].collect {|x| x[:created_on] }
    temp_cu = item[:created_ons].compact.uniq
    item[:min_created_on] = { :value => temp_cu.min }
    item[:min_created_on][:display] = item[:min_created_on][:value] ? item[:min_created_on][:value].localtime.strftime('%Y/%m/%d') : '---'
    item[:min_created_on][:elapsed_days] = item[:min_created_on][:value] ? ((Time.current - item[:min_created_on][:value]) / 24.0 / 60.0 / 60.0).to_i : -1
    item[:elapsed_created_ons] = item[:sri_cvss_status_ids].collect {|x| SidfmdIssue::STATUS_UNFINISHED.include?(x[:status_id]) ? x[:created_on] : nil }.compact
    item[:min_elapsed_created_on] = { :value => item[:elapsed_created_ons].compact.uniq.min }
    item[:min_elapsed_created_on][:display] = item[:min_elapsed_created_on][:value] ? ((Time.current - item[:min_elapsed_created_on][:value]) / 24.0 / 60.0 / 60.0).to_i : -1
  end

  # sri_cvss_status_idsを元に算出した項目の更新と、hostids, categoryidsの範囲を
  # こえる値を除去する。
  # dataへの変更を伴うので注意。
  #
  # content: SidfmdIssue.create_all_contentで取得したデータ
  # hostids: sidfmd_host_idの配列
  # categoryids: sidfmd_category_idの配列
  def self.update_values_by_scsids(data, hostids = [], categoryids = [], corresp_required)
    corresp_required = Sidfm.corresp_required unless corresp_required
    syscatids =
      SidfmdCategoriesHost.
      joins(:sidfmd_category).
      where(:sidfmd_host_id => hostids).
      where(:sidfmd_categories => { :is_system => true }).
      map(&:sidfmd_category_id)
    data.each do |row|
      row[:hosts].delete_if {|x| !hostids.include?(x[:id]) }
      row[:categories].delete_if {|x|
        if categoryids then
          (!x[:is_system] && !categoryids.include?(x[:id])) ||
            (x[:is_system] && !syscatids.include?(x[:id]))
        else
          (x[:is_system] && !syscatids.include?(x[:id]))
        end
      }
      prev_count = row[:sri_cvss_status_ids].size
      row[:sri_cvss_status_ids].delete_if {|x| !hostids.include?(x[:sidfmd_host_id]) }
      post_count = row[:sri_cvss_status_ids].size
      if row[:sri_cvss_status_ids].size > 0 and prev_count != post_count then
        update_value_by_scsids(row, corresp_required)
      end
    end
    data.delete_if {|x| x[:sri_cvss_status_ids].size == 0 }
  end

  # 対象となるコンテンツ全てを含む配列を返す。
  # カテゴリ選択のみ、反映させる
  def self.create_content(**opts)
    [ :corresp_required, :html_id ].each do |k|
      raise ArgumentError, "must be #{k.to_s}" unless opts.has_key?(k)
    end
    opts[:category_id] = 0 unless opts.has_key?(:category_id)
    opts[:role_admin] = false unless opts.has_key?(:role_admin)
    unless opts[:role_admin] then
      raise ArgumentError, "must be user_category_ids" unless opts.has_key?(:user_category_ids)
    end
    unless opts[:role_admin] then
      raise ArgumentError, "must be user_host_ids" unless opts.has_key?(:user_host_ids)
    end
    get_content_base(**opts)
  end

  def self.get_content_base(**opts)
    ret = []
    issues = SidfmdIssue
      .enable_hosts
      .joins(:issue)
      .joins(:sidfmd_host)
      .where(:html_id => opts[:html_id])
    if opts[:category_id] > 0 then
      issues = issues
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => opts[:category_id])
                 .select(:sidfmd_host_id))
      unless opts[:role_admin] then
        issues = issues
          .where(
            :sidfmd_host_id => SidfmdCategoriesHost.where(:sidfmd_host_id => opts[:user_host_ids]).distinct.select(:sidfmd_host_id)
          )
      end
    else
      issues = issues.where(
                 :sidfmd_host_id => SidfmdCategoriesHost.distinct.select(:sidfmd_host_id)
               )
      unless opts[:role_admin] then
        issues = issues
          .where(
            :sidfmd_host_id => SidfmdCategoriesHost.where(:sidfmd_host_id => opts[:user_host_ids]).distinct.select(:sidfmd_host_id)
          )
      end
    end
    issues = issues
      .select(:id)
      .select(:html_id)
      .select(:title)
      .select(:sidfmd_host_id)
      .select(:risk_id)
      .select(:area_id)
      .select("sidfmd_hosts.hostname AS hostname")
      .select("sidfmd_hosts.ext_name AS ext_name")
      .select("COALESCE(sidfmd_hosts.ext_name,sidfmd_hosts.hostname,'') AS display_name")
      .select("issues.id AS issue_id")
      .select("issues.status_id AS status_id")
      .select("TO_CHAR(issues.due_date,'YYYY/MM/DD') AS due_date")
      .select("TO_CHAR(issues.created_on, 'YYYY/MM/DD HH24:MI:SS +09') AS created_on")
      .select("(SELECT CAST(sidfmd_categories.id AS text) || ':' || sidfmd_categories.name FROM sidfmd_categories WHERE sidfmd_categories.is_system=TRUE AND sidfmd_categories.id=(SELECT sidfmd_category_id FROM sidfmd_categories_hosts WHERE sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id ORDER BY sidfmd_category_id LIMIT 1)) AS system_category")
      .select("(ARRAY_TO_STRING(ARRAY(SELECT CAST(sidfmd_categories.id AS text) || ':&&&&&:' || sidfmd_categories.name FROM sidfmd_categories WHERE sidfmd_categories.is_system=FALSE AND sidfmd_categories.id IN (SELECT sidfmd_category_id FROM sidfmd_categories_hosts WHERE sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id)),',&&&&&,')) AS no_system_categories")
      .select("(SELECT BOOL_OR(c32_exploitable) FROM (SELECT BOOL_OR(COALESCE(c3_exploitable,c2_exploitable,FALSE)) AS c32_exploitable FROM sidfmd_issue_cves WHERE sidfmd_issue_cves.sidfmd_issue_id=sidfmd_issues.id AND issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})) AS c32_exploitables) AS exploitable")
      .select("(SELECT MAX(COALESCE(c3_base_score,c2_base_score)) FROM sidfmd_issue_cves WHERE sidfmd_issue_cves.sidfmd_issue_id=sidfmd_issues.id) AS cvss_score")
      .select("
(
  SELECT sidfmd_issues.risk_id
  WHERE sidfmd_issues.html_id < 0
  UNION ALL
  SELECT sidfmd_issue_sris.risk_id AS risk_id 
  FROM sidfmd_issue_sris 
  WHERE sidfmd_issue_sris.sidfmd_issue_id=sidfmd_issues.id AND 
        sidfmd_issues.html_id > 0 AND
        sidfmd_issue_sris.category_id=
        (
         SELECT sidfmd_category_id FROM sidfmd_categories_hosts 
         WHERE sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id ORDER BY sidfmd_category_id LIMIT 1
        )
  UNION ALL
  SELECT sidfmd_issues.risk_id
  LIMIT 1
) AS sri")
      .select("(ARRAY_TO_STRING(ARRAY(SELECT CAST(journals.id AS text) || '!' ||  journals.journalized_type || '!' ||  CAST(journals.user_id AS text) || '!' || TO_CHAR(journals.created_on, 'YYYY/mm/dd HH24:MI:SS +09') FROM journals WHERE journals.journalized_id=issues.id AND journals.private_notes=FALSE ORDER BY journals.created_on DESC),',')) AS journals")
      .select("(SELECT TO_CHAR(MIN(issues.created_on),'YYYY/MM/DD HH24:MI:SS +09') FROM sidfmd_issues INNER JOIN issues ON issues.id=sidfmd_issues.issue_id WHERE sidfmd_issues.sidfmd_host_id=sidfmd_hosts.id AND sidfmd_issues.html_id=#{opts[:html_id]} AND issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})) AS elapsed_days")
      .select("ARRAY_TO_STRING(ARRAY(

SELECT
 DISTINCT(
  sidfmd_issue_cves.cve_id || '!' ||
  CAST(COALESCE(sidfmd_issue_cves.c2_base_score,-1.0) AS text) || '!' ||
  CAST(COALESCE(sidfmd_issue_cves.c3_base_score,-1.0) AS text) || '!' ||
  COALESCE(sidfmd_issue_cves.c2_vector,'') || '!' ||
  COALESCE(sidfmd_issue_cves.c3_vector,'') || '!' ||
  COALESCE(TO_CHAR(sidfmd_issue_cves.c2_create_datetime,'YYYY/MM/DD HH24:MI:SS'),'') || '!' ||
  COALESCE(TO_CHAR(sidfmd_issue_cves.c2_update_datetime,'YYYY/MM/DD HH24:MI:SS'),'') || '!' ||
  COALESCE(TO_CHAR(sidfmd_issue_cves.c3_create_datetime,'YYYY/MM/DD HH24:MI:SS'),'') || '!' ||
  COALESCE(TO_CHAR(sidfmd_issue_cves.c3_update_datetime,'YYYY/MM/DD HH24:MI:SS'),'')
 )
FROM sidfmd_issue_cves
WHERE sidfmd_issue_cves.sidfmd_issue_id=sidfmd_issues.id

),',') AS cves")

    issues.each do |issue|
      item = {
        :sidfmd_issue_id => issue[:id],
        :html_id => issue[:html_id],
        :title => issue[:title],
        :sidfmd_host_id => issue[:sidfmd_host_id],
        :hostname => issue[:hostname],
        :ext_name => issue[:ext_name],
        :display_name => issue[:display_name],
        :os => issue.sidfmd_host.get_os,
        :softek => { :risk_id => issue[:risk_id], :area_id => issue[:area_id] },
        :issue_id => issue[:issue_id],
        :status_id => issue[:status_id],
        :due_date => issue[:due_date],
        :created_on => { :value => DateTime.parse(issue[:created_on]) },
        :system_category => if /\A(\d+):(.*)\z/ =~ issue[:system_category] then
                              cat_id = Regexp.last_match(1).to_i
                              cat_name = Regexp.last_match(2)
                              { :id => cat_id, :name => cat_name }
                            else
                              nil
                            end,
        :no_system_categories => issue[:no_system_categories].split(/,&&&&&,/, -1).collect {|x|
          if /\A(\d+):&&&&&:(.*)\z/ =~ x then
            cat_id = Regexp.last_match(1).to_i
            cat_name = Regexp.last_match(2)
            if opts[:role_admin] or (!opts[:role_admin] and opts[:user_category_ids].include?(cat_id)) then
              { :id => cat_id, :name => cat_name }
            else
              nil
            end
          else
            nil
          end
        }.compact,
        :exploitable => issue[:exploitable],
        :cvss => { :score => issue[:cvss_score] },
        :sri => issue[:sri] ? issue[:sri] : 0,
        :journals => issue[:journals].split(/,/).collect {|x|
          if /\A(\d+)!([^:]*)!(.*)!(.*)\z/ =~ x then
            journal_id = Regexp.last_match(1).to_i
            journal_type = Regexp.last_match(2)
            user_id = Regexp.last_match(3).to_i
            created_on = Regexp.last_match(4)
            created_on = DateTime.parse(created_on) if created_on
            { :id => journal_id,
              :type => journal_type,
              :user_id => user_id,
              :created_on => created_on, }
          else
            nil
          end
        }.compact,
        :cves => issue[:cves] ? issue[:cves].split(/,/, -1).collect {|x|
          if /\A([^!]+)!([^!]+)!([^!]+)!([^!]*)!([^!]*)!([^!]*)!([^!]*)!([^!]*)!([^!]*)\z/ =~ x then
            cve_id = Regexp.last_match(1)
            c2_base_score = Regexp.last_match(2).to_f
            c3_base_score = Regexp.last_match(3).to_f
            c2_vector = Regexp.last_match(4)
            c3_vector = Regexp.last_match(5)
            c2_create_datetime = Regexp.last_match(6)
            c2_update_datetime = Regexp.last_match(7)
            c3_create_datetime = Regexp.last_match(8)
            c3_update_datetime = Regexp.last_match(9)
            { :cve_id => cve_id,
              :c2_base_score => c2_base_score,
              :c3_base_score => c3_base_score,
              :c2_vector => c2_vector,
              :c3_vector => c3_vector,
              :c2_create_datetime => c2_create_datetime,
              :c2_update_datetime => c2_update_datetime,
              :c3_create_datetime => c3_create_datetime,
              :c3_update_datetime => c3_update_datetime, }
          else
            []
          end
        }.compact.uniq : [],
      }
      item[:elapsed_days] = '---'
      if issue[:elapsed_days] and issue[:elapsed_days].size > 1 then
        item[:elapsed_days] = ((Time.current - DateTime.parse(issue[:elapsed_days]))/24.0/60.0/60.0).to_i
      end
      item[:created_on][:display] = item[:created_on][:value] ? item[:created_on][:value].localtime.strftime('%Y/%m/%d') : '---'
      item[:created_on][:elapsed_days] = item[:created_on][:value] ? ((Time.current - item[:created_on][:value]) / 24.0 / 60.0 / 60.0).to_i : -1
      if item[:cvss][:score] then
        item[:cvss][:label] = SidfmdIssueCve.labelling(item[:cvss][:score])
      else
        item[:cvss][:label] = :notreg
      end
      if opts[:html_id] < 0 then
        item[:sri] = issue[:risk_id]
      end
      item[:warning_orig] = {
        :sri => item[:sri] >= opts[:corresp_required][:sri],
        :cvss => item[:cvss][:score] ? (item[:cvss][:score] >= opts[:corresp_required][:cvss]) : true,
      }
      item[:warning] = {
        :sri => SidfmdIssue::STATUS_UNFINISHED.include?(item[:status_id]) ? item[:warning_orig][:sri] : false,
        :cvss => SidfmdIssue::STATUS_UNFINISHED.include?(item[:status_id]) ? item[:warning_orig][:cvss] : false,
      }
      item[:fixed_warning] = {
        :sri => (SidfmdIssue::STATUS_FINISHED.include?(item[:status_id]) & item[:warning_orig][:sri]),
        :cvss => (SidfmdIssue::STATUS_FINISHED.include?(item[:status_id]) & item[:warning_orig][:cvss]),
      }
      ret << item
    end
    ret
  end

  # 履歴一覧の元データ(脆弱性とホストとユーザに関連するもののみ)
  def self.create_journals_content(role_admin:, user_host_ids:)
    content = self.joins(:issue)
                  .joins(%| INNER JOIN journals ON journals.journalized_id = issues.id |)
                  .joins(%| LEFT OUTER JOIN journal_details ON journals.id = journal_details.journal_id |)
                  .joins(%| INNER JOIN sidfmd_hosts ON sidfmd_hosts.id = sidfmd_issues.sidfmd_host_id |)
                  .joins(%| INNER JOIN users ON journals.user_id = users.id |)
                  .joins(%| LEFT OUTER JOIN email_addresses ON users.id = email_addresses.user_id |)
                  .where(:sidfmd_hosts => { :flg_delete => false })
                  .select(
                    :id,
                    'journals.id AS journal_id',
                    'journals.created_on AS journal_created_on',
                    'users.id AS user_id',
                    'users.login AS login_name',
                    'email_addresses.address AS email_address',
                    " CASE 
                      WHEN users.type = 'AnonymousUser' THEN
                        ''
                      ELSE
                        (users.lastname || ' ' || users.firstname)
                      END
                      AS user_fullname
                    ",
                    %| CASE 
                       WHEN users.type = 'AnonymousUser' THEN
                         true
                       ELSE
                         false
                       END
                       AS is_anonymous
                    |,
                    %| CASE journal_details.prop_key 
                       WHEN 'status_id' THEN 'ステータス変更' 
                       WHEN 'due_date' THEN '対策予定日の変更'
                       ELSE NULL
                       END
                       AS operation
                    |,
                    'journal_details.old_value AS old_value', #status_idの場合もtext型なので注意
                    'journal_details.value AS current_value',
                    'journals.notes AS comment',
                    'sidfmd_hosts.hostname AS hostname',
                    'sidfmd_hosts.ext_name AS ext_name',
                    %|
                      CASE 
                      WHEN sidfmd_hosts.ext_name IS NULL THEN sidfmd_hosts.hostname
                      ELSE sidfmd_hosts.ext_name
                      END
                      AS display_name
                    |,
                    'sidfmd_hosts.id AS host_id',
                    'sidfmd_issues.html_id AS html_id',
                    'sidfmd_issues.title AS vul_title',
                    'sidfmd_issues.short_description AS vul_short_description',
                    'issues.status_id AS status_id',
                    'issues.due_date AS due_date'
                  )
    unless role_admin then
      content = content.where(:sidfmd_issues => { :sidfmd_host_id => user_host_ids })
    end
    content
  end

end
