/* coding: utf-8 */

//BootStrapTableのtdのスタイル。画面ごとに関数を作って設定する
function BootStrapTableCellStyle(value, row, index, field) {
    switch (field){
        case 'chkbox':
            return { classes: 'chk-box' };
        case 'cve_id':
            return { classes: 'cve-id' };
        case 'cvss':
            return { classes: 'cvss' };
        case 'vuls':
            return { classes: 'vuls' };
        case 'hosts':
            return { classes: 'hosts' };
        //case 'sidfm_ids':
        //    return { classes: 'sidfm-ids' };
        default:
            return { classes: '' };
    }
}

$(function() {
    activate_sidebar_item('sidebar-item-cves-manip');

    const summaryTableId = 'cves-summary-table';
    const summaryTableElm = $('#' + summaryTableId);
    const pageName = 'cves';

    //脆弱性表示
    const getCveVulsPath = $('#get-cve-vuls-path').val();
    const getCveHostsPath = $('#get-cve-hosts-path').val();

    const createVulTitle = (vul) => { //html_idはマイナスでないように注意
        return `[${vul.html_id}]${vul.title}`
    };

    const enableDisplayCveVuls = () => {
        $(`#${summaryTableId} .show-cve-vuls`).on('click', function(){
            const button = $(this);
            const buttonParent = button.parent();
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return;
                if( button.attr('isdata') === 'true' ){
                    return false;
                }
                button.attr('isdata', 'true');
                $.post(
                    getCveVulsPath,
                    {
                        cve_ids: JSON.stringify([ button.attr('data-cveid') ]),
                        except_html_ids: JSON.stringify([ button.attr('data-excepthtmlid') ])
                    },
                    function(vuls){
                        if( vuls.length === 0 ){
                            alert('脆弱性が削除されているため、 画面をリロードします。')
                            $('#pages_checked').val('[]');
                            $('#pages_all_checked').val('false');
                            $('#pages_form').submit();
                        }
                        const targetList = buttonParent.find('[data-itemlist]');
                        buttonParent.find('[data-itemsize]').attr('data-itemsize', vuls.length).text(vuls.length);
                        buttonParent.find('[data-displaytext]').attr('data-displaytext', createVulTitle(vuls[0]));
                        targetList.html('');
                        const appendElm = function(htmlId, vulTitle, vulShortDescription){
                            const elm = $('<div>').attr({
                                'class': 'host-list-item list-group-item text-truncate g-max-width-12em'
                            }).append(
                                $('<span>').attr({
                                    'data-toggle': 'tooltip',
                                    'data-placement': 'right',
                                    'data-boundary': 'window',
                                    'data-html': 'true',
                                    'title': `<div>${escape_html_special_chars(vulTitle)}</div>${vulShortDescription ? `<div class="vul-tooltip-short-description-area">${escape_html_special_chars(vulShortDescription)}</div>` : ""}`,
                                }).append(
                                    $('<a>').attr({
                                        'href': url_vulnerabilities_detail(htmlId)
                                    }).text(vulTitle)
                                )
                            );
                            targetList.append(elm);
                            targetList.find('[data-toggle=tooltip]').tooltip();
                        };
                        vuls.slice(0, 15).forEach(function(vul){//すぐに表示させるために先に15件要素を作成
                            appendElm(vul.html_id, createVulTitle(vul), vul.short_description);
                        });
                        button.attr({
                            'data-toggle': 'dropdown',
                            'data-boundary': 'window',
                            'aria-haspopup': 'true',
                            'aria-expanded': 'false',
                            'data-flip': 'false',
                        }).dropdown();
                        if( !targetList.is(':visible') ){ //クリックで処理しないとおかしくなる
                            button.click();
                        }
                        button.dropdown('update');
                        setTimeout(function(){
                            vuls.slice(15).forEach(function(vul){
                                appendElm(vul.html_id, createVulTitle(vul), vul.short_description);//残りを作成    
                            });
                        });
                    }
                );
            })();
        });
    };

    //ホスト表示
    const enableDisplayCveHosts = () => {
        $(`#${summaryTableId} .show-cve-hosts`).on('click', function(){
            const button = $(this);
            const buttonParent = button.parent();
            (async () => {
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
                if( roleOrCategoryError ) return;
                if( button.attr('isdata') === 'true' ){
                    return false;
                }
                button.attr('isdata', 'true');
                $.post(getCveHostsPath, { cve_ids: JSON.stringify([ button.attr('data-cveid') ]) }, function(hosts){
                    if( hosts.length === 0 ){
                        alert('ホストが削除されているため、 画面をリロードします。')
                        $('#pages_checked').val('[]');
                        $('#pages_all_checked').val('false');
                        $('#pages_form').submit();
                    }
                    const targetList = buttonParent.find('[data-itemlist]');
                    buttonParent.find('[data-itemsize]').attr('data-itemsize', hosts.length).text(hosts.length);
                    buttonParent.find('[data-displaytext]').attr('data-displaytext', createVulTitle(hosts[0]));
                    targetList.html('');
                    const appendElm = function(host){
                        const elm = $('<div>').attr({
                            'class': 'host-list-item list-group-item text-truncate g-max-width-12em'
                        }).append(
                            $('<span>').attr({
                                'data-toggle': 'tooltip',
                                'data-placement': 'right',
                                'data-boundary': 'window',
                                'data-html': 'true',
                                'title': 'ホスト名: ' + host['hostname'] + '<br/>表示名: ' + (host['ext_name'] ? host['ext_name'] : '(未設定)'),
                            }).append(
                                $('<a>').attr({
                                    'href': url_host_detail(host['sidfmd_host_id'])
                                }).text(host['display_name'])
                            )
                        );
                        targetList.append(elm);
                        targetList.find('[data-toggle=tooltip]').tooltip();
                    };
                    hosts.slice(0, 15).forEach(function(host){//すぐに表示させるために先に15件要素を作成
                        appendElm(host);
                    });
                    button.attr({
                        'data-toggle': 'dropdown',
                        'data-boundary': 'window',
                        'aria-haspopup': 'true',
                        'aria-expanded': 'false',
                        'data-flip': 'false',
                    }).dropdown();
                    if( !targetList.is(':visible') ){ //クリックで処理しないとおかしくなる
                        button.click();
                    }
                    button.dropdown('update');
                    setTimeout(function(){
                        hosts.slice(15).forEach(function(host){
                            appendElm(host);//残りを作成    
                        });
                    });
                });
            })();
        });
    };

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
    const bindSummaryTableRowEvents = () => {
        enable_bootstrap_actions('#' + summaryTableId);
        enable_dropdown_balloon(summaryTableId);
        enableDisplayCveVuls();
        enableDisplayCveHosts();
    };

    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all-cve-ids',
        chkboxSelector: `#${summaryTableId} .cveSelect`,
        allChkboxSelector: `#${summaryTableId} .allCveSelect`,
        ChkboxValueType: 'string',
        //bulkBtnSelector: '.action-btn',
    });

    set_style_for_ActionArea_on_filter_dropdown_event();
    sticky_control('SummaryTableButtonsArea', pageName);
    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) });

    //検索
    const summaryTableFilter = new SummaryTableFilter({dynamicSummaryTable});
   
    summaryTableFilter.enableFilterText(
       '#filter-cveids',
       '#filter-cveids-reset',
       { fieldSelector: '#filter-cveids-input' },
       'cve_ids'
    ); //CVE ID検索

    summaryTableFilter.enableFilterCvss('#exec-filter-cvsses', '#reset-filter-cvsses'); //CVSS検索
    summaryTableFilter.enableFilterCvssVector(
        '#filter-cve-vector-btn',
        '#execute-filter-cvss-vector',
        '#reset-filter-cvss-vector'
    ); //CVSS指標検索
    summaryTableFilter.enableFilterText(
       '#filter-vuls',
       '#filter-vuls-reset',
       { fieldSelector: '#filter-vuls-input' },
       'vuls'
    ); //脆弱性検索
    summaryTableFilter.enableFilterText(
       '#filter-hosts',
       '#filter-hosts-reset',
       { fieldSelector: '#filter-hosts-input' },
       'hosts'
    ); //ホスト検索

    summaryTableFilter.enableResetAllFilters('#filter-clear'); //全リセット

    //JSON/TSV出力
    $('#download-files-btn').on('click', function(){
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction();
            if( roleOrCategoryError ) return;
            var path = $('#get-download-max-size-path').val();
            $.post(path, {'target': 'cve_json_tsv'}, function(data){
                $('#json-tsv-max-download-size').text(data.max_size);
                $('#modalDownload').modal('show');
            });
        })();
    });
    $('#exec-download').on('click', function(){
        json_tsv_summary_table_progress_download($('#download_select').val(), $('#read-download-data-path').val(), { dynamicSummaryTable });
    });
});
