/* coding: utf-8 */
/* Copyright (C) 2019 SofTek Systems, Inc. */

function BootStrapTableCellStyle(value, row, index, field) {
    switch (field){
        case 'chkbox':
            return { classes: 'chk-box' };
        case 'login':
            return { classes: 'text-left' };
        case 'name':
            return { classes: 'text-left' };
        case 'authentication':
            return { classes: 'text-nowrap' };
        case 'authority':
            return { classes: 'authority' };
        case 'categories':
            return { classes: 'categories' };
        case 'category_groups':
            return { classes: 'category-groups' };
        case 'last_login_on':
            return { classes: 'last_login_on' };
        case 'mail_notification':
            return { classes: 'mail_notification' };
        case 'lock':
            return { classes: 'lock' };
        case 'operation':
            return { classes: 'operation' };
        default:
            return { classes: '' };
    }
}

function BootStrapTableRowStyle(row, index){
    return {
        classes: 'list_alternative users_table_row'
    }
}

$(function() {
    if( $.cookie('import_err') ){
        $('html,body').animate({ scrollTop: 0 }, '1');
    }
    $.removeCookie('import_err');

    const summaryTableId = 'users-summary-table';
    const summaryTableElm = $('#' + summaryTableId);   
    const pageName = 'users';
    const catGroups = JSON.parse($('#all-cat-groups').val());

    //ユーザ削除
    const checkApikeyRelatedUsers = ({deleteUserIds = [], userRoleIds = {}}) =>{
        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'POST',
                url: $('#api_get_vm_apikey_related_users_path').val(),
                data: { user_role_ids: JSON.stringify(userRoleIds), delete_user_ids: JSON.stringify(deleteUserIds), }
            }).done(function(ApikeyRelatedUsers){
                resolve(ApikeyRelatedUsers);
            }).fail(function(){
                alert('エラーが発生したため、変更に失敗しました。');
                $('#pages_form').submit();
                reject(false);
            });
        });
    };

    //ユーザ削除時に関連するルールをチェック
    const enableDeleteUser = () => {
        $(`#${summaryTableId} [data-deluser]`).on('click', function(){
            (async () => {
                $('body').append(create_loading_window());
                var button = $(this);
                var path = $('#sidfm_internal_api_check_related_rules_path').val();
                var del_user_id = button.attr('data-deluser');
                var submit_target = '[data-formdel="user_' + del_user_id + '"]';
                var form = $(submit_target);
                const paramsData = { id: form.find('[name="formdel[id]"]').val() };
                const actionPath = form.attr('action');
                $('#submit-related-rules-item-form').attr({
                    'data-params': JSON.stringify(paramsData),
                    'data-path': actionPath
                });
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                    additionalAlertMessage: '削除に失敗しました。'
                });
                if( roleOrCategoryError ) return;
                const apikeyRelatedUsers = await checkApikeyRelatedUsers({deleteUserIds: [del_user_id]});
                if( apikeyRelatedUsers === false ) return;
                var confirmed = false;
                if( apikeyRelatedUsers.delete_apikey_users.length ){
                    let conf = confirm('削除を実行すると、このユーザが設定されている全てのSIDfmVM APIキーが削除されます。よろしいですか?');
                    if( !conf ){
                        $('#checking-window').remove();
                        return;
                    }
                    confirmed = true;
                }
                $.ajax({
                    url: path,
                    type: 'POST',
                    data: {'page_name': 'user', 'delete_id': del_user_id, 'action_idx': 'delete'},
                }).done(function(data){
                    if( data['is_unchangeable_user'] ){
                        $('#checking-window').find('.progress-wait-area-label').text('');
                        setTimeout(function(){
                            alert(
                                'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、ユーザを削除することができません。\n' +
                                '※ユーザの削除がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
                            );
                            $('#checking-window').remove();
                        }, 100);
                        return false;
                    }else{
                        if(Object.keys(data['rule_messages']).length > 0){
                            $('#checking-window').remove();
                            display_related_rules(data);
                        }else{
                            var conf = confirmed ? true : confirm(button.attr('data-conftext'));
                            if(!conf){
                                $('#checking-window').remove();
                                return false;
                            }else{
                                dynamicSummaryTable.executeProcess({
                                    paramsData,
                                    path: actionPath,
                                    additionalAlertMessage: '削除に失敗しました。'
                                });
                                return false;
                                //form.submit();
                            }
                        }
                    }
                });
            })();
        });
    };

    const enableFilterCategoryLinks = () => {
        //カテゴリ名を押下した時のカテゴリ検索
        summaryTableFilter.enableFilterCategoryLink('[data-search="category"]', 'search_category', 'single');
    };

    const enableFilterCatGroupLinks = () => {
        //カテゴリグループを押下した時のグループ検索
        summaryTableFilter.enableFilterCatGroupLink('[data-search="cat_group"]', 'search_group');
    };

    //カテゴリグループ更新
    const selectedCatGroupsElm = $('#selected-cat-groups');
    const unselectedCatGroupsElm = $('#unselected-cat-groups');
    const selectedCatGroupsIdsElm = $('#selected-user-cat-group-ids');
    const updateCatGroupsUserIdElm = $('#cat-groups-userid');

    const createCatGroupOption = (group) => {
        return $('<option>').attr({
            'data-toggle': 'tooltip',
            'data-title': 'id: ' + group.id,
            'data-placement': 'right',
            'data-boundary': 'window'
        }).val(group.id)
          .text(group.name);
    };

    const setCatGroupsModal = (modalTitle, groupIds, userId) => {
        selectedCatGroupsElm.html('');
        unselectedCatGroupsElm.html('');
        selectedCatGroupsIdsElm.val(JSON.stringify(groupIds));
        updateCatGroupsUserIdElm.val(userId);
        $('#catGroupsModalLabel').text(modalTitle);
        const currentGrpElm = [];
        const notGrpElm = [];
        const grpHash = {};
        groupIds.forEach(grpId => grpHash[grpId] = true);
        catGroups.forEach(grp => {
            if( grpHash[grp.id] ){
                currentGrpElm.push(createCatGroupOption(grp));
            }else{
                notGrpElm.push(createCatGroupOption(grp));
            }
        });
        selectedCatGroupsElm.append(currentGrpElm);
        unselectedCatGroupsElm.append(notGrpElm);
        selectedCatGroupsElm.find('[data-toggle="tooltip"]').tooltip();
        unselectedCatGroupsElm.find('[data-toggle="tooltip"]').tooltip();
    }

    $('#catGroupsModal').on('hide.bs.modal', function(){
        selectedCatGroupsElm.scrollTop(0);
        unselectedCatGroupsElm.scrollTop(0);
    });

    $('#select-cat-group').on('click', function(){
        const ids = JSON.parse(selectedCatGroupsIdsElm.val());
        const selectedCatGroupOptions = unselectedCatGroupsElm.find('option:selected');
        selectedCatGroupOptions.each((idx, elm) => {
            ids.push(parseInt($(elm).val()));
        });
        selectedCatGroupsElm.append(selectedCatGroupOptions);
        selectedCatGroupsIdsElm.val(JSON.stringify(ids));
    });

    $('#unselect-cat-group').on('click', function(){
        let ids = JSON.parse(selectedCatGroupsIdsElm.val());
        const removeIds = [];
        const selectedCatGroupOptions = selectedCatGroupsElm.find('option:selected');
        selectedCatGroupOptions.each((idx, elm) => {
            removeIds.push(parseInt($(elm).val()));
        });
        ids = ids.filter(id => !removeIds.includes(id));
        unselectedCatGroupsElm.append(selectedCatGroupOptions);
        selectedCatGroupsIdsElm.val(JSON.stringify(ids));
    });

    $('#exec-update-cat-groups').on('click', function(){
        const button = $(this);
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '更新に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            button.addClass('pointer-events-none');
            const close = () => {
                button.removeClass('pointer-events-none');
                $('#checking-window').remove();
            };
            //$('body').append(create_loading_window());
            const groupIds = JSON.parse(selectedCatGroupsIdsElm.val());
            const paramsData = {
                user_id: updateCatGroupsUserIdElm.val(),
                group_ids: JSON.stringify(groupIds)
            };
            const isAdmin = $('#is-admin-usr').val() == 'true';
            
            const path = $('#sidfm_internal_api_check_related_rules_path').val();
            const actionPath = $('#update-cat-groups-path').val();
            $('#submit-related-rules-item-form').attr({
                'data-params': JSON.stringify(paramsData),
                'data-path': actionPath
            });
            let isOutOfCategory = false;
            if ( !isAdmin && !groupIds.length ) {
                const usersCategoryIds = await getUsersCategories([paramsData.user_id], 'category_ids_without_group_categories');
                isOutOfCategory = (!usersCategoryIds[paramsData.user_id] || !usersCategoryIds[paramsData.user_id].length);
            }
            $.ajax({
                url: path,
                type: 'POST',
                data: {
                    'page_name': 'user',
                    'action_idx': 'cat_group_change',
                    'user_cat_group_ids': JSON.stringify( { [paramsData.user_id] : groupIds })
                },
            }).done(function(data){
                if( data['is_unchangeable_user'] ){
                    //$('#checking-window').find('.progress-wait-area-label').text('');
                    setTimeout(function(){
                        alert(
                            'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、カテゴリグループの変更をすることができません。\n' +
                            '※変更がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
                        );
                        close();
                    },100);
                    return false;
                }else{
                    const exec = () => {
                        if(Object.keys(data['rule_messages']).length > 0){
                            close();
                            display_related_rules(data);
                        }else{
                            dynamicSummaryTable.executeProcess({
                                paramsData,
                                path: actionPath,
                                additionalAlertMessage: '更新に失敗しました。'
                            });
                            close();
                        }
                    };
                    if( isOutOfCategory ){
                        /*const selfUserid = parseInt($('#self_user_id').val());
                        if( paramsData.user_id == selfUserid ){
                            alert('ログインユーザの変更はできません。');
                            close();
                            return false;
                        }*/
                        let conf = confirm('このままカテゴリグループを変更すると、更新対象のユーザがページに表示されなくなります。\nページに表示されないユーザは操作できません。\n更新してもよろしいですか?');
                        if( !conf ){
                            close();
                            return false;
                        }
                        exec();
                    }else{
                        exec();
                    }
                }
            });
        })();
    });

    const enableUpdateCatGroups = () => {
        $('#' + summaryTableId + ' [data-modaltarget="#catGroupsModal"]').on('click', function(){
            const button = $(this);
            const currentGrpIds = JSON.parse(button.attr('data-groupids'));
            setCatGroupsModal(
                'カテゴリグループ編集 - ' + button.attr('data-login'),
                currentGrpIds,
                button.attr('data-userid')
            );
            $('#catGroupsModal').modal('show');
        });
    }

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
     const bindSummaryTableRowEvents = () => {
        enable_bootstrap_actions('#' + summaryTableId);
        enable_dropdown_balloon(summaryTableId);
        enableDeleteUser();
        enableFilterCategoryLinks();
        enableFilterCatGroupLinks();
        enableUpdateCatGroups();
    }

    set_style_for_ActionArea_on_filter_dropdown_event();

    //サマリー表の表示データの変更に合わせて変更
    const modifyPageContent = (data) => {
        $('#users_role_names').val(JSON.stringify(data.users_role_names));
        $('#userids_names').val(JSON.stringify(data.userids_names));
    };

    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all_userids',
        chkboxSelector: `#${summaryTableId} .userSelect`,
        allChkboxSelector: `#${summaryTableId} .allUserSelect`,
        bulkBtnSelector: '.action-btn',
        modifyPageContent
    });

    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) });   
 
    //検索処理
    const summaryTableFilter = new SummaryTableFilter({dynamicSummaryTable});
    summaryTableFilter.enableFilterChangeLink('.filter-change', []); //認証方式/権限名/メール設定/ロック検索
    summaryTableFilter.enableFilterChangeLink('.category-change'); //カテゴリ検索
    summaryTableFilter.enableFilterChangeLink('.group-filter-change'); //カテゴリグループ検索
    summaryTableFilter.enableFilterText(
       '#filter_login',
       '#filter_login_reset',
       { fieldSelector: '#filters_search_login' },
       'search_login'
    ); //ログイン名検索*/
   
    summaryTableFilter.enableFilterText(
       '#filter_name',
       '#filter_name_reset',
       { fieldSelector: '#search_name_keyword' },
       'search_name'
    ); //名前検索*/

    summaryTableFilter.enableResetAllFilters('#filter-clear'); //全リセット

    //最終ログイン日時検索
    $('#filter-buttons-area [data-filter="search_last_login_on"][data-toggle="dropdown"]').parent().on('show.bs.dropdown', function(){
        const filters = JSON.parse($('#pages_filters').val());
        const values = filters['search_last_login_on'];
        if( values ){
            if( values[0] === 'not-login-users' ){
                $('#search-not-login-users').prop('checked', true);
                $('#filters_search_last_login_on_begin, #filters_search_last_login_on_end').val('');
            }else{
                $('#search-not-login-users').prop('checked', false);
                $('#filters_search_last_login_on_begin').val(values[0] ? values[0] : '');
                $('#filters_search_last_login_on_end').val(values[1] ? values[1] : '');
            }
        }else{
            $('#search-not-login-users').prop('checked', false);
            $('#filters_search_last_login_on_begin, #filters_search_last_login_on_end').val('');
        }
    });

    $('#filter-last-login-on').on('click', function(){
        const filters = JSON.parse($('#pages_filters').val());
        let kwd = '';
        if( $('#search-not-login-users').prop('checked') ){
            kwd = [ 'not-login-users' ];
        }else{
            const begin_date = $('#filters_search_last_login_on_begin').val();
            const end_date = $('#filters_search_last_login_on_end').val();
            if( begin_date.length > 0 || end_date.length > 0 ){
                kwd = [ begin_date, end_date ];
            }else{
                kwd = [];
            }
        }
        filters['search_last_login_on'] = kwd;
        summaryTableFilter.executeFilter(filters, 'search_last_login_on');
    });

    $('#reset-filter-last-login-on').on('click', function(){
        const filters = JSON.parse($('#pages_filters').val());
        filters['search_last_login_on'] = [];
        summaryTableFilter.executeFilter(filters, 'search_last_login_on');
        $('#filters_search_last_login_on_begin, #filters_search_last_login_on_end').val('');
    });

    //プリセットでの最終ログイン日時の設定
    $('.filter-last-login-on-preset').on('click', function(e){
        e.stopPropagation();
        var begin_date = $(this).attr('data-begindate');
        var end_date = $(this).attr('data-enddate');
        $('#filters_search_last_login_on_begin').val(begin_date ? begin_date : '');
        $('#filters_search_last_login_on_end').val(end_date ? end_date : '');
    });

    function toggle_search_last_login_on_items(){
        var selectors = [ 
          '#filters_search_last_login_on_begin',
          '#filters_search_last_login_on_end',
          '.filter-last-login-on-preset'
        ].join(', ');
        if( $('#search-not-login-users').prop('checked') ){
            $(selectors).prop('disabled', true);
        }else{
            $(selectors).prop('disabled', false);
        }
    }
    $('#search-not-login-users').on('change', function(){
        toggle_search_last_login_on_items();
    });

    $('#search_last_login_on_btn').on('show.bs.dropdown', function(){
        toggle_search_last_login_on_items();
    });

    function createCategoryOptionElm(id, name, desc){
        return $('<option>').attr({
            'value': id,
            'class': 'text-truncate g-max-width-21em',
            'data-toggle': 'tooltip',
            'data-title': desc,
            'data-placement': 'right',
            'data-boundary': 'window'
        }).text(name)
    }
    
    //モーダルでのカテゴリ編集処理
    $('#categoriesModal').on('show.bs.modal', function(event) {
        var button = $(event.relatedTarget);
        var title = button.data('title');
        var userid = parseInt(button.data('userid'));
        $(this).find('.modal-title').text(title);
        $('#categories_userid').val(userid);
        var cats = JSON.parse($('#all-current-user-cats').val());
        var user_cats = button.data('categoryids');
        //set_paginator_parameter("#cat_per_page", "#cat_filters", "#cat_offset", "#cat_order");
        $('#selected_categories > option').remove();
        $('#unselected_categories > option').remove();
        const user_roles = JSON.parse($('#users_role_names').val());
        const desc = $('#categoriesModal-desc-of-admin-user');
        if( user_roles[userid] === 'SIDfmVM管理者' ){
            desc.show();
        }else{
            desc.hide();
        }
        var cat_names = Object.values(cats).sort(function(a, b){ 
            if(a.toLowerCase() > b.toLowerCase()){
                return 1;
            }else{
                return -1;
            }
        });
        var categories_desc = JSON.parse($('#categories-desc').val());
        for(var i = 0; i < cat_names.length; i++){
            var cat_name = cat_names[i];
            for(cat_id in cats){
                var t_cat_name = cats[cat_id];
                if(t_cat_name === cat_name){
                    $($.inArray(parseInt(cat_id), user_cats) != -1 ? '#selected_categories' : '#unselected_categories')
                        .append( createCategoryOptionElm(cat_id, cat_name, categories_desc[cat_id]) );
                }
            }
        }
        $('#selected_category_ids').val(JSON.stringify(user_cats));
        $('#selected_categories, #unselected_categories').find('[data-toggle="tooltip"]').tooltip();
        return true;
    });
      
    $('#select_category').on('click', function() {
        move_selected_items('#selected_category_ids',
                            '#unselected_categories',
                            '#selected_categories', false);
       $('#selected_categories option').tooltip();
    });
    
    $('#unselect_category').on('click', function() {
        move_selected_items('#selected_category_ids',
                            '#selected_categories',
                            '#unselected_categories', true);
        $('#unselected_categories option').tooltip();
    });

    $('#close-categoriesModal-desc-of-admin-user').on('click', function(){
        $('#categoriesModal-desc-of-admin-user').hide();
    });

    //変更できないユーザを表示
    //メッセージは今のところカテゴリ一括変更の時のみ表示する。
    //削除時とカテゴリ変更はアラートを出す。
    //インポートの時の処理は、インポートのファイルで行う。
    function display_unchangeable_users(users){
        $('#UnchangeableUsersModalTbody > *').remove();
        $('#UnchangeableUsersModalDescription').html(
            '以下のユーザのカテゴリの変更を行うと、ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、カテゴリ変更の処理を実行できません。'
        );
        var num = 15;
        var append_row = function(user){
            var tr = $('<tr>').append(
                $('<td>').text(user['login']),
                $('<td>').text(user['lastname'] + ' ' + user['firstname']),
                $('<td>').text(user['mail'])
            );
            $('#UnchangeableUsersModalTbody').append(tr);
        };
        users.slice(0, num).forEach(function(user){ //すぐに表示されるように、はじめに表示分だけappend
            append_row(user);
        });

        $('#UnchangeableUsersModal').modal('show');

        users.slice(num, users.length).forEach(function(user){
            append_row(user);
        });
    }

    function getUsersCategories(userIds, target){
        return new Promise((resolve, reject) => {
            $.post(
                $('#get-users-categories-path').val(),
                { user_ids: JSON.stringify(userIds), target },
                function(usersCategoryIds){
                    resolve(usersCategoryIds);
                }
            ).fail(function(){
                alert('エラーが発生しました。');
                $('#pages_form').submit();
            });
        });
    }

    //カテゴリ更新時に確認ウィンドウを表示
    $('#cat_submit').on('click', function() {
        var self_userid = parseInt($('#self_user_id').val());
        var is_admin = $('#is-admin-usr').val() == 'true';
        var categories = $('#selected_categories option').val();
        var userid = parseInt($('#categories_userid').val());
        const button = $(this);
        button.addClass('pointer-events-none');
        const close = () => {
            button.removeClass('pointer-events-none');
            $('#checking-window').remove();
        };
        //$('body').append(create_loading_window());
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '更新に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            var selected_category_ids = $('#selected_category_ids').val();
            let isOutOfCategory = false;
            if ( !is_admin && !JSON.parse(selected_category_ids).length ) {
                const usersCategoryIds = await getUsersCategories([userid], 'group_category_ids');
                isOutOfCategory = (!usersCategoryIds[userid] || !usersCategoryIds[userid].length)
            }
            var path = $('#sidfm_internal_api_check_related_rules_path').val();
            var form = $('#categories_form');
            const paramsData = {
                userid,
                selected_category_ids
            };
            const actionPath = form.attr('action');
            $('#submit-related-rules-item-form').attr({
                'data-params': JSON.stringify(paramsData),
                'data-path': actionPath
            });

            $.ajax({
                url: path,
                type: 'POST',
                data: {
                    'page_name': 'user',
                    'action_idx': 'category_change',
                    'user_categoryids': JSON.stringify( { [userid] : JSON.parse(selected_category_ids) })
                },
            }).done(function(data){
                if( data['is_unchangeable_user'] ){
                    //$('#checking-window').find('.progress-wait-area-label').text('');
                    setTimeout(function(){
                        alert(
                            'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、カテゴリの変更をすることができません。\n' +
                            '※カテゴリの変更がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
                        );
                        close();
                    },100);
                    return false;
                }else{
                    const exec = () => {
                        if(Object.keys(data['rule_messages']).length > 0){
                            close();
                            display_related_rules(data);
                        }else{
                            dynamicSummaryTable.executeProcess({
                                paramsData,
                                path: actionPath,
                                additionalAlertMessage: '更新に失敗しました。'
                            });
                            close();
                        }
                    };
                    if ( isOutOfCategory ) { 
                        var conf = confirm('このままカテゴリを変更すると、更新対象のユーザがページに表示されなくなります。\nページに表示されないユーザは操作できません。\n更新してもよろしいですか?');
                        if( !conf ){
                            close();
                            return false;
                        }
                        exec();
                    }else{
                        exec();
                    }
                }
            });
        })();
        return false;
    });
    
     $('#bulk_cat_submit').on('click', function() {
        var self_userid = parseInt($('#self_user_id').val());
        var is_admin = $('#is-admin-usr').val() == 'true';
        var bulk_categories = $('#bulk_selected_categories option').val();
        const userids = JSON.parse($('#categories_bulk_userids').val());
        const button = $(this);
        button.addClass('pointer-events-none');
        const close = () => {
            button.removeClass('pointer-events-none');
            $('#checking-window').remove();
        };
        //$('body').append(create_loading_window());
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '更新に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            var path = $('#sidfm_internal_api_check_related_rules_path').val();
            var form = $('#categories_bulk_form');
            var selected_category_ids = JSON.parse($('#bulk_selected_category_ids').val());
            let isOutOfCategory = false;
            if ( !is_admin && !selected_category_ids.length ) {
                const usersCategoryIds = await getUsersCategories(userids, 'group_category_ids');
                isOutOfCategory = userids.some(userId => ( !usersCategoryIds[userId] || !usersCategoryIds[userId].length ));
            }
            const paramsData = {
                userids: JSON.stringify(userids),
                selected_category_ids: JSON.stringify(selected_category_ids)
            };
            const actionPath = form.attr('action');
            $('#submit-related-rules-item-form').attr({
                'data-params': JSON.stringify(paramsData),
                'data-path': actionPath
            });
            var user_categoryids = {};
            userids.forEach(function(user_id){
                user_categoryids[user_id] = selected_category_ids; 
            });
            $.ajax({
                url: path,
                type: 'POST',
                data: {
                    'page_name': 'user',
                    'action_idx': 'bulk_category_change',
                    'user_categoryids': JSON.stringify(user_categoryids)
                },
            }).done(function(data){
                if( data['is_unchangeable_user'] ){
                    $('#checking-window').remove();
                    display_unchangeable_users(data['unchangeable_users']);
                }else{
                    let isConfirmed = false;
                    const exec = () => {
                        if(Object.keys(data['rule_messages']).length > 0){
                            close();
                            display_related_rules(data);
                        }else{
                            if( !isConfirmed ){
                                var conf = confirm('指定したユーザ全てのカテゴリが更新されます。実行してもよろしいですか?');
                                if ( !conf ){
                                     close();
                                     return false;
                                }
                            }
                            dynamicSummaryTable.executeProcess({
                                paramsData,
                                path: actionPath,
                                additionalAlertMessage: '更新に失敗しました。'
                            });
                            close();
                        }
                    };
                    if( isOutOfCategory ){
                        let conf = confirm('このままカテゴリを変更すると、ログインユーザのカテゴリ範囲外になるため、ページに表示されなくなるユーザがあります。\nページに表示されないユーザは操作できません。\n更新してもよろしいですか?');
                        if( !conf ){
                            close();
                            return false;
                        }
                        isConfirmed = true;
                        exec();
                    }else{
                        exec();
                    }
                }
            });
        })();
        return false;
    });

    //モーダルでのユーザ新規登録・変更処理
    const selectedCategoryIdsForCreateElm = $('#selected-category-ids-for-create');
    const selectedCategoriesForCreateElm = $('#selected-categories-for-create');
    const unselectedCategoriesForCreateElm = $('#unselected-categories-for-create');
   
    const selectedCatGroupIdsForCreateElm = $('#selected-cat-group-ids-for-create');
    const selectedCatGroupsForCreateElm = $('#selected-cat-groups-for-create');
    const unselectedCatGroupsForCreateElm = $('#unselected-cat-groups-for-create');

    function resetCreateCategoriesArea(){
        selectedCategoryIdsForCreateElm.val('[]');
        const categories = JSON.parse($('#all-categories').val());
        selectedCategoriesForCreateElm.find('option').remove();
        unselectedCategoriesForCreateElm.find('option').remove();
        unselectedCategoriesForCreateElm.append(
            categories.map(cat => createCategoryOptionElm(cat.id, cat.name, cat.desc) )
        );
        unselectedCategoriesForCreateElm.find('[data-toggle="tooltip"]').tooltip();
    }

    function resetCreateCatGroupsArea(){
        selectedCatGroupIdsForCreateElm.val('[]');
        const groups = JSON.parse($('#all-cat-groups').val());
        selectedCatGroupsForCreateElm.find('option').remove();
        unselectedCatGroupsForCreateElm.find('option').remove();
        unselectedCatGroupsForCreateElm.append(
            groups.map(grp => createCatGroupOption(grp))
        );
        unselectedCatGroupsForCreateElm.find('[data-toggle="tooltip"]').tooltip();
    }

    function toggleCategoriesWarningMessage(){
        if( $('#is-admin-usr').val() !== 'true' ){
            const isSelectedCat = !!JSON.parse(selectedCategoryIdsForCreateElm.val()).length || !!JSON.parse(selectedCatGroupIdsForCreateElm.val()).length
            if( isSelectedCat ){
                $('#categories-warning').hide();
            }else{
                $('#categories-warning').show();
            }
        }
    }

    function toggleCatGroupAreaForCreate(){
        const isAdminAuthSelected = $('#pj_auth_sel').val() == $('#sidfm-admin-id').val();
        selectedCatGroupsForCreateElm.prop('disabled', isAdminAuthSelected);
        unselectedCatGroupsForCreateElm.prop('disabled', isAdminAuthSelected);
    }

    $('#modal-form').on('hide.bs.modal', function(){
        selectedCategoriesForCreateElm.scrollTop(0);
        unselectedCategoriesForCreateElm.scrollTop(0);
        selectedCatGroupsForCreateElm.scrollTop(0);
        unselectedCatGroupsForCreateElm.scrollTop(0);
    });

    $('#select-category-for-create').on('click', function(){
        const ids = JSON.parse(selectedCategoryIdsForCreateElm.val());
        const selectedCats = unselectedCategoriesForCreateElm.find('option:selected');
        selectedCats.each((idx, elm) => {
            ids.push(parseInt($(elm).val()));
        });
        selectedCategoriesForCreateElm.append(selectedCats);
        selectedCategoryIdsForCreateElm.val(JSON.stringify(ids));
        btn_status_change();
        toggleCategoriesWarningMessage();
    });

    $('#unselect-category-for-create').on('click', function(){
        let ids = JSON.parse(selectedCategoryIdsForCreateElm.val());
        const removeIds = [];
        const selectedCats = selectedCategoriesForCreateElm.find('option:selected');
        selectedCats.each((idx, elm) => {
            removeIds.push(parseInt($(elm).val()));
        });
        ids = ids.filter(id => !removeIds.includes(id));
        unselectedCategoriesForCreateElm.append(selectedCats);
        selectedCategoryIdsForCreateElm.val(JSON.stringify(ids));
        btn_status_change();
        toggleCategoriesWarningMessage();
    });

    $('#select-cat-group-for-create').on('click', function(){
        const ids = JSON.parse(selectedCatGroupIdsForCreateElm.val());
        const selectedCatGroups = unselectedCatGroupsForCreateElm.find('option:selected');
        selectedCatGroups.each((idx, elm) => {
            ids.push(parseInt($(elm).val()));
        });
        selectedCatGroupsForCreateElm.append(selectedCatGroups);
        selectedCatGroupIdsForCreateElm.val(JSON.stringify(ids));
        btn_status_change();
        toggleCategoriesWarningMessage();
    });

    $('#unselect-cat-group-for-create').on('click', function(){
        let ids = JSON.parse(selectedCatGroupIdsForCreateElm.val());
        const removeIds = [];
        const selectedCatGroupOptions = selectedCatGroupsForCreateElm.find('option:selected');
        selectedCatGroupOptions.each((idx, elm) => {
            removeIds.push(parseInt($(elm).val()));
        });
        ids = ids.filter(id => !removeIds.includes(id));
        unselectedCatGroupsForCreateElm.append(selectedCatGroupOptions);
        selectedCatGroupIdsForCreateElm.val(JSON.stringify(ids));
        btn_status_change();
        toggleCategoriesWarningMessage();
    });

    $('#pj_auth_sel').on('change', function(){
        toggleCatGroupAreaForCreate();
    });
    
    $('#modal-form').on('show.bs.modal', function(event) {
        var button = $(event.relatedTarget);
        var func = button.data('func');
        var userid = parseInt(button.data('userid'));
        var title = button.data('title');
        var login = button.data('login');
        var firstname = button.data('firstname');
        var lastname = button.data('lastname');
        var email = button.data('email');
        var pw = button.data('pw');
        var pwconf = button.data('pwconf');
        var auth = button.data('auth');
        var roleid = button.data('roleid');
        var redmine_admin = button.data('redmineadmin');
        var user_admin = button.data('useradmin');
        var mailNotification = button.data('mailnotification').toString();
        if (typeof userid === 'undefined' || (userid == "")) {
            userid = '0';
        }
        var is_force_password_change = button.data('pwchange');
        var user_lock_status = button.data('status');
        var user_data = {
          'func': func,
          'user_id': userid,
          'login': login,
          'firstname': firstname,
          'lastname': lastname,
          'email': email,
          'auth': auth,
          'role_id': roleid,
          'is_admin': redmine_admin,
          'mail_notification': mailNotification,
          'is_force_password_change': is_force_password_change
        };
        var modal = $(this);
        modal.find('.modal-title .text-truncate').text(title);
        $('#form_userid').val(userid);
        $('#form_login').val(login);
        $('#form_login_disabled').val(login);
        $('#form_firstname').val(firstname);
        $('#form_lastname').val(lastname);
        $('#form_email').val(email);
        $('#form_pw').val(pw);
        $('#form_pwconf').val(pwconf);
        $('#form_func').val(func);
        $('#mail_notification').val(mailNotification);
        if( user_lock_status && parseInt(user_lock_status) === parseInt($('#status-lock-value').val()) ){
            $('#user-lock-chkbox').prop('checked', true);
            user_data['is_locked'] = true;
        }else{
            $('#user-lock-chkbox').prop('checked', false);
            user_data['is_locked'] = false;
        }
        $('#form-user-data').val(JSON.stringify(user_data));
        $('#loginlabel').hide();
        $('#user_pwlabel').hide();
        $('#pwlabel').hide();
        $('#lastname_label').hide();
        $('#firstname_label').hide();
        $('#emaillabel').hide();
        if (redmine_admin === true){
            $('#form_sel').prop('disabled', true);
            $('#pj_auth_sel').prop('disabled', true);
        } else {
	    if (user_admin === true) {
		$('#form_sel').prop('disabled', false);
		$('#pj_auth_sel').prop('disabled', false);
	    } else {
		$('#form_sel').prop('disabled', true);
		$('#pj_auth_sel').prop('disabled', true);
	    }
        }
        $('#form_login').prop('readonly', false);
        $('#form_sel option').prop('selected', false);
        $('#form_sel  option[value="' + auth  + '"]').prop('selected', true);
        var op = $('#pj_auth_sel option[value="-1"]')
        op.text('権限を削除する');
        op.prop('hidden', true);
        op.prop('selected', false);
        $('#pj_auth_sel option[value="' + roleid + '"]').prop('selected', true);
        $('#hidden_auth').prop('disabled', true);
        $('#hidden_pj_auth').prop('disabled', true);
        if ($('#is-view-usr').val() === 'true') {
            $('#form_sel, #pj_auth_sel').prop('disabled', true);
            
        }
        if(login === 'sidfmadm'){
            $('#form_login').prop('readonly', true);
            $('#pj_auth_sel').prop('disabled', true);
        }
        if ($('#form_sel').attr('disabled') != undefined) {
            $('#hidden_auth').prop('disabled', false);
            $('#hidden_auth').val(auth);
        }
        if ($('#pj_auth_sel').attr('disabled') != undefined) {
            $('#hidden_pj_auth').prop('disabled', false);
            $('#hidden_pj_auth').val(roleid);
        }
        if (mailNotification === 'true'){
            $('#mail_notification').prop('checked', true);
        }else{
            $('#mail_notification').prop('checked', false);
        }
        if( $('#force-change-password-chkbox').length === 1 ){ //この要素は管理者権限の時のみある)
            $('#force-change-password-chkbox').prop('checked', is_force_password_change);
        }
        auth_text = $('#form_sel option:selected').text();
        pj_auth_text = $('#pj_auth_sel option:selected').text();   
        $('#form_submit').prop('disabled', true);
        if (func === "new") {
            $('#test-mail-btn, #reset-change-user-info').hide();
            $('#form_func').val('new');
            $('#form_submit').text('登録');
            $('#form').attr('action', $('#url_new').val());
            resetCreateCategoriesArea();
            resetCreateCatGroupsArea();
            $('#categories-area-for-create, #cat-groups-area-for-create').show();
            toggleCategoriesWarningMessage();
            toggleCatGroupAreaForCreate();
        } else {
            //set_paginator_parameter("#form_per_page", "#form_filters", "#form_offset", "#form_order")
            $('#form_id').val(userid);
            $('#form_func').val('mod');
            $('#reset-change-user-info').show();
            if (userid === parseInt($('#self_user_id').val())){
                op.prop('hidden', true);
                $('#test-mail-btn').show();
                $('#form-user-lock-section').hide();
            }else{
               op.prop('hidden', false);
               $('#test-mail-btn').hide();
               if (parseInt(roleid) === -1){
                 op.text('必要な場合は選択');
               }
               $('#form-user-lock-section').show();
            }
            $('#form_submit').text('更新');
            $('#form').attr('action', $('#url_mod').val());
            $('#categories-area-for-create, #cat-groups-area-for-create').hide();
        }
        toggle_pw_fields();
    });
  
    //formの内容をリセット  
    $('#reset-change-user-info').on('click', function(){
        var user_data = JSON.parse($('#form-user-data').val());
        $('#form_login').val(user_data['login']);
        $('#form_firstname').val(user_data['firstname']);
        $('#form_lastname').val(user_data['lastname']);
        $('#form_email').val(user_data['email']);   
        $('#form_pw, #form_pwconf').val('');
        $('#form_sel').val(user_data['auth']);
        $('#pj_auth_sel').val(user_data['role_id']);
        $('#mail_notification').val(user_data['mail_notification']).prop('checked', user_data['mail_notification'] === 'true' ? true : false);
        $('#force-change-password-chkbox').prop('checked', user_data['is_force_password_change']);
        $('#user-lock-chkbox').prop('checked', user_data['is_locked']);
        $('#form_submit').prop('disabled',true);
        $('#loginlabel').hide();
        $('#user_pwlabel').hide();
        $('#pwlabel').hide();
        $('#lastname_label').hide();
        $('#firstname_label').hide();
        $('#emaillabel').hide();
        toggle_pw_fields();
    });

    $('#mail_notification').on('change', function () {
            if($(this).prop('checked')) {
                $(this).val('true');
            }else {
                $(this).val('false');
            }
            btn_status_change();
    });

    $('#force-change-password-chkbox').change(function() {
        btn_status_change();
    });
    $('#form_login').on('input',function(){ 
        if (!$('#form_login').val().match(/^[a-zA-Z0-9_\-@\.]*$/)){
            $('#loginlabel').text('ログイン名に使用できない文字が含まれています。\n※半角英数字及び _.-@ のみ使用可能です')
                            .show();
        } else {
            $('#loginlabel').hide();
        }
        btn_status_change();
    });

    function check_password(){
        var conffd = $('#form_pwconf').val();
        var pwfd = $('#form_pw').val();
        $('#user_pwlabel, #pwlabel').text('パスワードが一致しません');
        if ((pwfd.length > 0 && conffd.length > 0)  && (conffd != pwfd)) {
            $('#pwlabel, #user_pwlabel').show();
        } else {
            $('#pwlabel, #user_pwlabel').hide();
        }
    }

    $('#form_pw').on('input',function() {
        check_password();
        btn_status_change();
    });
 

    $('#form_lastname').on('input',function() {
        btn_status_change();
    });
    
    $('#form_firstname').on('input',function() {
        btn_status_change();
    });

    
    $('#form_pwconf').on('input',function(){
        check_password();
        btn_status_change();
           
    });
    
    $('#form_email').on('input',function(){
        if (!$('#form_email').val().match(/^[\x20-\x7e¥]*$/)){
            $('#emaillabel')
                .text('半角英数字記号で入力してください')
                .show();
        } else {
            $('#emaillabel').hide();
        }
        btn_status_change();
    });
  
    $('#mod-body select').change(function() {
        var user_data = JSON.parse($('#form-user-data').val());
        if ( user_data['func'] != "new" ){
            btn_status_change(); 
        }
    });

    $('#user-lock-chkbox').on('change', function(){
        var user_data = JSON.parse($('#form-user-data').val());
        if ( user_data['func'] != "new" ){
            btn_status_change();
        }
    });

    $('#form_sel').on('change', function(){
        btn_status_change();
        toggle_pw_fields();
        var openid_connect_auth_id = $('#openid-connect-auth-id').val();
        if( parseInt(openid_connect_auth_id) === parseInt($('#form_sel').val()) ){
            $('#pwlabel, #user_pwlabel').hide();  
        }else{
            check_password();
        }
    });

    function toggle_pw_fields(){
        var pw_fields = $('#form_pw, #form_pwconf');
        var openid_connect_auth_id = $('#openid-connect-auth-id').val();
        if( parseInt(openid_connect_auth_id) === parseInt($('#form_sel').val()) ){
            pw_fields.prop('disabled', true);
        }else{
             pw_fields.prop('disabled', false);
        }
    }    

    function btn_status_change() {     
        var user_data = JSON.parse($('#form-user-data').val());
        if ( user_data['func'] != "new") {
            if ( $('#form_login').val() != user_data['login'] || $('#form_firstname').val() != user_data['firstname'] || 
                 $('#form_lastname').val() != user_data['lastname'] || $('#form_email').val() != user_data['email'] || 
                 ($('#form_pw').val() != '' && $('#form_pwconf').val() != '') || $('#form_sel').val() != user_data['auth'] || 
                 $('#pj_auth_sel').val() != user_data['role_id'] || $('#mail_notification').val() != user_data['mail_notification'] || 
                 $('#force-change-password-chkbox').prop('checked') != user_data['is_force_password_change'] ||
                 $('#user-lock-chkbox').prop('checked') != user_data['is_locked'] ) {
                 $('#form_submit').prop('disabled', false);
            }else{
                $('#form_submit').prop('disabled', true);
            }
        } else {
            let isCatPass = true;
            if( $('#is-admin-usr').val() !== 'true' ){
                isCatPass = !!JSON.parse(selectedCategoryIdsForCreateElm.val()).length || !!JSON.parse(selectedCatGroupIdsForCreateElm.val()).length
            }
            var openid_connect_auth_id = $('#openid-connect-auth-id').val();
            if(isCatPass && $('#form_login').val() != '' && $('#form_firstname').val() != '' && $('#form_lastname').val() != '' && $('#form_email').val() != ''){
                if( parseInt(openid_connect_auth_id) === parseInt($('#form_sel').val()) ){
                    $('#form_submit').prop('disabled', false);
                }else if( $('#form_pw').val() != '' && $('#form_pwconf').val() != '' ){   
                    $('#form_submit').prop('disabled', false);
                }else{
                    $('#form_submit').prop('disabled', true);
                }
            } else if(!isCatPass || $('#form_login').val() === '' || $('#form_firstname').val() === '' || $('#form_lastname').val() === '' || $('#form_pw').val() === '' || $('#form_pwconf').val() === '' || $('#form_email').val() === ''){
                $('#form_submit').prop('disabled', true);
            }  
        }
    } 
       
  //更新ボタン押した後にエラーメッセージを消す処理(入力が空白の時にエラーメッセージを表示するため、入力されたら消す）
    $('#form_lastname').on('input',function (){
    $('#lastname_label').hide();
    });
    $('#form_firstname').on('input',function (){
       $('#firstname_label').hide();
    });
    
    function is_auth_openid_connect(){
        var openid_connect_auth_id = $('#openid-connect-auth-id').val();
        return (parseInt(openid_connect_auth_id) == parseInt($('#form_sel').val()));
    }

    function check_register_user(){
        return new Promise((resolve, reject) => {
            $.ajax({
                url: $('#check_user_info_registerable_path').val(),
                type: 'POST',
                data: {
                    'user_id': $('#form_userid').val(),
                    'login': $('#form_login').val(),
                    'mail_address': $('#form_email').val(),
                    'password': is_auth_openid_connect() ? '' : $('#form_pw').val(),
                    'role_id': $('#pj_auth_sel').val(),
                    'func': $('#form_func').val()
                }
            }).done(function(err_data){
                if( Object.keys(err_data).length > 0 ){
                    $('#checking-window').remove();
                    login_err_message = err_data['login'];
                    address_err_message = err_data['address'];
                    pw_err_message = err_data['password'];
                    if(login_err_message){
                        $('#loginlabel').text(login_err_message).show();
                    }
                    if(address_err_message){
                        $('#emaillabel').text(address_err_message).show();
                    }
                    if(pw_err_message){
                        $('#user_pwlabel, #pwlabel').text(pw_err_message).show();
                    }
                    if(Object.keys(err_data).length == 1){
                        if (err_data['vm_apikey_allow_update_true']){
                            const allowUpdateLabel = $('#vm-apikey-allow-update-label').val();
                            let conf = confirm(`変更を実行すると、このユーザが設定されている全てのSIDfmVM APIキーの「${allowUpdateLabel}」が「不許可」に変更されます。\nよろしいですか?`);
                            resolve(conf);
                        }else if(err_data['vm_apikey_delete']){
                            let conf = confirm('変更を実行すると、このユーザが設定されている全てのSIDfmVM APIキーが削除されます。\nよろしいですか?');
                            resolve(conf);
                        }
                    }else{
                        resolve(false);
                    }
                }else{
                    resolve(true);
                }
            }).fail(function(){
                alert('エラーが発生したため、' + ($('#form_func').val() === 'new' ? '登録' : '変更') + 'に失敗しました。');
                $('#pages_form').submit();
                reject(false);
            });
        });
    }

    function check_users_related_rule_info_by_role(user_ids, role_id, action_name){
        let err = false;
        let user_role_ids = {};
        user_ids.forEach( user_id => user_role_ids[user_id] = parseInt(role_id) );

        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'POST',
                url: $('#sidfm_internal_api_check_related_rules_path').val(),
                data: { 'page_name': 'user', 'action_idx': action_name, 'user_role_ids': JSON.stringify(user_role_ids) }
            }).done(function(data){
                resolve(data);
            }).fail(function(){
                alert('エラーが発生したため、変更に失敗しました。');
                $('#pages_form').submit();
                reject(false);
            });
        });
    } 

    function display_user_role_related_rules(rule_info){
        is_action = false;
        if( !rule_info ) {
            alert('エラーが発生したため、変更に失敗しました。');
            $('#pages_form').submit();
            is_action = true;
        }
        if( rule_info['is_unchangeable_user'] ){
            $('#checking-window').find('.progress-wait-area-label').text('');
            setTimeout(function(){
                alert(
                    'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、ユーザの権限を変更することができません。\n' +
                    '※ユーザの変更がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
                );
                $('#checking-window').remove();
            }, 100);
            is_action = true;
        }else if( Object.keys(rule_info.rule_messages).length > 0 ){
            $('#checking-window').remove();
            display_related_rules(rule_info);
            is_action = true;
        }
        return is_action;
    }

    function get_user_cat_groups(user_ids){
        return new Promise((resolve, reject) => {
            $.ajax({
                type: 'POST',
                url: $('#get-user-cat-group-path').val(),
                data: { 'user_ids': JSON.stringify(user_ids) }
            }).done(function(data){
                resolve(data);
            }).fail(function(){
                alert('エラーが発生したため、変更に失敗しました。');
                $('#pages_form').submit();
                reject(false);
            });
        });
    }

    //submit処理
    const password_min_length = parseInt($('#password_min_length').val());
    $('#form_submit').on('click', function() {
        var func = $('#form_func').val();
        var is_openid_connect_auth = is_auth_openid_connect();
        //var filtered_mailaddress = $('#form_email').val().replace(/\r?\n/g, '');
        f_err = false;
        if ($('#form_login').val() == '') {
            $('#loginlabel').text('ログイン名が入力されていません').show();
            f_err = true;
        }
        /*if (!$('#form_login').val().match(/^[\x20-\x7e]*$/)){
              f_err = true;
        }*/
        if ($('#form_lastname').val() == '' || $('#form_firstname').val() == ''){
            if ($('#form_lastname').val() == ''){
                $('#lastname_label').text('姓が入力されていません').show();
            }
            if ($('#form_firstname').val() == ''){
                $('#firstname_label').text('名が入力されていません').show();
            }
            f_err = true;
        }
        if ($('#form_email').val() == ''){ 
            $('#emaillabel').text('メールアドレスが入力されていません').show();
            f_err = true;
        }
        /*if (!filtered_mailaddress.match(/^([^@\s]+)@(([-a-z0-9]+\.)+[a-z]{2,})$/i)){
            $('#emaillabel').text('メールアドレスに誤りがあります。').show();
            f_err = true;
        }*/
        if ($('#form_login').val().length > 60) {
              $('#loginlabel').text('ログイン名は60文字以内で入力してください').show();
              f_err = true;
        }  
        if ($('#form_lastname').val().length > 30) {
              $('#lastname_label').text('姓は30文字以内で入力してください').show();
              f_err = true;
        } 
        if ($('#form_firstname').val().length > 30) {
              $('#firstname_label').text('名は30文字以内で入力してください').show();
              f_err = true;
        }
        if ((func === 'new') && $('#form_pw').val() == '' && !is_openid_connect_auth) {
              $('#user_pwlabel').text('パスワードを入力してください').show();
              f_err = true;
        }
        if ($('#form_pw').val() != $('#form_pwconf').val() && !is_openid_connect_auth) {
              $('#user_pwlabel, #pwlabel').text('パスワードが一致しません').show();
              f_err = true;
        } 
        if ($('#form_pw').val().length > 0 && $('#form_pw').val().length < password_min_length && !is_openid_connect_auth) {
              $('#user_pwlabel, #pwlabel').text('パスワードは' + password_min_length + '文字以上で入力してください').show();
              f_err = true;
        } 
        /*if ((func == 'mod') && $('#form_pw').val().length >= 1 && $('#form_pw').val().length < password_min_length) {
              $('#user_pwlabel, #pwlabel').text('パスワードは' + password_min_length + '文字以上で入力してください').show();
              f_err = true;
        }*/ 
        /*if ($('#form_login').val().indexOf('/') != -1) {
              $('#loginlabel').text('(ログイン名に「/」は使えません)').show();
              f_err = true;
        }*/
        if( func != 'new' ){
            //権限削除か管理者へ変更で
            //カテゴリグループが削除されるならアラート表示
            const userData = JSON.parse($('#form-user-data').val());
            const vmAdminId = $('#sidfm-admin-id').val();
            if( userData['auth'] != vmAdminId ){
                const selectedRole = $('#pj_auth_sel').val();
                if( selectedRole == vmAdminId ||  parseInt(selectedRole) < 0 ){
                    //$.post();
                }
            }
        }
        $('body').append(create_loading_window());
        const execRegist = async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: ((func != 'new' ? '更新' : '登録') + 'に失敗しました。')
            });
            if( roleOrCategoryError ) return;
            if( func != 'new' ){
                //管理者へ変更で
                //カテゴリグループが削除されるならアラート表示
                const userData = JSON.parse($('#form-user-data').val());
                const vmAdminId = $('#sidfm-admin-id').val();
                if( userData['role_id'] != vmAdminId ){
                    const selectedRole = $('#pj_auth_sel').val();
                    if( selectedRole == vmAdminId ){
                        const userCatGroups = await get_user_cat_groups([userData.user_id]);
                        if( userCatGroups[userData.user_id] && userCatGroups[userData.user_id].length ){
                            let confDeleteGroups = confirm('SIDfmVM管理者権限に変更すると、ユーザに設定されているカテゴリグループは削除されます。\nよろしいですか?' );
                            if( !confDeleteGroups ){
                                $('#checking-window').remove();
                                return;
                            }
                        }
                    }
                }
            }
            //登録/変更実行
            const form = $('#form');
            const paramsData = {};
            form.find('[name]').each((idx, elm) => {
                elm = $(elm);
                if( !elm.prop('disabled') ){
                    switch (elm.attr('name')) {
                        case 'form[userid]':
                            paramsData.userid = elm.val();
                            break;
                        case 'form[login]':
                            paramsData.login = elm.val();
                            break;
                        case 'form[pw]':
                            paramsData.pw = elm.val();
                            break;
                        case 'form[pwconf]':
                            paramsData.pwconf = elm.val();
                            break;
                        case 'form[lastname]':
                            paramsData.lastname = elm.val();
                            break;
                        case 'form[firstname]':
                            paramsData.firstname = elm.val();
                            break;
                        case 'form[email]':
                            paramsData.email = elm.val();
                            break;
                        case 'form[auth]':
                            paramsData.auth = elm.val();
                            break;
                        case 'form[pj_auth]':
                            paramsData.pj_auth = elm.val();
                            break;
                        case 'form[mail_notification]':
                            if( elm.prop('checked') ) paramsData.mail_notification = elm.val();
                            break;
                        case 'form[force_change_password]':
                            if( elm.prop('checked') ) paramsData.force_change_password = elm.val();
                            break;
                        case 'form[user_lock]':
                             if( elm.prop('checked') ) paramsData.user_lock = elm.val();
                             break;
                    }
                }
            });
            //console.log(paramsData);
            const actionPath = form.attr('action');
            const is_validate = await check_register_user();
            if( !f_err && is_validate){
                if( func != 'new' && $('#is-view-usr').val() !== 'true' ){
                    const rule_info = await check_users_related_rule_info_by_role([ $('#form_userid').val() ], $('#pj_auth_sel').val(), 'auth_change');
                    const display_rules = display_user_role_related_rules(rule_info);
                    if(display_rules){
                        $('#submit-related-rules-item-form').attr({
                            'data-params': JSON.stringify(paramsData),
                            'data-path': actionPath
                        });
                        return false;
                    }
                }
                if( func == 'new' ){
                    paramsData.category_ids = selectedCategoryIdsForCreateElm.val();
                    paramsData.cat_group_ids = selectedCatGroupIdsForCreateElm.val();
                }
                dynamicSummaryTable.executeProcess({
                    paramsData,
                    path: actionPath,
                    additionalAlertMessage: '更新に失敗しました。'
                });
                //$('#form').submit();
            }else{
                $('#checking-window').remove();
                return false;
            }
        };
        execRegist();
        return false;
        /*
            if (func === 'mod'){
                var messages = '';
                if ($('#form_login').val() != loginval){ 
                    messages += '「ログイン名」: ' + loginval + ' ⇨ ' + $('#form_login').val() + '\n';}
                if ($('#form_firstname').val() != fname){
                    messages += '「名」: ' + fname + ' ⇨ ' + $('#form_firstname').val() + '\n';}
                if ($('#form_lastname').val() != lname){ 
                    messages += '「姓」: ' + lname + ' ⇨ ' + $('#form_lastname').val() + '\n';}
                if ($('#form_email').val() != emailval){
                    messages += '「メールアドレス」: ' + emailval + ' ⇨ ' + $('#form_email').val() + '\n';}
                if ($('#form_pw').val() != psw){
                    messages += '「パスワード」: 非表示\n';}
                if ($('#form_sel').val() != authval){
                    messages += '「認証方式」: ' + auth_text + ' ⇨ ' + $('#form_sel option:selected').text() + '\n';}
                if ($('#pj_auth_sel').val() != roleidval){
                     messages += '「ユーザ権限」: ' + pj_auth_text + ' ⇨ ' + $('#pj_auth_sel option:selected').text() + '\n';}
                if ($('#mail_notification').val() != mailNotificationVal){
                     var validity = mailNotificationVal == 'true' ? '有効 ⇨ 無効' : '無効 ⇨ 有効';
                     messages += '「メール通知設定」: ' + validity + '\n'}
                return confirm('以下の項目が変更されます。\n' + messages + 'よろしいでしょうか?');
            } else {
                return confirm('登録します。よろしいでしょうか?');
            }
        }*/      
    });

        
    //ユーザ毎のcheckboxを変更した際の処理
    $('.userSelect').on('change', function() {
        var checkbox = $(this);
        var userid = checkbox.val() - 0;
        var checked = JSON.parse($('#pages_checked').val());
        var all_userids = JSON.parse($('#all_userids').val());
        var all_userids_size = all_userids.length;
        if (checkbox.prop('checked')){
            if (!checked.includes(userid)) {
                checked.push(userid);
                if (checked.length == all_userids_size) {
                    $('.allUserSelect').prop('checked', true);
                    $('#pages_all_checked').val('true');
                } else {
                    $('.allUserSelect').prop('checked', false);
                    $('#pages_all_checked').val('false');
                }
                $('#pages_checked').val(JSON.stringify(checked));
            }
        } else {
            var idx = checked.indexOf(userid);
            if (idx == 0) {
                checked = checked.slice(1, all_userids_size);
            } else if (idx == all_userids_size - 1) {
                checked = checked.slice(0, all_userids_size - 1 );
            } else if (idx > 0) {
                var c_left = checked.slice(0, idx);
                var c_right = checked.slice(idx + 1, all_userids_size);
                checked = c_left.concat(c_right);
            }
            $('#pages_checked').val(JSON.stringify(checked));
            $('.allUserSelect').prop('checked', false);
            $('#pages_all_checked').val('false');
        }
        if(checked.length > 0){
            $('.action-btn').prop('disabled', false);
        }else{
            $('.action-btn').prop('disabled', true);
        }    
        return true;
    });

    //全ユーザチェックのcheckboxの処理
    $('.allUserSelect').on('change', function() {
        var checkbox = $(this);
        if (checkbox.prop('checked')) {
            $('#pages_checked').val($('#all_userids').val());
            $('#pages_all_checked').val('true');
            $('.userSelect').prop('checked', true);
            $('.action-btn').prop('disabled', false);
        } else {
            $('#pages_checked').val(JSON.stringify([]));
            $('#pages_all_checked').val('false');
            $('.userSelect').prop('checked', false);
            $('.action-btn').prop('disabled', true);
        }
        return true;
    });
    
    //認証方式一括処理
    $('#authenticateBulkModal').on('show.bs.modal', function(event) {
         make_bulk_data("#authenticate_bulk_userids", "#authenticateBulkTargetList");
         $('#form_select_authentication option:first').prop('selected', true);
         //set_paginator_parameter("#authenticate_bulk_per_page", "#authenticate_bulk_filters", "#authenticate_bulk_offset", "#authenticate_bulk_order");
    });

    $('#exec-authenticate-bulk').on('click', function(){
        const form = $('#authenticate_bulk_form');
        let conf = confirm(form.attr('data-confirm'));
        if( !conf ) return;
        const paramsData = {
            userids: $('#authenticate_bulk_userids').val(),
            auth: $('#form_select_authentication').val()
        };
        dynamicSummaryTable.executeProcess({
            paramsData,
            path: form.attr('action'),
            additionalAlertMessage: '更新に失敗しました。'
        });
        return false;
    });
                         
    // ユーザ権限一括処理
    $('#authorityBulkModal').on('show.bs.modal', function(event) {
        make_bulk_data("#authority_bulk_userids", "#authorityBulkTargetList");
        $('#select_authority option:first').prop('selected', true);
        //set_paginator_parameter("#authority_bulk_per_page", "#authority_bulk_filters", "#authority_bulk_offset", "#authority_bulk_order");
    });

    $('#exec-bulk-change-roles').on('click', function(){
        const user_ids = JSON.parse($('#pages_checked').val());
        const roleId = $('#select_authority').val();
        const userRoleIds = {};
        user_ids.forEach(userId => {
            userRoleIds[userId] = roleId;
        });
        const execRegist = async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '更新に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            const apikeyRelatedUsers = await checkApikeyRelatedUsers({userRoleIds});
            if( apikeyRelatedUsers === false ) return;
            let confDeleteUserGroupsText = '';
            if( roleId == $('#sidfm-admin-id').val() ){
                const userGroups = await get_user_cat_groups(user_ids);
                if( Object.values(userGroups).find(grp => !!grp.length) ){
                    confDeleteUserGroupsText = 'SIDfmVM管理者権限に変更すると、ユーザに設定されているカテゴリグループは削除されます。' ;
                }
            }
            if( apikeyRelatedUsers.not_allowed_allow_update_true_users.length ){
                const label = $('#vm-apikey-allow-update-label').val();
                let conf = confirm(
                    `変更を実行すると、以下のユーザの設定されている、全てのSIDfmVM APIキーの${label}の項目が「不許可」に変更されます。` +
                    (confDeleteUserGroupsText ? `また、${confDeleteUserGroupsText}\n` : '') + 'よろしいですか?\n' +
                    'ログイン名:\n' +
                    apikeyRelatedUsers.not_allowed_allow_update_true_users.map(user => user.login).join('\n')
                );
                if( !conf ) return;
            }else{
                const confText = (confDeleteUserGroupsText ? `${confDeleteUserGroupsText}\nよろしいですか?` : '指定したユーザ全ての権限が更新されます。実行してもよろしいですか？');//$('#update-authority-bulk-form').attr('data-confirm');
                let conf = confirm(confText);
                if( !conf ) return;
            }
            const form = $('#update-authority-bulk-form');
            const actionPath = form.attr('action');
            paramsData = {
                userids: JSON.stringify(user_ids),
                auth: roleId
            };
            const rule_info = await check_users_related_rule_info_by_role(user_ids, roleId, 'bulk_auth_change');
            const display_rules = display_user_role_related_rules(rule_info);
            if( display_rules ){
                $('#submit-related-rules-item-form').attr({
                    'data-params': JSON.stringify(paramsData),
                    'data-path': actionPath
                });
                return;
            }
            dynamicSummaryTable.executeProcess({
                paramsData,
                path: actionPath,
                additionalAlertMessage: '更新に失敗しました。'
            });
        };
        execRegist();
        return false;
    });

    // メール通知設定一括処理
    $('#MailNotificationBulkModal').on('show.bs.modal', function(event) {
        make_bulk_data("#mail_notification_bulk_userids", "#MailNotificationBulkTargetList");
        $('#mail-notification-select option:first').prop('selected', true);
        //set_paginator_parameter("#mail_notification_bulk_per_page", "#mail_notification_bulk_filters", "#mail_notification_bulk_offset", "#mail_notification_bulk_order");
    });

    $('#exec-mail-notification-bulk-form').on('click', function(){
        const form = $('#mail_notification_bulk_form');
        let conf = confirm(form.attr('data-confirm'));
        if( !conf ) return;
        const paramsData = {
             userids: $('#mail_notification_bulk_userids').val(),
             notification: $('#mail-notification-select').val()
        };
        dynamicSummaryTable.executeProcess({
            paramsData,
            path: form.attr('action'),
            additionalAlertMessage: '更新に失敗しました。'
        });
        return false;
    });

    //パスワード強制変更一括
    $('#PasswordChangeBulkModal').on('show.bs.modal', function(event) {
        make_bulk_data("#bulk_force_password_change_user_ids", "#PasswordChangeBulkTargetList");
        $('#bulk-force-password-change-select option:first').prop('selected', true);
        //set_paginator_parameter("#bulk_force_password_change_pages_per_page", "#bulk_force_password_change_pages_filters", "#bulk_force_password_change_pages_offset", "#bulk_force_password_change_pages_order");
    });
   
    $('#exec-bulk-force-password-change').on('click', function(){
        const form = $('#form-bulk-force-password-change');
        let conf = confirm(form.attr('data-confirm'));
        if( !conf ) return;
        const paramsData = {
             user_ids: $('#bulk_force_password_change_user_ids').val(),
             is_force_change_password: $('#bulk-force-password-change-select').val()
        };
        dynamicSummaryTable.executeProcess({
            paramsData,
            path: form.attr('action'),
            additionalAlertMessage: '更新に失敗しました。'
        });
        return false;
    });
 
    //アクションボタンのmodalを開くとき
    $('.action-btn').on('click', function(){
        var checked = JSON.parse($('#pages_checked').val());
        if(checked.length === 0){
            alert('対象ユーザがチェックされておりません。');
            return false;
        }
        var target_modal = $(this).attr('data-target');
        //カテゴリ・カテゴリグループ一括更新の時に、管理者権限のユーザと自身を対象から外す
        if(target_modal === 'categoriesBulkModal' || target_modal === 'catGroupsBulkModal'){
            var user_roles = JSON.parse($('#users_role_names').val());
            var filtered_userids = [];
            var untargeted_users = [];
            var self_user_id = $('#self_user_id').val();
            if( target_modal === 'categoriesBulkModal' ){
                var is_admin = $('#is-admin-usr').val() === 'true';
                for(var i = 0; i < checked.length; i++){
                    var user_id = checked[i];
                    if(is_admin || (user_roles[user_id] != 'SIDfmVM管理者' && self_user_id != user_id)){
                        filtered_userids.push(user_id);
                    }else{
                        untargeted_users.push('.userSelect[value="' + user_id + '"]');
                    }
                }
            }else{
                for(var i = 0; i < checked.length; i++){
                    var user_id = checked[i];
                    if(user_roles[user_id] != 'SIDfmVM管理者' && self_user_id != user_id){
                        filtered_userids.push(user_id);
                    }else{
                        untargeted_users.push('.userSelect[value="' + user_id + '"]');
                    }
                }
            }
            if(filtered_userids.length === 0){
                var is_admin = $('#is-admin-usr').val() === 'true';
                if( target_modal === 'categoriesBulkModal' ){
                    alert('対象となるユーザは存在しません。' + (is_admin ? '' : '\n※ログインユーザのカテゴリは変更できません。'));
                }else{
                    alert('対象となるユーザは存在しません。\n※管理者権限のユーザ' + (is_admin ? '' : '、及びログインユーザ') + 'のカテゴリグループは変更できません。');
                }
                return false;
            }
            if(untargeted_users.length > 0){
                $('#pages_checked').val(JSON.stringify(filtered_userids));
                $(untargeted_users.join(', ')).prop('checked', false);
                $('#allSelect').prop('checked', false);
                if(target_modal === 'categoriesBulkModal'){
                    $('#bulk-categories-caution').show();
                }else{
                    $('#bulk-cat-groups-caution').show();
                }
            }else{
                if(target_modal === 'categoriesBulkModal'){
                    $('#bulk-categories-caution').hide();
                }else{
                    $('#bulk-cat-groups-caution').hide();
                }
            }
        }
        //$('#wrap-ActionArea').css('z-index', '10');
        $('#' + target_modal).modal('show');
    });
    
    //一括処理時のテーブル作成処理
    function make_bulk_data(bulk_userids, TargetList) {
        var checked = JSON.parse($('#pages_checked').val());
        var userids_names = JSON.parse($('#userids_names').val());
        if (checked.length > 0) {
            $(bulk_userids).val(JSON.stringify(checked));
            $(TargetList + ' > *').remove();
            for (var i = 0; i < checked.length; i++){
                userid = checked[i];
                if (userid in userids_names){
                    var c_tr = $('<tr></tr>', { class: 'list_alternative2' });
                    var c_td = $('<td></td>', { class: 'hostname' });
                    c_td.text(userids_names[userid]);
                    c_tr.append(c_td);
                    $(TargetList).append(c_tr);
                }
            }
        } else {
            alert('対象ユーザがチェックされておりません。');
            location.reload(true);
            return false;
        }
        return true;
        
    } 

    //カテゴリ一括処理
     $('#categoriesBulkModal').on('show.bs.modal', function(event) {
        var button =$(event.relatedTarget);
        make_bulk_data("#categories_bulk_userids", "#categoriesBulkTargetList");
        //set_paginator_parameter("#cat_bulk_per_page", "#cat_bulk_filters", "#cat_bulk_offset", "#cat_bulk_order");
        $('#bulk_selected_categories, #bulk_unselected_categories').html('');
        $('#bulk_selected_category_ids').val('[]');
        const selectableCategories = JSON.parse($('#selectable-bulk-categories').val());
        options = selectableCategories.map(cat => {
            return $('<option>').attr({
                class: "text-truncate g-max-width-21em",
                value: cat.id,
                'data-toggl': "tooltip",
                'data-title': cat.desc,
                'data-placement': "right",
                'data-boundary': "window"
            }).text(cat.name);
        });
        $('#bulk_unselected_categories').append(options);
        return true;
    });

    $('#bulk_select_category').on('click', function() {
        move_selected_items('#bulk_selected_category_ids',
                            '#bulk_unselected_categories',
                            '#bulk_selected_categories', false);
        $('#bulk_selected_categories option').tooltip();
    });

    $('#bulk_unselect_category').on('click', function() {
        move_selected_items('#bulk_selected_category_ids',
                            '#bulk_selected_categories',
                            '#bulk_unselected_categories', true);
        $('#bulk_unselected_categories option').tooltip();
    });

    //カテゴリグループ一括更新
    $('#catGroupsBulkModal').on('show.bs.modal', function(event) {
        make_bulk_data('#bulk-cat-groups-userids', '#catGroupsBulkTargetList');
        $('#selected-bulk-cat-groups, #unselected-bulk-cat-groups').html('');
        $('#selected-bulk-user-cat-group-ids').val('[]');
        $('#unselected-bulk-cat-groups').append(
          catGroups.map(group => createCatGroupOption(group))
        )
        $('#unselected-bulk-cat-groups').find('[data-toggle="tooltip"]').tooltip();
        return true;
    });

    $('#catGroupsBulkModal').on('hide.bs.modal', function(){
        $('#selected-bulk-cat-groups, #unselected-bulk-cat-groups').scrollTop(0);
    });

    $('#select-bulk-cat-group').on('click', function(){
        const selectedGroups = $('#unselected-bulk-cat-groups option:selected');
        const ids = JSON.parse($('#selected-bulk-user-cat-group-ids').val());
        selectedGroups.each((idx, elm) => {
            ids.push(parseInt($(elm).val()));
        });
        $('#selected-bulk-cat-groups').append(selectedGroups);
        $('#selected-bulk-user-cat-group-ids').val(JSON.stringify(ids));
    });

    $('#unselect-bulk-cat-group').on('click', function(){
        const selectedGroups = $('#selected-bulk-cat-groups option:selected');
        let ids = JSON.parse($('#selected-bulk-user-cat-group-ids').val());
        const removeIds = [];
        selectedGroups.each((idx, elm) => {
            removeIds.push(parseInt($(elm).val()));
        });
        ids = ids.filter(id => !removeIds.includes(id));
        $('#unselected-bulk-cat-groups').append(selectedGroups);
        $('#selected-bulk-user-cat-group-ids').val(JSON.stringify(ids));
    });

    $('#exec-bulk-update-cat-groups').on('click', function(){
        const button = $(this);
        button.addClass('pointer-events-none');
        const close = () => {
            button.removeClass('pointer-events-none');
            $('#checking-window').remove();
        };
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '更新に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            const isAdmin = $('#is-admin-usr').val() == 'true';
            const paramsData = {
                user_ids: $('#bulk-cat-groups-userids').val(),
                group_ids: $('#selected-bulk-user-cat-group-ids').val()
            };
            const userIds = JSON.parse(paramsData.user_ids);
            const groupIds = JSON.parse(paramsData.group_ids);
            let isOutOfCategory = false;
            if ( !isAdmin && !groupIds.length ) {
                const usersCategoryIds = await getUsersCategories(userIds, 'category_ids_without_group_categories');
                isOutOfCategory = userIds.some(userId => (!usersCategoryIds[userId] || !usersCategoryIds[userId].length));
            }
            var path = $('#sidfm_internal_api_check_related_rules_path').val();
            const actionPath = $('#update-cat-groups-bulk-path').val();
            $('#submit-related-rules-item-form').attr({
                'data-params': JSON.stringify(paramsData),
                'data-path': actionPath
            });

            const userCatGroupIds = {};
            userIds.forEach(userId => userCatGroupIds[userId] = groupIds);
    
            $.ajax({
                url: path,
                type: 'POST',
                data: {
                    'page_name': 'user',
                    'action_idx': 'cat_group_change',
                    'user_cat_group_ids': JSON.stringify(userCatGroupIds)
                },
            }).done(function(data){
                if( data['is_unchangeable_user'] ){
                    //$('#checking-window').find('.progress-wait-area-label').text('');
                    setTimeout(function(){
                        alert(
                            'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、カテゴリグループの変更をすることができません。\n' +
                            '※変更がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
                        );
                        close();
                    },100);
                    return false;
                }else{
                    let isConfirmed = false;
                    const exec = () => {
                        if(Object.keys(data['rule_messages']).length > 0){
                            close();
                            display_related_rules(data);
                        }else{
                            if( !isConfirmed ){
                                var conf = confirm('指定したユーザ全てのカテゴリグループが更新されます。実行してもよろしいですか?');
                                if ( !conf ){
                                     close();
                                     return false; 
                                } 
                            }  
                            dynamicSummaryTable.executeProcess({
                                paramsData,
                                path: actionPath,
                                additionalAlertMessage: '更新に失敗しました。'
                            });
                            close();
                        }
                    };
                    if( isOutOfCategory ){
                        let conf = confirm('このままカテゴリグループを変更すると、ログインユーザのカテゴリ範囲外になるため、ページに表示されなくなるユーザがあります。\nページに表示されないユーザは操作できません。\n更新してもよろしいですか?');
                        if( !conf ){
                            close();
                            return false;
                        }
                        isConfirmed = true;
                        exec();
                    }else{
                        exec();
                    }
                }
            });
        })();
    });

    function set_paginator_parameter(p_per_page, p_filters, p_offset, p_order){
        var offset = $('#pages_offset').val();
        var filters = JSON.parse($('#pages_filters').val());
        var per_page = $('#pages_per_page').val();
        var order = $('#pages_order').val();
        $(p_per_page).val(per_page);
        $(p_filters).val(JSON.stringify(filters));
        $(p_offset).val(offset);
        $(p_order).val(order);
    }
    
    $('#TestEmailModal').on('show.bs.modal', function(){
        var email = $('#user_email').val()
        $('#test_email_recipient').text(email);
        $('#hidden_email_address').val(email);
        $('#modal-form').css('z-index','500');
        set_paginator_parameter("#test_mail_pages_per_page", "#test_mail_pages_filters", "#test_mail_pages_offset", "#test_mail_pages_order");
    });
 
    $('#TestEmailModal').on('hidden.bs.modal',function(){
        $('#modal-form').css('z-index', '1050');
    });

    sticky_control('UsersTable', pageName);
    $(window).on('load', function() {
        setTimeout(function(){
            set_scroll();
            $(window).on('resize scroll', set_scroll);
        },200);
    });
    
    //ブラウザバックだったらチェックボックスの表示を修正
    /*$.post($('#check_browser_back_path').val(), {'page_name': 'user'},
        function(info){
            if(info === 'browser_back'){
                var is_all_checked = $('#pages_all_checked').val();
                if(is_all_checked === 'true'){
                    var all_ids = JSON.parse($('#all_userids').val());
                    $('#UsersTable').find('#allSelect, .userSelect').prop('checked', true);
                    $('#pages_checked').val(JSON.stringify(all_ids.filter(Boolean)));
                }else{
                    $('#UsersTable').find('#allSelect, .userSelect').prop('checked', false);
                    var pages_checked = JSON.parse($('#pages_checked').val());
                    var target_elm = '';
                    for (var id of pages_checked){
                        target_elm += '#table_selected' + id + ',';
                    }
                    target_elm = target_elm.slice(0, -1);
                    $(target_elm).prop('checked', true);
                }
            }
    });*/

    //サマリー表のチェックボックスとpages_checkedの整合性を取る
    $(window).on('pageshow', function(){
        var is_all_checked = $('#pages_all_checked').val();
        if(is_all_checked === 'true'){
            var all_ids = JSON.parse($('#all_userids').val());
            $('#UsersTable').find('#allSelect, .userSelect').prop('checked', true);
            $('#pages_checked').val(JSON.stringify(all_ids.filter(Boolean)));
        }else{
            $('#UsersTable').find('#allSelect, .userSelect').prop('checked', false);
            var pages_checked = JSON.parse($('#pages_checked').val());
            var target_elm = '';
            pages_checked.forEach(function(id){
                target_elm += '#table_selected' + id + ',';
            });
            target_elm = target_elm.slice(0, -1);
            $(target_elm).prop('checked', true);
        }
    });
    
    //アクションボタンの活性・非活性
    if( JSON.parse($('#pages_checked').val()).length > 0 ){
        $('.action-btn').prop('disabled', false);
    }else{
        $('.action-btn').prop('disabled', true);
    }

  //関連するルールを確認するmodalでOKボタンを押した時に処理を実行
  $('#submit-related-rules-item-form').on('click', function(){
      const button = $(this);
      const paramsData = JSON.parse(button.attr('data-params'));
      dynamicSummaryTable.executeProcess({
          paramsData,
          path: button.attr('data-path'),
          additionalAlertMessage: '更新に失敗しました。'
      });
      return false;
  });

  //エクスポート
  $('#exec-download').on('click', function(){
      $('#download-target').val('tsv');
      json_tsv_summary_table_progress_download($('#download-target').val(), $('#read-download-data-path').val(), { dynamicSummaryTable });
  });

  //インポート
  $('#submit-import-data').on('click', function(){
      var userinfo_file = $('#userinfo-file').get(0).files[0];
      if( !userinfo_file ){
          alert('ファイルが選択されておりません。');
          return false;
      }

      var fd = new FormData();
      fd.append('userinfo_file', userinfo_file);

      var read_import_data_path = $('#read-import-data-path').val();
      var progress_import_path = $('#progress-import-path').val();
      var progress_window = get_progress_bar();
      $('body').append(progress_window);
      (async () => {
          const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
              additionalAlertMessage: 'インポートに失敗しました。'
          });
          if( roleOrCategoryError ) return;
          var percentage = 0;
          var total_size = 0;
          var executed_size = 0;
    
          var progress_import = function(){
              $.ajax({
                  url: progress_import_path ,
                  type: 'GET',
              }).done(function(res){
                  if( res['executed_size'] > 0 ){
                      executed_size += res['executed_size'];
                      var percentage = (executed_size / total_size * 100);
                      if( percentage >= 99 && percentage < 100 ){
                          percentage = 99;
                      }else{
                          percentage = Math.round(percentage);
                      }
                      $('#progress-bar').attr('aria-valuenow', percentage).css('width', percentage + '%').text(percentage + '%');
                      progress_import();    
                  }else{
                     setTimeout(function(){//probress barのbarが満たされるまで待つ
                         location.href = $('#import-path').val();
                     }, 500);
                  }
              }).fail(function(){
                  alert('ファイルのインポートに失敗しました。画面をリロードします。');
                  $('#pages_form').submit(); 
              });
          };
    
          $.ajax({
              url: read_import_data_path,
              type: 'POST',
              data: fd,
              processData: false,
              contentType: false,
          }).done(function(res){
              if( res['err'] ){
                  alert('ファイルのインポートに失敗しました。画面をリロードします。');
                  $.cookie('import_err', 'true');
                  $('#pages_form').submit();
              }else{
                  total_size = res['total_size'];
                  progress_import();
              }
          }).fail(function(){
              alert('ファイルのインポートに失敗しました。画面をリロードします。');
              $('#pages_form').submit();
          });
      })();
  });

  //テストメール送信
  $('#exec-test-email').on('click', function(){
      const form = $('#test_email_form');
      let conf = confirm(form.attr('data-confirm'));
      if( !conf ) return;
      const paramsData = {
          email_address: form.find('[name="form[email_address]"]').val(),
          subject: form.find('[name="form[subject]"]').val(),
          message: form.find('[name="form[message]"]').val()
      };
      dynamicSummaryTable.executeProcess({
           paramsData,
           path: $('#sidfm_users_test_email_path').val(),
           additionalAlertMessage: 'メールの送信に失敗しました。',
           funcAfterUpdate: () => {
               dynamicSummaryTable.closeVisibleModals();
           }
      });
      return false;
  });
 
    activate_sidebar_item('sidebar-item-user-manip')     
});
