#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

# 遅延評価用のクラス
# valueメソッドを最初に使用した際に評価を行い、以降は同じ値を返す。
# ex.
#    a = DelayEvaluation.new('http://hogehoge/') do |x|
#      Net::HTTP.get(x)
#    end
#    # この時点では実行されていない。
#    a.value # http://hogehoge/ にアクセスした結果が返る
#    a.value # 最初に実行されたものと同じ結果が返る。
class DelayEvaluation
  private

  def initialize(*args, &block)
    @block = block
    @args = args
    @instance = nil
  end

  public

  def value
    unless @instance then
      if @args.size > 0 then
        @instance = @block.call(*@args)
      else
        @instance = @block.call
      end
    end
    @instance
  end

  alias :v :value
end

if $0 == __FILE__ then
  require 'pp'

  b = 123
  a = DelayEvaluation.new('xxx') {|x|
    b = x.size
    321
  }
  pp a
  pp b
  pp a.v
  pp b
  pp a.v

  z = DelayEvaluation.new { 1+1 }
  pp z
  pp z.v
  
end
