# -*- coding: utf-8 -*-

# Copyright (C) 2020-2021 SofTek Systems, Inc. All rights reserved.

module SIDfm
  module VMRule

    # 条件
    class Cond

      def self.log_info(*s)
        SIDfm::VMRule.log_info(*s)
      end

      def self.log_error(*s)
        SIDfm::VMRule.log_error(*s)
      end

      def self.log_debug(*s)
        SIDfm::VMRule.log_debug(*s)
      end

      private

      def initialize(rule, args = {})
        raise ArgumentError unless rule
        @rule = rule
        @args = args

        @prev_vuls_content = (@args[:prev_vuls_content] || [])
        @hosts_content_keys = []
        @vuls_content_keys = []
        @vulsync_content_keys = []
        SIDfm::VMRule::HOSTS_COTENT_KEYS.each do |key|
          if @args.has_key?(key) and @args[key].respond_to?(:'[]') then
            @hosts_content_keys << key
          end
        end
        SIDfm::VMRule::VULS_CONTENT_KEYS.each do |key|
          if @args.has_key?(key) and @args[key].respond_to?(:'[]') then
            @vuls_content_keys << key
          end
        end
        SIDfm::VMRule::VULSYNC_CONTENT_KEYS.each do |key|
          if @args.has_key?(key) and @args[key].respond_to?(:'[]') then
            @vulsync_content_keys << key
          end
        end
        (@hosts_content_keys + @vuls_content_keys + @vulsync_content_keys).each do |key|
          @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym] = @args[key].deep_dup
        end
        self.class.log_debug("@args[:category_id] = #{@args[:category_id]}")
        self.class.log_debug("@args[:host_ids] = #{@args[:host_ids]}")
        if @args[:category_id] and @args[:category_id] > 0 then
          # ルールもしくはトリガーにカテゴリが指定されていた場合の処理
          cr = Sidfm.corresp_required
          @hosts_content_keys.each do |key|
            @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym].delete_if {|x| !@args[:host_ids].include?(x[:sidfmd_host_id]) }
          end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            SidfmdIssue.update_values_by_scsids(@args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym], @args[:host_ids], [ @args[:category_id] ], cr)
          end
        elsif @args[:host_ids] then
          # トリガーにホストが指定されていた場合の処理
          cr = Sidfm.corresp_required
          @hosts_content_keys.each do |key|
            @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym].delete_if {|x| !@args[:host_ids].include?(x[:sidfmd_host_id]) }
          end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            SidfmdIssue.update_values_by_scsids(@args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym], @args[:host_ids], nil, cr)
          end
        end
      end

      # sidfmd_host_idからsidfmd_category_idの一覧を返す。
      @@host_cat_ids = nil

      def self.get_catids_by_hostid(hostid)
        if @@host_cat_ids then
          (@@host_cat_ids[hostid] || [])
        else
          @@host_cat_ids = {}
          SidfmdCategoriesHost.all.each do |i|
            if @@host_cat_ids.has_key?(i[:sidfmd_host_id]) then
              @@host_cat_ids[i[:sidfmd_host_id]] << i[:sidfmd_category_id]
            else
              @@host_cat_ids[i[:sidfmd_host_id]] = [ i[:sidfmd_category_id] ]
            end
          end
          (@@host_cat_ids[hostid] || [])
        end
      end

      # :hostsと:categoriesを更新
      # :sri_cvss_status_idsを更新してから実行する。
      def self.update_hosts_cats(row)
        scsids_hostids = row[:sri_cvss_status_ids].collect {|x| x[:sidfmd_host_id] }.compact.uniq
        row[:hosts].delete_if {|x| !scsids_hostids.include?(x[:id]) }
        catids = scsids_hostids.collect {|x| SIDfm::VMRule::Cond.get_catids_by_hostid(x) }.flatten.compact.uniq
        row[:categories].delete_if {|x| !catids.include?(x[:id]) }
      end

      # detailが無いSidfmRuleCond（不正）
      def cond_invalid?(cond)
        cond.log_err("無効な条件")
        false
      end

      def cond_match_host?(cond)
        self.class.log_debug("cond_match_host?")
        cnt = 0
        detail = cond.detail
        inv = detail.inv
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['host-' + (inv ? 'unmatch' : 'match')]} 処理開始")
        host_ids = detail.sidfmd_rule_cm_host_subs.collect {|x| x.sidfmd_host_id }
        host_ids.compact!
        host_ids.uniq!
        corresp_required = Sidfm.corresp_required
        self.class.log_debug(" inv=#{inv}")
        self.class.log_debug(" host_ids=#{host_ids.inspect}")
        if host_ids.size > 0 then
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(x[:sidfmd_host_id]) then
                ret = inv ? host_ids.include?(x[:sidfmd_host_id]) : !host_ids.include?(x[:sidfmd_host_id]) 
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                inv ? host_ids.include?(y[:sidfmd_host_id]) : !host_ids.include?(y[:sidfmd_host_id])
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                ret = false
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
        else
          cond.log_warn("ホストが指定されていないため、条件は無視されました。")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['host-' + (inv ? 'unmatch' : 'match')]} 処理終了:#{ret}")
        ret
      end

      def cond_match_app?(cond)
        self.class.log_debug("cond_match_app?")
        cnt = 0
        detail = cond.detail
        inv = detail.inv
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['product-' + (inv ? 'unmatch' : 'match')]} 処理開始")
        apps = detail.sidfmd_rule_cm_app_subs
        lapps = detail.sidfmd_rule_cm_lapp_subs 
        self.class.log_debug(" inv=#{inv}")
        self.class.log_debug(" apps=#{apps.inspect}")
        if apps.size > 0 or lapps.size > 0 then
          # 予めappsの情報を取得しておく
          # product_id, version_idはnilの場合0の場合があるので、無理矢理0に統一。
          db_apps = []
          apps.each do |app|
            temp = {
              :product_id => app[:product_id] || 0,
              :version_id => app[:version_id] || 0,
            }
            next if (temp[:product_id] == 0 and temp[:version_id] == 0)
            sidfmd_product = nil
            sidfmd_version = nil
            if temp[:version_id] != 0 then
              sidfmd_version = SidfmdVersion.find_by_vid(temp[:version_id])
            end
            if sidfmd_version then
              sidfmd_product = sidfmd_version.sidfmd_product
              if sidfmd_product then
                temp[:product_id] = sidfmd_product.pid
              end
            end
            if sidfmd_product.nil? then
              sidfmd_product = SidfmdProduct.find_by_pid(temp[:product_id])
            end
            if sidfmd_product then
              temp[:name] = sidfmd_product[:name]
            end
            if sidfmd_version then
              temp[:fullname] = sidfmd_version[:fullname]
            end
            next if temp[:product_id] == 0
            temp[:is_queue_app] = SidfmdQueueApp::PRODUCT_NAMES.values.include?(temp[:name]) ? true : false
            db_apps << temp
          end
          #ローカルプロダクト
          db_lapps = []
          lapps.each do |lapp|
            temp = {
              :lproduct_id => lapp[:sidfmd_lproduct_id] || 0,
              :lversion_id => lapp[:sidfmd_lversion_id] || 0,
            }
            next if (temp[:lproduct_id] == 0 and temp[:lversion_id] == 0)
            lver = nil
            if temp[:lversion_id] != 0 then
              lver = SidfmdLversion.find_by_id(temp[:lversion_id])
            end
            if lver then
              lprod = lver.sidfmd_lproduct
              if lprod then
                temp[:lproduct_id] = lprod.id
              end
            end
            next if temp[:lproduct_id] == 0
            db_lapps << temp
          end
          self.class.log_debug(" db_apps")
          self.class.log_debug(db_apps)
          target_host_ids = []
          os_host_ids = []
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            # osチェック(agent)
            app_fullnames = []
            app_names = []
            db_apps.each do |i|
              if i[:fullname] then
                app_fullnames << i[:fullname]
              else
                app_names << i[:name]
              end
            end
            app_fullnames.uniq!
            app_fullnames.compact!
            app_names.uniq!
            app_names.compact!
            query1 = SidfmdQueue.joins(:sidfmd_host).
              select(:id).
              where("sidfmd_queues.sidfmd_host_id=sidfmd_hosts.id").
              order(:created_at => :desc).
              limit(1)
            query2 = SidfmdHost.
              select(:id).
              select("(#{query1.to_sql}) AS find_key")
            if @args[:host_ids] then
              query2 = query2.where(:id => @args[:host_ids])
            end
            query3 = SidfmdHost.
              select(:id).
              joins("INNER JOIN (#{query2.to_sql}) AS hosts_queues ON hosts_queues.id=sidfmd_hosts.id").
              joins("INNER JOIN sidfmd_queue_dists ON hosts_queues.find_key=sidfmd_queue_dists.sidfmd_queue_id").
              where("hosts_queues.find_key IS NOT NULL")
            if app_fullnames.size > 0 and app_names.size > 0 then
              query3 = query3.
                where("(sidfmd_queue_dists.name || ' ' || sidfmd_queue_dists.version) IN (?) OR sidfmd_queue_dists.name IN (?)", app_fullnames, app_names)
            elsif app_fullnames.size > 0 then
              query3 = query3.
                where("(sidfmd_queue_dists.name || ' ' || sidfmd_queue_dists.version) IN (?)", app_fullnames)
            elsif app_names.size > 0 then
              query3 = query3.
                where("sidfmd_queue_dists.name IN (?)", app_names)
            else
              # last resort
              query3 = query3.where("false")
            end
            if @args[:host_ids] then
              query3 = query3.where(:sidfmd_hosts => { :id => @args[:host_ids] })
            end
            query3.each do |host|
              target_host_ids << host[:id]
              os_host_ids << host[:id]
            end
            # hostappチェック (手動のOSチェック含む) - product
            query1 = SidfmdHostApp.
              select(:sidfmd_host_id)
            if @args[:host_ids] then
              query1 = query1.where(:sidfmd_host_id => @args[:host_ids])
            end
            query1 = query1.
              where(:product_id => db_apps.collect {|x| (x[:version_id].nil? or x[:version_id]==0) ? x[:product_id] : nil }.uniq.compact)
            query1.each do |host|
              target_host_ids << host[:sidfmd_host_id]
            end
            # hostappチェック (手動のOSチェック含む) - version
            query1 = SidfmdHostApp.
              select(:sidfmd_host_id)
            if @args[:host_ids] then
              query1 = query1.where(:sidfmd_host_id => @args[:host_ids])
            end
            query1 = query1.
              where(:version_id => db_apps.collect {|x| (x[:version_id] and x[:version_id] > 0) ? x[:version_id] : nil  }.uniq.compact)
            query1.each do |host|
              target_host_ids << host[:sidfmd_host_id]
            end
            # hostlappのチェック - lproduct
            query1 = SidfmdHostLapp.
              select(:sidfmd_host_id)
            if @args[:host_ids] then
              query1 = query1.where(:sidfmd_host_id => @args[:host_ids])
            end
            query1 = query1.
              where(:sidfmd_lproduct_id => db_lapps.collect {|x| x[:lversion_id] == 0 ? x[:lproduct_id] : nil }.uniq.compact)
            query1.each do |host|
              target_host_ids << host[:sidfmd_host_id]
            end
            # hostlappのチェック - lversion
            query1 = SidfmdHostLapp.
              select(:sidfmd_host_id)
            if @args[:host_ids] then
              query1 = query1.where(:sidfmd_host_id => @args[:host_ids])
            end
            query1 = query1.
              where(:sidfmd_lversion_id => db_lapps.collect {|x| x[:lversion_id] > 0 ? x[:lversion_id] : nil }.uniq.compact)
            query1.each do |host|
              target_host_ids << host[:sidfmd_host_id]
            end
            # queue_appチェック
            if db_apps.inject(false) {|r, x| r |= x[:is_queue_app] } then
              query1 = SidfmdQueue.joins(:sidfmd_host).
                select(:id).
                where("sidfmd_queues.sidfmd_host_id=sidfmd_hosts.id").
                order(:created_at => :desc).
                limit(1)
              query2 = SidfmdHost.
                select(:id).
                select("(#{query1.to_sql}) AS queue_id")
              if @args[:host_ids] then
                query2 = query2.where(:sidfmd_host_id => @args[:host_ids])
              end
              query3 = SidfmdHost.
                select(:id).
                joins("INNER JOIN (#{query2.to_sql}) AS hosts_queues ON hosts_queues.id=sidfmd_hosts.id").
                joins("INNER JOIN sidfmd_queue_apps ON hosts_queues.queue_id=sidfmd_queue_apps.sidfmd_queue_id").
                where("hosts_queues.queue_id IS NOT NULL").
                where("sidfmd_queue_apps.name ~~* any(array[?]) or (sidfmd_queue_apps.name || ' ' || sidfmd_queue_apps.version) ~~* any(array[?])", app_names.uniq.compact, app_fullnames.collect {|x| "#{x}%" }.uniq.compact)
              if @args[:host_ids] then
                query3 = query3.where(:sidfmd_host_id => @args[:host_ids])
              end
              query3.each do |host|
                target_host_ids << host[:id]
              end
            end
            target_host_ids.uniq!
            target_host_ids.compact!
            content.delete_if {|x|
              ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(x[:sidfmd_host_id]) then
                temp_ret = target_host_ids.include?(x[:sidfmd_host_id])
                ret = inv ? temp_ret : !temp_ret
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            target_html_ids = []
            # affprodチェック。product_idで照合。
            prod_only_product_ids = db_apps.collect {|x| x[:version_id] == 0 ? x[:product_id] : nil }.compact.uniq
            query1 = SidfmdIssue.
              joins(:sidfmd_issue_affprods).
              where(:sidfmd_issue_affprods => { :product_id => prod_only_product_ids })
            if @args[:host_ids] then
              query1 = query1.where(:sidfmd_host_id => @args[:host_ids])
            end
            query1.each do |issue|
              target_html_ids << issue[:html_id]
            end

            # affverチェック。version_idで照合。
            version_ids = db_apps.collect {|x| x[:version_id] > 0 ? x[:version_id] : nil }.compact.uniq
            query1 = SidfmdIssue.
              joins(:sidfmd_issue_affvers).
              where("sidfmd_issue_affvers.product_version_id IN (?) OR sidfmd_issue_affvers.product_id IN (?)", version_ids, prod_only_product_ids)
            if @args[:host_ids] then
              query1 = query1.where(:sidfmd_host_id => @args[:host_ids])
            end
            query1.each do |issue|
              target_html_ids << issue[:html_id]
            end

            # lproduct・lversionチェック。lproduct_id(SidfmdLproduct#id)・lversion_id(SidfmdLversion#id)で照合
            prod_only_lproduct_ids = db_lapps.collect {|x| x[:lversion_id] == 0 ? x[:lproduct_id] : nil }.compact.uniq
            lversion_ids = db_lapps.collect {|x| x[:lversion_id] > 0 ? x[:lversion_id] : nil }.compact.uniq
            query1 = SidfmdIssue.where("html_id IN (?) OR html_id IN (?)",
              SidfmdLocalVulLapp.joins(:sidfmd_local_vul)
                .where(
                  "sidfmd_local_vul_lapps.sidfmd_lproduct_id IN (?) OR sidfmd_local_vul_lapps.sidfmd_lversion_id IN (?)",
                  prod_only_lproduct_ids,
                  lversion_ids
                )
                .select('sidfmd_local_vuls.html_id'),
              SidfmdLocalVulApp.joins(:sidfmd_local_vul)
                .where(
                  "sidfmd_local_vul_apps.pid IN (?) OR sidfmd_local_vul_apps.vid IN (?)",
                  prod_only_product_ids,
                  version_ids
                )
                .select('sidfmd_local_vuls.html_id')
            )
            if @args[:host_ids] then
              query1 = query1.where(:sidfmd_host_id => @args[:host_ids])
            end
            query1.each do |issue|
              target_html_ids << issue[:html_id]
            end
            # affpkgについてはチェック不要（affprod, affverで起因がOSとして登録されている）
            # 最後にtarget_html_idsでフィルタ。
            target_html_ids.uniq!
            target_html_ids.compact!
            content.delete_if {|x|
              ret = true
              content_hosts = x[:hosts].collect {|y| y[:id] }
              content_hosts.compact!
              if @args[:host_ids].nil? or
                  (@args[:host_ids] & content_hosts).size > 0 then
                if target_html_ids.include?(x[:html_id]) then
                  temp_ret = !((content_hosts & target_host_ids).size > 0)
                  ret = inv ? !temp_ret : temp_ret
                else
                  ret = !inv
                end
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
        else
          cond.log_warn("プロダクトが指定されていないため、条件は無視されました。")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['product-' + (inv ? 'unmatch' : 'match')]} 処理終了:#{ret}")
        ret
      end

      def cond_match_category?(cond)
        self.class.log_debug("cond_match_category?")
        cnt = 0
        detail = cond.detail
        inv = detail.inv
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['category-' + (inv ? 'unmatch' : 'match')]} 処理開始")
        category_ids = detail.sidfmd_rule_cm_category_subs.collect {|x| x.sidfmd_category_id }
        self.class.log_debug(" inv=#{inv}")
        self.class.log_debug(" category_ids=#{category_ids.inspect}")
    
        system_category_ids = SidfmdCategory.where(:is_system => true).where(:id => category_ids).pluck(:id)
        custom_category_ids = SidfmdCategory.where(:is_system => false).where(:id => category_ids).pluck(:id)
        is_system_category_ids = system_category_ids.any?       
        is_custom_category_ids = custom_category_ids.any?
        corresp_required = Sidfm.corresp_required
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            flg = !inv
            if is_system_category_ids and is_custom_category_ids then
              sys_flg = false
              if x[:system_category] && x[:system_category][:id] && system_category_ids.include?(x[:system_category][:id]) then
                sys_flg = true
              end
              cus_flg = false
              if x[:no_system_categories] then
                custom_cat = x[:no_system_categories].find {|category|
                  category[:id] && custom_category_ids.include?(category[:id])
                }
                if custom_cat then
                  cus_flg = true
                end
              end
              flg = (sys_flg and cus_flg) ? inv : !inv
            elsif is_system_category_ids then
              if x[:system_category] && x[:system_category][:id] && system_category_ids.include?(x[:system_category][:id]) then
                flg = inv
              end
            elsif is_custom_category_ids then
              if x[:no_system_categories] then
                custom_cat = x[:no_system_categories].find {|category|
                  category[:id] && custom_category_ids.include?(category[:id])
                }
                if custom_cat then
                  flg = inv
                end
              end
            end
            flg
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end

        target_host_ids = if is_system_category_ids && is_custom_category_ids then
                            SidfmdCategoriesHost.where(
                              :sidfmd_host_id => SidfmdCategoriesHost.where(:sidfmd_category_id => system_category_ids).select(:sidfmd_host_id)
                            ).where(
                              :sidfmd_host_id => SidfmdCategoriesHost.where(:sidfmd_category_id => custom_category_ids).select(:sidfmd_host_id)
                            ).distinct.pluck(:sidfmd_host_id)
                          elsif is_system_category_ids then
                            SidfmdCategoriesHost.where(:sidfmd_category_id => system_category_ids).distinct.pluck(:sidfmd_host_id)
                          else
                            SidfmdCategoriesHost.where(:sidfmd_category_id => custom_category_ids).distinct.pluck(:sidfmd_host_id)
                          end
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            x[:sri_cvss_status_ids].delete_if {|y|
              inv ? target_host_ids.include?(y[:sidfmd_host_id]) : !target_host_ids.include?(y[:sidfmd_host_id])
            }
            if x[:sri_cvss_status_ids].size > 0 then
              SidfmdIssue.update_value_by_scsids(x, corresp_required)
              SIDfm::VMRule::Cond.update_hosts_cats(x)
              ret = false
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['category-' + (inv ? 'unmatch' : 'match')]} 処理終了:#{ret}")
        ret
      end

      def cond_status?(cond)
        self.class.log_debug("cond_status?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['status-match']} 処理開始")
        cnt = 0
        detail = cond.detail
        status_id1 = detail.status_id1
        status_id2 = detail.status_id2
        status_ids = []
        if status_id1 then
          status_ids << status_id1
        end
        if status_id2 then
          status_ids << status_id2
        end
        status_ids.uniq!
        status_ids.compact!
        corresp_required = Sidfm.corresp_required
        self.class.log_debug(" status_ids = #{status_ids.inspect}")
        if status_ids.size > 0 then
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before") 
            content.delete_if {|x|
              ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(x[:sidfmd_host_id]) then
                ret = !status_ids.include?(x[:min_status_id])
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
         end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                !status_ids.include?(y[:status_id])
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                ret = false
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
        else
          cond.log_warn("ステータスが指定されていないため、条件は無視されました。")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['status-match']} 処理終了:#{ret}")
        ret
      end

      def cond_issue_update?(cond)
        self.class.log_debug("cond_issue_update?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['issue-update']} 処理開始")
        ret = false
        if @args[:trigger] && @args[:trigger] == :vulsync then
          detail = cond.detail
          cond_or = detail[:cond_or]
          cnt_create = detail[:cnt_create]
          cnt_remove = detail[:cnt_remove]
          self.class.log_debug(" cond_or=#{cond_or}")
          self.class.log_debug(" cnt_create=#{cnt_create}")
          self.class.log_debug(" cnt_remove=#{cnt_remove}")

          # 条件チェックにはフィルタ前のコンテンツを参照
          content_add_size = @args[:content_add].size
          #content_modify_size = @args[:content_modify].size
          content_remove_size = @args[:content_remove].size

          if (cond_or and (content_add_size >= cnt_create or
                           content_remove_size > cnt_remove)) or
              (!cond_or and (content_add_size >= cnt_create and
                             content_remove_size >= cnt_remove)) then
            # 条件一致時
            # なにもしない
            self.class.log_debug(" cond match: no-op")
            ret = true
          else
            # 条件不一致時
            # vulsyncのコンテンツを全て削除して0件にする
            # vulsyncに該当する脆弱性以外を削除
            self.class.log_debug(" cond dismatch: clear")
            @vulsync_content_keys.each do |key|
              content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
              self.class.log_debug("#{key.to_s} #{content.size} - before")
              content.clear
              content_size = content.size
              self.class.log_debug("#{key.to_s} #{content_size} - after")
            end
          end
        else
          # trigger!=vulsyncの時は無視
          self.class.log_debug(" issue_update ignored(vulsync only)")
          cond.log_info("タイミングが#{SidfmdRule::RULE_TRIGGER_NAMES['vulsync']}ではないため、条件は無視されました。")
          # 不一致ではなく無視なので、条件成立扱い
          ret = true
        end
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['issue-update']} 処理終了:#{ret}")
        ret
      end

      def cond_sri_above?(cond)
        self.class.log_debug("cond_sri_above?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['sri-above']} 処理開始")
        cnt = 0
        detail = cond.detail
        risk_id = detail.risk_id
        corresp_required = Sidfm.corresp_required
        self.class.log_debug(" risk_id = #{risk_id}")
        if SidfmdIssueSri::VALUES.include?(risk_id) then
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(x[:sidfmd_host_id]) then
                ret = !(x[:max_sri] and x[:max_sri] >= risk_id)
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                y[:sri] < risk_id
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                ret = false
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
        else
          cond.log_warn("不正なSRI値が指定されているため、条件は無視されました。")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['sri-above']} 処理終了:#{ret}")
        ret
      end

      def cond_sri_below?(cond)
        self.class.log_debug("cond_sri_below?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['sri-below']} 処理開始")
        cnt = 0
        detail = cond.detail
        risk_id = detail.risk_id
        corresp_required = Sidfm.corresp_required
        if SidfmdIssueSri::VALUES.include?(risk_id) then
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(x[:sidfmd_host_id]) then
                ret = !(x[:max_sri] and x[:max_sri] <= risk_id)
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                y[:sri] > risk_id
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                ret = false
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
        else
          cond.log_warn("不正なSRI値が指定されているため、条件は無視されました。")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['sri-below']} 処理終了:#{ret}")
        ret
      end

      def cond_cvss_above?(cond)
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-above']} 処理開始")
        cnt = 0
        detail = cond.detail
        cvss_value = detail.cvss_value
        is_notreg = detail.is_notreg
        exploitable = detail.exploitable
        corresp_required = Sidfm.corresp_required
        if cvss_value and cvss_value >= 0.0 and cvss_value <= 10.0 then
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(x[:sidfmd_host_id]) then
                if exploitable.nil? or
                    (exploitable and x[:exploitable]) or
                    (!exploitable and !x[:exploitable]) then
                  ret = !((is_notreg and x[:max_cvss][:label] == :notreg) or
                          (x[:max_cvss][:label] != :notreg and
                           x[:max_cvss][:score] >= cvss_value))
                end
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                if y[:cvss].nil? then
                  if is_notreg then
                    false
                  else
                    true
                  end
                else
                  y[:cvss] < cvss_value
                end
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                if !exploitable.nil? then
                  if (exploitable and x[:exploitable]) or
                      (!exploitable and !x[:exploitable]) then
                    ret = false
                  end
                else
                  ret = false
                end
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
           end
         else
          cond.log_warn("不正なCVSS値が指定されているため、条件は無視されました。")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-above']} 処理終了:#{ret}")
        ret
      end

      def cond_cvss_below?(cond)
        self.class.log_debug("cond_sri_below?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-below']} 処理開始")
        cnt = 0
        detail = cond.detail
        cvss_value = detail.cvss_value
        is_notreg = detail.is_notreg
        exploitable = detail.exploitable
        corresp_required = Sidfm.corresp_required
        if cvss_value and cvss_value >= 0.0 and cvss_value <= 10.0 then
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(x[:sidfmd_host_id]) then
                if exploitable.nil? or
                    (exploitable and x[:exploitable]) or
                    (!exploitable and !x[:exploitable]) then
                  ret = !((is_notreg and x[:max_cvss][:label] == :notreg) or
                          (x[:max_cvss][:label] != :notreg and
                           x[:max_cvss][:score] <= cvss_value))
                end
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                if y[:cvss].nil? then
                  if is_notreg then
                    false
                  else
                    true
                  end
                else
                  y[:cvss] > cvss_value
                end
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                if !exploitable.nil? then
                  if (exploitable and x[:exploitable]) or
                      (!exploitable and !x[:exploitable]) then
                    ret = false
                  end
                else
                  ret = false
                end
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
        else
          cond.log_warn("不正なCVSS値が指定されているため、条件は無視されました。")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-below']} 処理終了:#{ret}")
        ret
      end

      def cond_cvss_notreg?(cond)
        self.class.log_debug("cond_cvss_notreg?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-notreg']} 処理開始")
        cnt = 0
        detail = cond.detail
        is_notreg = detail.is_notreg
        corresp_required = Sidfm.corresp_required
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                @args[:host_ids].include?(x[:sidfmd_host_id]) then
              ret = !((is_notreg and x[:max_cvss][:label] == :notreg) or
                      (!is_notreg and x[:max_cvss][:label] != :notreg))
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            x[:sri_cvss_status_ids].delete_if {|y|
              ((is_notreg and !y[:cvss].nil?) or
               (!is_notreg and y[:cvss].nil?))
            }
            if x[:sri_cvss_status_ids].size > 0 then
              SidfmdIssue.update_value_by_scsids(x, corresp_required)
              SIDfm::VMRule::Cond.update_hosts_cats(x)
              ret = false
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-notreg']} 処理終了:#{ret}")
        ret
      end

      def cond_warning?(cond)
        self.class.log_debug("cond_warning?")
        cnt = 0
        detail = cond.detail
        is_warning = detail.is_warning
        is_cvss = detail.is_cvss
        corresp_required = Sidfm.corresp_required
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES[(is_cvss ? 'cvss' : 'sri') + '-warning']} 処理開始")
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                @args[:host_ids].include?(x[:sidfmd_host_id]) then
              ret = !(x[:warning][is_cvss ? :cvss : :sri] == is_warning)
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          if is_cvss then
            # CVSS要注意
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                is_warning ? (y[:cvss] and y[:cvss] < corresp_required[:cvss]) : (y[:cvss].nil? or y[:cvss] >= corresp_required[:cvss])
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                ret = false
              end
              ret
            }
          else
            # SRI要注意
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                is_warning ? y[:sri] < corresp_required[:sri] : y[:sri] >= corresp_required[:sri]
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                ret = false
              end
              ret
            }
          end
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES[(is_cvss ? 'cvss' : 'sri') + '-warning']} 処理終了:#{ret}")
        ret
      end

      def cond_history_post?(cond)
        self.class.log_debug("cond_history_post?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['history-create']} 処理開始")
        cnt = 0
        detail = cond.detail
        limit_cnt = detail.cnt
        corresp_required = Sidfm.corresp_required
        hp_users = detail.sidfmd_rule_chp_user_subs
        users_selected = hp_users.exists? 
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          h_host_ids = nil
          if @args[:host_ids] then
            h_host_ids = content.collect {|x| x[:sidfmd_host_id] }.uniq.compact
            h_host_ids &= @args[:host_ids]
          end

          #journal_details =
          #  JournalDetail
          #  .where(JournalDetail.arel_table[:journal_id].eq(Journal.arel_table[:id]))
          #  .select(Arel.star.count)
          #  .as("detail_cnt")

          journals = 
            Journal
            .select(:id)
            .select(:journalized_id)
            #.select(journal_details.to_sql)

          if users_selected then
            journals = if detail.flg_exclusion then
                         journals.where.not(:user_id => hp_users.select(:user_id))
                       else
                         journals.where(:user_id => hp_users.select(:user_id))
                       end
          end

          if cond.last_checked_at then
            journals = journals
              .where("journals.created_on >= ?", cond.last_checked_at)
          else
            journals = journals
              .where("journals.created_on >= current_timestamp - '#{SIDfm::VMRule::FIRST_LIMIT_DAYS} days'::interval")
          end

          hosts_cnt = 
            SidfmdHost
            .joins(:sidfmd_issues)
            .joins(:sidfmd_issues => :issue)
            .joins("
             LEFT OUTER JOIN (#{journals.to_sql}) AS journal_cnt
              ON issues.id=journal_cnt.journalized_id
            ")
            .select(:id)
            .select("COUNT(journal_cnt.id) AS journal_size")
            .group(:id)
=begin
          hosts_cnt = 
            SidfmdHost
            .joins(:sidfmd_issues)
            .joins(:sidfmd_issues => :issue)
            .joins("
             LEFT OUTER JOIN (#{journals.to_sql}) AS journal_cnt
              ON issues.id=journal_cnt.journalized_id AND journal_cnt.detail_cnt=0
            ")
            .select(:id)
            .select("COUNT(journal_cnt.id) AS journal_size")
            .group(:id)
=end

          if h_host_ids then
            hosts_cnt = hosts_cnt.where(:id => h_host_ids)
          end

          hosts =
            SidfmdHost
            .joins("
             INNER JOIN (#{hosts_cnt.to_sql}) AS hosts_cnt
              ON hosts_cnt.id=sidfmd_hosts.id
            ")
            .where("hosts_cnt.journal_size >= ?", limit_cnt)
            .select(:id)

          if h_host_ids then
            hosts = hosts.where(:id => h_host_ids)
          end

          limit_host_ids = hosts.collect {|x| x[:id]}.compact.uniq

          content.delete_if {|x|
            !limit_host_ids.include?(x[:sidfmd_host_id])
          }

          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content.size} - after")
        end
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          #detail_cnt =
          #  JournalDetail
          #  .where("journal_details.journal_id=journals.id")
          #  .select("COUNT(*)")
          c_journals =
            Journal
            .joins(:issue)
            .joins(:issue => :sidfmd_issue)
            .where(:sidfmd_issues => {
                     :html_id => content.collect {|x| x[:html_id] }.uniq.compact
                   })
            .select(:html_id)
            .select(:issue_id)
            .select("COUNT(issue_id) AS cnt")
            .group(:html_id, :issue_id)
            #.select("(#{detail_cnt.to_sql}) AS detail_cnt")
            #.group(:html_id, :issue_id, :id)
          if cond.last_checked_at then
            c_journals = c_journals
              .where("journals.created_on >= ?", cond.last_checked_at)
          else
            c_journals = c_journals
              .where("journals.created_on >= current_timestamp - '#{SIDfm::VMRule::FIRST_LIMIT_DAYS} days'::interval ")
          end
          if users_selected then
            c_journals = if detail.flg_exclusion then
                           c_journals.where.not(:user_id => hp_users.select(:user_id))
                         else
                           c_journals.where(:user_id => hp_users.select(:user_id))
                         end
          end
          journals =
            Journal
            .joins(:issue)
            .joins(:issue => :sidfmd_issue)
            .joins("INNER JOIN (#{c_journals.to_sql}) AS c_journals ON issues_journals.id=c_journals.issue_id")
            .select("sidfmd_issues_issues.html_id")
            .select("sidfmd_issues_issues.sidfmd_host_id")
          if @args[:host_ids] then
            journals = journals
              .where(:sidfmd_issues_issues => { :sidfmd_host_id => @args[:host_ids] })
          end
          journals = journals
            .where("c_journals.cnt >= #{limit_cnt}")

=begin
          journals = journals
            .where("c_journals.cnt >= #{limit_cnt} AND c_journals.detail_cnt=0")
=end

          hostid_htmlid = {}
          journals.each do |journal|
            if hostid_htmlid.has_key?(journal[:html_id]) then
              hostid_htmlid[journal[:html_id]] |= [ journal[:sidfmd_host_id] ]
            else
              hostid_htmlid[journal[:html_id]] = [ journal[:sidfmd_host_id] ]
            end
          end
          content.delete_if {|x|
            ret = true
            if hostid_htmlid[x[:html_id]] then
              x[:sri_cvss_status_ids].delete_if {|y|
                !hostid_htmlid[x[:html_id]].include?(y[:sidfmd_host_id])
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                ret = false
              end
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['history-create']} 処理終了:#{ret}")
        ret
      end

      def cond_duedate_expired?(cond)
        self.class.log_debug("cond_duedate_expiored?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['due-date-expired']} 処理開始")
        corresp_required = Sidfm.corresp_required
        cnt = 0
        # ホスト一覧
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                (@args[:host_ids].include?(x[:sidfmd_host_id])) then
              due_dates = x[:due_dates_unfinished].compact.sort.uniq
              if due_dates.size > 0 then
                due_dates.each do |due_date|
                  begin
                    parsed_due_date = Time.parse("#{due_date} 23:59:59+09:00")
                    if Time.current >= parsed_due_date then
                      ret = false
                    end
                  rescue Exception
                    # logging
                  end
                end
              end
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        # 脆弱性一覧
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            x[:sri_cvss_status_ids].delete_if {|y|
              flg = true
              if y[:due_date] and SidfmdIssue::STATUS_UNFINISHED.include?(y[:status_id]) then
                begin
                  dd = Time.parse("#{y[:due_date]} 23:59:59+09:00")
                  if Time.current >= dd then
                    flg = false
                  end
                rescue Exception
                  # invalid time format
                end
              end
              flg
            }
            if x[:sri_cvss_status_ids].size > 0 then
              SidfmdIssue.update_value_by_scsids(x, corresp_required)
              SIDfm::VMRule::Cond.update_hosts_cats(x)
              ret = false
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['due-date-expired']} 処理終了:#{ret}")
        ret
      end

      def cond_duedate_approaching?(cond)
        self.class.log_debug("cond_duedate_approaching?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['due-date-approach']} 処理開始")
        cnt = 0
        detail = cond.detail
        days = detail[:days] || 0
        corresp_required = Sidfm.corresp_required
        # ホスト一覧
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                (@args[:host_ids].include?(x[:sidfmd_host_id])) then
              due_dates = x[:due_dates_unfinished].compact.sort.uniq
              if due_dates.size > 0 then
                due_dates.each do |due_date|
                  begin
                    parsed_due_date = Time.parse("#{due_date} 23:59:59+09:00")
                    approaching_due_date = parsed_due_date.ago(days.days)
                    time_current = Time.current
                    if time_current <= parsed_due_date && time_current >= approaching_due_date then
                      ret = false
                      break
                    end
                  rescue Exception => e
                    # logging
                    self.class.log_debug(e)
                  end
                end
              end
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        # 脆弱性一覧
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            x[:sri_cvss_status_ids].delete_if {|y|
              flg = true
              if y[:due_date] and SidfmdIssue::STATUS_UNFINISHED.include?(y[:status_id]) then
                begin
                  dd = Time.parse("#{y[:due_date]} 23:59:59+09:00")
                  a_dd = dd.ago(days.days)
                  time_current = Time.current
                  if time_current <= dd && time_current >= a_dd then
                    flg = false
                  end
                rescue Exception
                  # invalid time format
                end
              end
              flg
            }
            if x[:sri_cvss_status_ids].size > 0 then
              SidfmdIssue.update_value_by_scsids(x, corresp_required)
              SIDfm::VMRule::Cond.update_hosts_cats(x)
              ret = false
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
          self.class.log_debug(" #{content.collect {|x| x[:due_dates_unfinished].uniq.compact }.uniq.compact}")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['due-date-approach']} 処理終了:#{ret}")
        ret
      end

      def cond_duedate_undef?(cond)
        self.class.log_debug("cond_duedate_undef?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['due-date-undef']} 処理開始")
        cnt = 0
        corresp_required = Sidfm.corresp_required
        # ホスト一覧
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                (@args[:host_ids].include?(x[:sidfmd_host_id])) then
              ret = !x[:due_dates_unfinished].include?(nil)
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        # 脆弱性一覧
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            x[:sri_cvss_status_ids].delete_if {|y|
              flg = true
              if SidfmdIssue::STATUS_UNFINISHED.include?(y[:status_id]) then
                flg = !!y[:due_date] 
              end
              flg
            }
            if x[:sri_cvss_status_ids].size > 0 then
              SidfmdIssue.update_value_by_scsids(x, corresp_required)
              SIDfm::VMRule::Cond.update_hosts_cats(x)
              ret = false
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['due-date-undef']} 処理終了:#{ret}")
        ret
      end
      
      def cond_cvss_exploitable?(cond)
        self.class.log_debug("cond_cvss_exploitable?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-exploitable']} 処理開始")
        cnt = 0
        detail = cond.detail
        value = detail.value
        is_change = detail.is_change
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                @args[:host_ids].include?(x[:sidfmd_host_id]) then
              # hosts_content is is_change unsupported
              ret = !((value and x[:exploitable]) or (!value and !x[:exploitable]))
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            content_hosts = x[:hosts].collect {|y| y[:id] }
            content_hosts.compact!
            if @args[:host_ids].nil? or
                (@args[:host_ids] & content_hosts).size > 0 then
              ret = !((value and x[:exploitable]) or (!value and !x[:exploitable]))
              if !ret and is_change then
                prev_row = @prev_vuls_content.find {|y| y[:html_id] == y[:html_id] }
                if prev_row then
                  prev_ret = !((value and prev_row[:exploitable]) or (!value and !prev_row[:exploitable]))
                  if prev_ret == ret then
                    # not changed
                    ret = true
                  end
                else
                  # prev_row = nil -> may be changed(add?)
                end
              end
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-exploitable']} 処理終了:#{ret}")
        ret
      end

      def cond_cvss_vector?(cond)
        self.class.log_debug("cond_cvss_vector?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-vector']} 処理開始")
        # 完全一致のフラグ。デフォルトは部分一致。
        # 部分一致：いずれかのcveが指定した条件にマッチした場合条件成立。未定義があっても成立。
        # 完全一致：全てのcveが指定した条件にマッチした時のみ条件成立。未定義があると成立しない。
        flg_perfect = cond.detail.flg_perfect
        # 未定義を無視するフラグ。デフォルトは無視しない。
        # trueにすると完全一致の時に未定義があっても成立するようになる。
        # flg_perfect=trueの場合のみ動作に影響がある。
        flg_ignore_undef = cond.detail.flg_ignore_undef
        self.class.log_debug(" flg_perfect: #{flg_perfect}")
        self.class.log_debug(" flg_ignore_undef: #{flg_ignore_undef}")
        cond.log_info("条件オプション：#{flg_perfect ? '完全一致' : '部分一致'}")
        cond.log_info("条件オプション：完全一致時に指標未定義を無視#{flg_ignore_undef ? 'する' : 'しない'}")
        cnt = 0
        detail = cond.detail
        pat_regexp = detail.generate_regexp
        # vuls
        hostids = []
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            content_hosts = x[:hosts].collect {|y| y[:id] }
            content_hosts.compact!
            if @args[:host_ids].nil? or
                (@args[:host_ids] & content_hosts).size > 0 then
              if pat_regexp then
                if x[:cves] and x[:cves].size > 0 then
                  if flg_perfect then
                    flag = true
                    match_flg = false
                    x[:cves].each {|y|
                      if !flg_ignore_undef and (y[:c3_vector].nil? or y[:c3_vector].size == 0) then
                        #self.class.log_debug("!!!!rule_id=#{@rule.id},html_id=#{x[:html_id]},cve_id=#{y[:cve_id]},ignore undef,#{y.inspect}")
                        flag = false
                        break
                      end
                      if y[:c3_vector] and y[:c3_vector].size > 0 and pat_regexp !~ y[:c3_vector] then
                        #self.class.log_debug("!!!!rule_id=#{@rule.id},html_id=#{x[:html_id]},cve_id=#{y[:cve_id]},no match vector=#{y[:c3_vector]}")
                        flag = false
                        break
                      end
                      if y[:c3_vector] and y[:c3_vector].size > 0 and pat_regexp =~ y[:c3_vector] then
                        match_flg = true
                      end
                    }
                    ret = false if (flag and match_flg)
                  else
                    flag = false
                    x[:cves].each {|y|
                      if y[:c3_vector] and pat_regexp =~ y[:c3_vector] then
                        flag = true
                        break
                      end
                    }
                    ret = false if flag
                  end
                end
              end
            end
            unless ret then
              hostids |= (x[:hosts] ? x[:hosts].collect {|y| y[:id] } : [])
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        hostids.uniq!
        hostids.compact!
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                @args[:host_ids].include?(x[:sidfmd_host_id]) then
              ret = !hostids.include?(x[:sidfmd_host_id])
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['cvss-vector']} 処理終了:#{ret}")
        ret
      end

      # 同期中ホスト
      def cond_host_syncing?(cond)
        self.class.log_debug("cond_host_syncing?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['host-syncing']} 処理開始")
        cnt = 0
        detail = cond.detail
        inv = (detail.inv || false)
        corresp_required = Sidfm.corresp_required
        hostid2sync = {}
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                @args[:host_ids].include?(x[:sidfmd_host_id]) then
              if !hostid2sync.has_key?(x[:sidfmd_host_id]) then
                host = SidfmdHost.enable_hosts.where(:id => x[:sidfmd_host_id]).first
                if host then
                  hostid2sync[x[:sidfmd_host_id]] = host.is_syncing
                else
                  hostid2sync[x[:sidfmd_host_id]] = false
                end
              end
              ret = inv ? hostid2sync[x[:sidfmd_host_id]] : !hostid2sync[x[:sidfmd_host_id]]
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            x[:sri_cvss_status_ids].delete_if {|y|
              scsi_ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(y[:sidfmd_host_id]) then
                if !hostid2sync.has_key?(y[:sidfmd_host_id]) then
                  host = SidfmdHost.enable_hosts.where(:id => y[:sidfmd_host_id]).first
                  if host then
                    hostid2sync[y[:sidfmd_host_id]] = host.is_syncing
                  else
                    hostid2sync[y[:sidfmd_host_id]] = false
                  end
                end
                scsi_ret = inv ? hostid2sync[y[:sidfmd_host_id]] : !hostid2sync[y[:sidfmd_host_id]]
              end
              scsi_ret
            }
            if x[:sri_cvss_status_ids].size > 0 then
              SidfmdIssue.update_value_by_scsids(x, corresp_required)
              SIDfm::VMRule::Cond.update_hosts_cats(x)
              ret = false
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['host-syncing']} 処理終了:#{ret}")
        ret
      end

      # 期間指定
      def cond_timerange?(cond)
        self.class.log_debug("cond_timerange?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['timerange']} 処理開始")
        detail = cond.detail
        corresp_required = Sidfm.corresp_required
        unit = detail.unit
        value = detail.value
        target = detail.target
        unless SidfmdRuleCTimerange::TARGET_VALUES.include?(target) then
          cond.log_error("不正なオプション：target=#{target.inspect}")
          return false
        end
        cnt = 0
        if [ 'hour', 'day', 'week', 'month' ].include?(unit) and
            (('hour' == unit and value and value >= 1 and value <= 24) or
             ([ 'day', 'week', 'month' ].include?(unit) and value and value >= 1)
             ) then
          issues = SidfmdIssue.joins(:issue)
          if @args[:host_ids] then
            issues = issues.where(:sidfmd_host_id => @args[:host_ids])
          end
          before_time = case unit
                        when 'hour'  then Time.current.ago(value.hours)
                        when 'day'   then Time.current.ago(value.days)
                        when 'week'  then Time.current.ago(value.weeks)
                        when 'month' then Time.current.ago(value.months)
                        else Time.current
                        end
          case target
          when SidfmdRuleCTimerange::TARGET_DETECTED_DATE then
            issues = (
              detail.inv ?
              issues.where.not("issues.updated_on >= ?", before_time) :
              issues.where("issues.updated_on >= ?", before_time)
            )
          when SidfmdRuleCTimerange::TRAGET_CREATED_DATE then #日付があるもののみが対象
            htmlid_date = (
              detail.inv ?
              SidfmdHtmlidDate.where.not("create_datetime >= ?", before_time) :
              SidfmdHtmlidDate.where("create_datetime >= ?", before_time)
            ).where.not(:create_datetime => nil) #(nullはないと思いますが)nullのものは含めない
            issues = issues.where(:sidfmd_issues => { :html_id => htmlid_date.select(:htmlid) })
          when SidfmdRuleCTimerange::TRAGET_UPDATED_DATE then #日付があるもののみが対象
            htmlid_date = (
              detail.inv ?
              SidfmdHtmlidDate.where.not("update_datetime >= ?", before_time) :
              SidfmdHtmlidDate.where("update_datetime >= ?", before_time)
            ).where.not(:update_datetime => nil) #(nullはないと思いますが)nullのものは含めない
            issues = issues.where(:sidfmd_issues => { :html_id => htmlid_date.select(:htmlid) })
          end
          issues = issues.select(:sidfmd_host_id).select(:html_id)
          target_htmlhostids = []
          issues.each do |issue|
            target_htmlhostids << {
              :sidfmd_host_id => issue[:sidfmd_host_id],
              :html_id => issue[:html_id],
            }
          end
          vul_host_ids = []
          (@vuls_content_keys + @vulsync_content_keys).each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              x[:sri_cvss_status_ids].delete_if {|y|
                scsi_ret = true
                if @args[:host_ids].nil? or
                    @args[:host_ids].include?(y[:sidfmd_host_id]) then
                  scsi_ret = target_htmlhostids.find {|z|
                    z[:html_id] == x[:html_id] && z[:sidfmd_host_id] == y[:sidfmd_host_id]
                  }.nil?
                end
                scsi_ret
              }
              if x[:sri_cvss_status_ids].size > 0 then
                SidfmdIssue.update_value_by_scsids(x, corresp_required)
                SIDfm::VMRule::Cond.update_hosts_cats(x)
                vul_host_ids += x[:sri_cvss_status_ids].collect {|y| y[:sidfmd_host_id]}.uniq.compact
                ret = false
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
          vul_host_ids.uniq!
          vul_host_ids.compact!
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            self.class.log_debug("#{key.to_s} #{content.size} - before")
            content.delete_if {|x|
              ret = true
              if @args[:host_ids].nil? or
                  @args[:host_ids].include?(x[:sidfmd_host_id]) then
                ret = target_htmlhostids.find {|y| y[:sidfmd_host_id] == x[:sidfmd_host_id] }.nil?
                unless ret then
                  ret = !vul_host_ids.include?(x[:sidfmd_host_id])
                end
              end
              ret
            }
            content_size = content.size
            cnt += content_size
            self.class.log_debug("#{key.to_s} #{content_size} - after")
          end
        else
          cond.log_error("不正なオプション：unit=#{unit.inspect}, value=#{value.inspect}")
        end
        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['timerange']} 処理終了:#{ret}")
        ret
      end

      # カスタムタグ
      def cond_ctag?(cond)
        self.class.log_debug("cond_ctag?")
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['ctag']} 処理開始")
        cnt = 0
        detail = cond.detail
        enable_ctag_ids = [] #あれば成立
        not_enable_ctag_ids = [] #なければ成立
        detail.sidfmd_rule_cm_ctags.each do |cm_ctag|
          if cm_ctag.is_enable then
            enable_ctag_ids << cm_ctag.sidfmd_ctag_id
          else
            not_enable_ctag_ids << cm_ctag.sidfmd_ctag_id
          end
        end
        enable_ctag_ids.uniq!
        not_enable_ctag_ids.uniq!
        corresp_required = Sidfm.corresp_required
        self.class.log_debug(" enable_ctag_ids = #{enable_ctag_ids.inspect}")
        self.class.log_debug(" not_enable_ctag_ids = #{not_enable_ctag_ids.inspect}")
        @hosts_content_keys.each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before") 
          content.delete_if {|x|
            ret = true
            if @args[:host_ids].nil? or
                @args[:host_ids].include?(x[:sidfmd_host_id]) then
              if enable_ctag_ids.size == 0 and not_enable_ctag_ids.size == 0 then
                if x[:ctags].size == 0 then
                  #「カスタムタグが1つもない」に一致
                  ret = false
                end
              else
                target_ctag_ids = x[:ctags].map {|y| y[:ctag_id]}
                if enable_ctag_ids.size > 0 and not_enable_ctag_ids.size > 0 then
                  e_ctags = (enable_ctag_ids & target_ctag_ids)
                  n_ctags = (not_enable_ctag_ids & target_ctag_ids)
                  if e_ctags.size > 0 and n_ctags.size == 0 then
                    ret = false
                  end
                elsif enable_ctag_ids.size > 0 then
                  if (enable_ctag_ids & target_ctag_ids).size > 0 then
                    # あれば成立
                    ret = false
                  end
                elsif not_enable_ctag_ids.size > 0 then
                  if (not_enable_ctag_ids & target_ctag_ids).size == 0 then
                    # なければ成立
                    ret = false
                  end
                else
                  # blackhole
                end
              end
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end
        (@vuls_content_keys + @vulsync_content_keys).each do |key|
          content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
          self.class.log_debug("#{key.to_s} #{content.size} - before")
          content.delete_if {|x|
            ret = true
            t_sidfmd_host_ids = []
            ctags_sidfmd_host_ids = x[:ctags].collect {|y| y[:sidfmd_host_id] }.uniq
            scsids_sidfmd_host_ids = x[:sri_cvss_status_ids].collect {|y| y[:sidfmd_host_id] }.uniq
            none_ctag_host_ids = scsids_sidfmd_host_ids - ctags_sidfmd_host_ids
            if enable_ctag_ids.size == 0 and not_enable_ctag_ids.size == 0 then
              if none_ctag_host_ids.size > 0 then
                t_sidfmd_host_ids = (t_sidfmd_host_ids | none_ctag_host_ids)
              end
            else
              ctags_sidfmd_host_ids.each do |ctags_sidfmd_host_id|
                target_ctag_ids = x[:ctags].collect {|y| y[:sidfmd_host_id] == ctags_sidfmd_host_id ? y[:ctag_id] : nil }.compact.uniq
                if enable_ctag_ids.size > 0 and not_enable_ctag_ids.size > 0 then
                  e_ctags = (enable_ctag_ids & target_ctag_ids)
                  n_ctags = (not_enable_ctag_ids & target_ctag_ids)
                  if e_ctags.size > 0 and n_ctags.size == 0 then
                    t_sidfmd_host_ids << ctags_sidfmd_host_id
                  end
                elsif enable_ctag_ids.size > 0 then
                  if (enable_ctag_ids & target_ctag_ids).size > 0 then
                    t_sidfmd_host_ids << ctags_sidfmd_host_id
                  end
                elsif not_enable_ctag_ids.size > 0 then
                  if (not_enable_ctag_ids & target_ctag_ids).size == 0 then
                    t_sidfmd_host_ids << ctags_sidfmd_host_id
                  elsif none_ctag_host_ids.size > 0 then
                    t_sidfmd_host_ids = (t_sidfmd_host_ids | none_ctag_host_ids)
                  end
                else
                  # blackhole
                end
              end
            end

            x[:sri_cvss_status_ids].delete_if {|y|
              !t_sidfmd_host_ids.include?(y[:sidfmd_host_id])
            }
            x[:ctags].delete_if {|y|
              !t_sidfmd_host_ids.include?(y[:sidfmd_host_id])
            }
            if x[:sri_cvss_status_ids].size > 0 then
              SidfmdIssue.update_value_by_scsids(x, corresp_required)
              SIDfm::VMRule::Cond.update_hosts_cats(x)
              ret = false
            end
            ret
          }
          content_size = content.size
          cnt += content_size
          self.class.log_debug("#{key.to_s} #{content_size} - after")
        end

        ret = cnt > 0
        cond.log_info("条件:#{SidfmdRule::RULE_COND_NAMES['ctag']} 処理終了:#{ret}")
        ret
      end

      public

      def check?
        ret = false
        conds = @rule.sidfmd_rule_conds
        conds.each do |cond|
=begin
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            if content then
              cond.log_info("before: #{key.to_s} #{content.size} : #{content.collect {|x| x[:hostname] }.join(',')}")
            else
              cond.log_info("before: #{key.to_s} nil")
            end
          end
=end
          is_cond = false
          begin
            is_cond =
              case cond.detail
              when SidfmdRuleCMatchHost          then cond_match_host?(cond)
              when SidfmdRuleCMatchCategory      then cond_match_category?(cond)
              when SidfmdRuleCMatchApp           then cond_match_app?(cond)
              when SidfmdRuleCIssueUpdate        then cond_issue_update?(cond)
              when SidfmdRuleCSriAbove           then cond_sri_above?(cond)
              when SidfmdRuleCSriBelow           then cond_sri_below?(cond)
              when SidfmdRuleCCvssAbove          then cond_cvss_above?(cond)
              when SidfmdRuleCCvssBelow          then cond_cvss_below?(cond)
              when SidfmdRuleCCvssNotreg         then cond_cvss_notreg?(cond)
              when SidfmdRuleCStatus             then cond_status?(cond)
              when SidfmdRuleCWarning            then cond_warning?(cond)
              when SidfmdRuleCHistoryPost        then cond_history_post?(cond)
              when SidfmdRuleCDuedateExpired     then cond_duedate_expired?(cond)
              when SidfmdRuleCDuedateApproaching then cond_duedate_approaching?(cond)
              when SidfmdRuleCDuedateUndef       then cond_duedate_undef?(cond) 
              when SidfmdRuleCCvssExploitable    then cond_cvss_exploitable?(cond)
              when SidfmdRuleCCvssVector         then cond_cvss_vector?(cond)
              when SidfmdRuleCHostSyncing        then cond_host_syncing?(cond)
              when SidfmdRuleCTimerange          then cond_timerange?(cond)
              when SidfmdRuleCCtag               then cond_ctag?(cond)
              else cond_invalid?(cond)
              end
          rescue Exception => e
            self.class.log_error(e)
          end
=begin
          @hosts_content_keys.each do |key|
            content = @args[(SIDfm::VMRule::P_FIL + key.to_s).to_sym]
            if content then
              cond.log_info("after: #{key.to_s} #{content.size} : #{content.collect {|x| x[:hostname] }.join(',')}")
            else
              cond.log_info("after: #{key.to_s} nil")
            end
          end
=end
          cond.update_attribute :last_checked_at, Time.current
          ret |= is_cond
        end
        ret
      end

    end

  end
end
